/************************************************************************//**
 *
 * @file    eepromc_reg.h
 *
 * @brief   Definition of the EEPROMC Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *
 ******************************************************************************/
#ifndef EEPROMC_REG_H__
#define EEPROMC_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// EEPROMC Control Register 0
typedef union EPRCCTL0_REG_TAG {
    unsigned short  reg;
    struct EPRCCTL0_REG_BIT_TAG {
        /**
         * This bit starts/stops reprogramming mode.
         * .EP_WMODE
         *  This bit starts/stops reprogramming mode.
         *    1 (R/W): Start reprogramming mode
         *    0 (R/W): Stop reprogramming mode
         */
        unsigned short      EP_WMODE    : 1;
        /**
         * This bit controls the programming power supply.
         * .EP_PWRSET
         *  This bit controls the programming power supply.
         *    1 (R/W): Programming power supply ON
         *    0 (R/W): Programming power supply OFF
         */
        unsigned short      EP_PWRSET   : 1;
        unsigned short      reserved_1  : 6;    ///< Reserved.
        /**
         * This bit controls the reset signal of the EEPROM.
         * .EP_XPOR
         *  This bit controls the reset signal of the EEPROM.
         *    1 (R/W): Negate the reset signal.
         *    0 (R/W): Assert the reset signal.
         */
        unsigned short      EP_XPOR     : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPRCCTL0_REG;

/// EEPROMC Control Register 1
typedef union EPRCCTL1_REG_TAG {
    unsigned short  reg;
    struct EPRCCTL1_REG_BIT_TAG {
        /**
         * This bit controls the clock pulse output to reprogram the EEPROM.
         * .EP_CK
         *  This bit controls the clock pulse output to reprogram the EEPROM.
         *    1 (W): Output one clock pulse
         *    0 (W): Ineffective
         */
        unsigned short      EP_CK       : 1;
        unsigned short      reserved_1  :15;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPRCCTL1_REG;

/// EEPROMC Address Register
typedef union EPRCADR_REG_TAG {
    unsigned short  reg;
    struct EPRCADR_REG_BIT_TAG {
        /**
         * These bits specify the EEPROM physical address (0 to 255) to be reprogrammed.
         * .EP_ADDR[7:0]
         *  These bits specify the EEPROM physical address (0 to 255) to be reprogrammed.
         *  EEPROM (logical) address = 0x6000 + EPRCADR.EP_ADDR[7:0] bits
         */
        unsigned short      EP_ADDR     : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPRCADR_REG;

/// EEPROMC Write Data Register
typedef union EPRCWDAT_REG_TAG {
    unsigned short  reg;
    struct EPRCWDAT_REG_BIT_TAG {
        /**
         * These bits specify the 8-bit data to program the EEPROM.
         * .EP_WDAT[7:0]
         *  These bits specify the 8-bit data to program the EEPROM.
         */
        unsigned short      EP_WDAT     : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPRCWDAT_REG;

/// EEPROMC Interrupt Flag Register
typedef union EPRCINTF_REG_TAG {
    unsigned short  reg;
    struct EPRCINTF_REG_BIT_TAG {
        /**
         * This bits indicate the Reprogram/read completion interrupt cause occurrence status.
         * .RXBIF
         *  This bits indicate the Reprogram/read completion interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      RXBIF       : 1;
        /**
         * This bits indicate the ECC interrupt cause occurrence status.
         * .ECCERIF
         *  This bits indicate the ECC interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      ECCERIF     : 1;
        unsigned short      reserved_1  :14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPRCINTF_REG;

/// EEPROMC Interrupt Enable Register
typedef union EPRCINTE_REG_TAG {
    unsigned short  reg;
    struct EPRCINTE_REG_BIT_TAG {
        /**
         * This bits enable Reprogram/read completion interrupt.
         * .RXBIE
         *  This bits enable Reprogram/read completion interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      RXBIE       : 1;
        /**
         * This bits enable ECC interrupt.
         * .ECCERIE
         *  This bits enable ECC interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      ECCERIE     : 1;
        unsigned short      reserved_1  :14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPRCINTE_REG;

#ifdef __cplusplus
}
#endif

#endif // EEPROMC_REG_H__
