/************************************************************************//**
 *
 * @file    smcif_reg.h
 *
 * @brief   Definition of the Smart Card Interface (SMCIF) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2017-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Oct. 2017    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef SMCIF_REG_H__
#define SMCIF_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// SMCIF Ch.N Clock Control Register
typedef union SMCNCLK_REG_TAG {
    unsigned short  reg;
    struct SMCNCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of SMCIF.
         * .CLKSRC[1:0]
         *  These bits select the clock source of SMCIF.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the SMCIF operating clock.
         * +---------------+-----------------------------------------+
         * |               |                CLKSRC[1:0]              |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * |  CLKDIV[1:0]  +---------+----------+---------+----------+
         * |               | IOSCCLK | OSC1CLK  | OSC3CLK | EXOSCCLK |
         * +---------------+---------+----------+---------+----------+
         * |      0x3      | 1/8     | 1/1      | 1/8     | 1/1      |
         * +---------------+---------+          +---------+          +
         * |      0x2      | 1/4     |          | 1/4     |          |
         * +---------------+---------+          +---------+          +
         * |      0x1      | 1/2     |          | 1/2     |          |
         * +---------------+---------+          +---------+          +
         * |      0x0      | 1/1     |          | 1/1     |          |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the SMCIF operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of SMCIF.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the SMCIF operating clock.
         * +---------------+-----------------------------------------+
         * |               |                CLKSRC[1:0]              |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * |  CLKDIV[1:0]  +---------+----------+---------+----------+
         * |               | IOSCCLK | OSC1CLK  | OSC3CLK | EXOSCCLK |
         * +---------------+---------+----------+---------+----------+
         * |      0x3      | 1/8     | 1/1      | 1/8     | 1/1      |
         * +---------------+---------+          +---------+          +
         * |      0x2      | 1/4     |          | 1/4     |          |
         * +---------------+---------+          +---------+          +
         * |      0x1      | 1/2     |          | 1/2     |          |
         * +---------------+---------+          +---------+          +
         * |      0x0      | 1/1     |          | 1/1     |          |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * This bit sets whether the SMCIF operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the SMCIF operating clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNCLK_REG;

/// SMCIF Ch.N Mode Register
typedef union SMCNMOD_REG_TAG {
    unsigned short  reg;
    struct SMCNMOD_REG_BIT_TAG {
        /**
         * This bit selects master or slave mode.
         * .MST
         *  This bit selects master or slave mode.
         *    1 (R/W): Master mode
         *    0 (R/W): Slave mode
         */
        unsigned short      MST         : 1;
        /**
         * This bit sets the SMCCLK0 pin level when the clock output is disabled.
         * .CLKPOL
         *  This bit sets the SMCCLKn pin level when the clock output is disabled. This bit is effective only in
         *  master mode.
         *    1 (R/W): High level
         *    0 (R/W): Low level
         */
        unsigned short      CLKPOL      : 1;
        /**
         * This bit sets the SMCIO0 pin output mode.
         * .OUTMD
         *  This bit sets the SMCIOn pin output mode.
         *    1 (R/W): Open drain output
         *    0 (R/W): Push-pull output
         */
        unsigned short      OUTMD       : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit sets the SMCIO0 pin data input/output direction.
         * .DIR
         *  This bit sets the SMCIOn pin data input/output direction.
         *    1 (R/W): MSB first
         *    0 (R/W): LSB first
         */
        unsigned short      DIR         : 1;
        /**
         * This bit sets the SMCIO0 pin data input/output level.
         * .INV
         *  This bit sets the SMCIOn pin data input/output level.
         *    1 (R/W): Inverted input/output
         *    0 (R/W): Non-inverted input/output
         */
        unsigned short      INV         : 1;
        /**
         * This bit selects the parity mode.
         * .PRY
         *  This bit selects the parity mode.
         *    1 (R/W): Odd parity mode
         *    0 (R/W): Even parity mode
         */
        unsigned short      PRY         : 1;
        /**
         * This bit selects the protocol mode.
         * .PRT
         *  This bit selects the protocol mode.
         *    1 (R/W): T = 1 mode
         *    0 (R/W): T = 0 mode
         */
        unsigned short      PRT         : 1;
        unsigned short      reserved_2  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNMOD_REG;

/// SMCIF Ch.N Baud Rate Register
typedef union SMCNBR_REG_TAG {
    unsigned short  reg;
    struct SMCNBR_REG_BIT_TAG {
        /**
         * These bits set the transfer rate for SMCIF Ch.n.
         * .FDR[10:0]
         *  These bits set the transfer rate for SMCIF Ch.n.
         * +----------+-----------------------------------------------------------------------+
         * |          |                                    D                                  |
         * | FDR[1:0] +-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |          |   1   |   2   |   4   |   8   |   16  |  32   |   64  |   12  |   20  |
         * +---+------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   |  372 | 0x173 | 0x0b9 | 0x05c | 0x02e |       |       |       | 0x01e |       |
         * +   +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   |  558 | 0x22d | 0x116 | 0x08b | 0x045 | 0x022 |       |       | 0x02e | 0x01b |
         * +   + -----+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   |  744 | 0x2e7 | 0x173 | 0x0b9 | 0x05c | 0x02e |       |       | 0x03d |       |
         * +   +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   | 1116 | 0x45b | 0x22d | 0x116 | 0x08b | 0x045 | 0x022 |       | 0x05c | 0x037 |
         * +   +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   | 1488 | 0x5cf | 0x2e7 | 0x173 | 0x0b9 | 0x05c | 0x02e |       | 0x07b | 0x04a |
         * + F +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   | 1860 | 0x743 | 0x3a1 | 0x1d0 | 0x0e8 | 0x074 | 0x03a |       | 0x09a | 0x05c |
         * +   +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   |  512 | 0x1ff | 0x0ff | 0x07f | 0x03f | 0x01f | 0x00f | 0x007 | 0x02a | 0x019 |
         * +   +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   |  768 | 0x2ff | 0x17f | 0x0bf | 0x05f | 0x02f | 0x017 | 0x00b | 0x03f | 0x026 |
         * +   + -----+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   | 1024 | 0x3ff | 0x1ff | 0x0ff | 0x07f | 0x03f | 0x01f | 0x00f | 0x055 | 0x033 |
         * +   +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   | 1536 | 0x5ff | 0x2ff | 0x17f | 0x0bf | 0x05f | 0x02f | 0x017 | 0x07f | 0x04c |
         * +   +------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         * |   | 2048 | 0x7ff | 0x3ff | 0x1ff | 0x0ff | 0x07f | 0x03f | 0x01f | 0x0aa | 0x066 |
         * +---+------+-------+-------+-------+-------+-------+-------+-------+-------+-------+
         */
        unsigned short      FDR         :11;
        unsigned short      reserved_1  : 5;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNBR_REG;

/// SMCIF Ch.N Control Register
typedef union SMCNCTL_REG_TAG {
    unsigned short  reg;
    struct SMCNCTL_REG_BIT_TAG {
        /**
         * This bit enables the SMCIF operations.
         * .MODEN
         *  This bit enables the SMCIF operations.
         *    1 (R/W): Enable SMCIF operations (The operating clock is supplied.)
         *    0 (R/W): Disable SMCIF operations (The operating clock is stopped.)
         */
        unsigned short      MODEN       : 1;
        /**
         * This bit issues software reset to SMCIF.
         * .SFTRST
         *  This bit issues software reset to SMCIF.
         *    1 (W): Issue software reset
         *    0 (W): Ineffective
         *    1 (R): Software reset is executing.
         *    0 (R): Software reset has finished. (During normal operation)
         */
        unsigned short      SFTRST      : 1;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * This bit enables or disables the data retransmission and retransmission request functions when an error
         * has been detected. This bit is effective only in T = 0 mode.
         * .CRP
         *  This bit enables or disables the data retransmission and retransmission request functions when an error
         *  has been detected. This bit is effective only in T = 0 mode.
         *  For data reception
         *    1 (R/W): Enable data retransmission request signal issuance
         *    0 (R/W): Disable data retransmission request signal issuance
         *  For data transmission
         *    1 (R/W): Enable error signal detection and data retransmission
         *    0 (R/W): Disable error signal detection and data retransmission
         */
        unsigned short      CRP         : 1;
        unsigned short      reserved_2  : 3;    ///< Reserved.
        /**
         * This bit enables or disables data transmit operations.
         * .TXEN
         *  This bit enables or disables data transmit operations.
         *    1 (R/W): Enable data transmission
         *    0 (R/W): Disable data transmission
         */
        unsigned short      TXEN        : 1;
        /**
         * This bit enables or disables data receive operations.
         * .RXEN
         *  This bit enables or disables data receive operations.
         *    1 (R/W): Enable data reception
         *    0 (R/W): Disable data reception
         */
        unsigned short      RXEN        : 1;
        /**
         * This bit controls the clock output to the SMCCLKn pin.
         * .CLKOUT
         *  This bit controls the clock output to the SMCCLKn pin.
         *    1 (R/W): Clock is output.
         *    0 (R/W): Clock is stopped.
         */
        unsigned short      CLKOUT      : 1;
        /**
         * This bit starts a clock count operation using the wait time counter.
         * .ICNT
         *  This bit starts a clock count operation using the wait time counter.
         *    1 (W): Start counting
         *    0 (W): Stop counting
         *    1 (R): During counting
         *    0 (R): Counting stopped
         */
        unsigned short      ICNT        : 1;
        unsigned short      reserved_3  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNCTL_REG;

/// SMCIF Ch.N Transmit Data Register
typedef union SMCNTXD_REG_TAG {
    unsigned short  reg;
    struct SMCNTXD_REG_BIT_TAG {
        /**
         * Data can be written to the transmit data buffer through these bits.
         * .TXD[7:0]
         *  Data can be written to the transmit data buffer through these bits.
         */
        unsigned short      TXD         : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNTXD_REG;

/// SMCIF Ch.N Receive Data Register
typedef union SMCNRXD_REG_TAG {
    unsigned short  reg;
    struct SMCNRXD_REG_BIT_TAG {
        /**
         * The receive data buffer can be read through these bits.
         * .RXD[7:0]
         *  The receive data buffer can be read through these bits. The receive data buffer consists of a 2-byte
         *  FIFO, and older received data is read first.
         */
        unsigned short      RXD         : 8;
        /**
         * This bit indicates the parity check results of the received data.
         * .RXP
         *  This bit indicates the parity check results of the received data.
         *    1 (R): Parity error occurred
         *    0 (R): No parity error occurred
         */
        unsigned short      RXP         : 1;
        unsigned short      reserved_1  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNRXD_REG;

/// SMCIF Ch.N Wait Time Compare Data Register 0
typedef union SMCNWTC0_REG_TAG {
    unsigned short  reg;
    struct SMCNWTC0_REG_BIT_TAG {
        /**
         * These bits set the wait time in etu units.
         * .WTC[15:0]
         *  These bits set the wait time in etu units.
         *  The value set in these registers is compared with the etu counter value without being changed.
         */
        unsigned short      WTC         :16;
    } bit;
    unsigned char   byte_reg[2];
} SMCNWTC0_REG;

/// SMCIF Ch.N Wait Time Compare Data Register 1
typedef union SMCNWTC1_REG_TAG {
    unsigned short  reg;
    struct SMCNWTC1_REG_BIT_TAG {
        /**
         * These bits set the wait time in etu units.
         * .WTC[23:16]
         *  These bits set the wait time in etu units.
         *  The value set in these registers is compared with the etu counter value without being changed.
         */
        unsigned short      WTC         : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNWTC1_REG;

/// SMCIF Ch.N Wait Time Compare Data Register
typedef union SMCNWTC_DWORD_REG_TAG {
    unsigned long   reg;
} SMCNWTC_DWORD_REG;

/// SMCIF Ch.N Guard Time Compare Data Register
typedef union SMCNGTC_REG_TAG {
    unsigned short  reg;
    struct SMCNGTC_REG_BIT_TAG {
        /**
         * These bits set the guard time in etu units.
         * .GTC[7:0]
         *  These bits set the guard time in etu units.
         */
        unsigned short      GTC         : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNGTC_REG;

/// SMCIF Ch.N Status and Interrupt Flag Register
typedef union SMCNINTF_REG_TAG {
    unsigned short  reg;
    struct SMCNINTF_REG_BIT_TAG {
        /**
         * This bit indicate Transmit buffer empty interrupt cause occurrence status.
         * .TBEIF
         *  This bit indicate Transmit buffer empty interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      TBEIF       : 1;
        /**
         * This bit indicate Receive buffer one byte full interrupt cause occurrence status.
         * .RB1FIF
         *  This bit indicate Receive buffer one byte full interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      RB1FIF      : 1;
        /**
         * This bit indicate Receive buffer two byte full interrupt cause occurrence status.
         * .RB2FIF
         *  This bit indicate Receive buffer two byte full interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      RB2FIF      : 1;
        /**
         * This bit indicate Overrun error interrupt cause occurrence status.
         * .OEIF
         *  This bit indicate Overrun error interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      OEIF        : 1;
        /**
         * This bit indicate Parity error interrupt cause occurrence status.
         * .PEIF
         *  This bit indicate Parity error interrupt ause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      PEIF        : 1;
        /**
         * This bit indicate Error signal detection interrupt cause occurrence status.
         * .EDTIF
         *  This bit indicate Error signal detection interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      EDTIF       : 1;
        /**
         * This bit indicate End-of-transmission interrupt cause occurrence status.
         * .TENDIF
         *  This bit indicate End-of-transmission interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      TENDIF      : 1;
        /**
         * This bit indicate Wait time error interrupt cause occurrence status.
         * .WTEIF
         *  This bit indicate Wait time error interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         */
        unsigned short      WTEIF       : 1;
        /**
         * This bit indicates the transmit operation status.
         * .TBSY
         *  This bit indicates the transmit operation status.
         *    1 (R): During transmitting
         *    0 (R): Idle
         */
        unsigned short      TBSY        : 1;
        /**
         * This bit indicates the receive operation status.
         * .RBSY
         *  This bit indicates the receive operation status.
         *    1 (R): During receiving
         *    0 (R): Idle
         */
        unsigned short      RBSY        : 1;
        /**
         * This bit indicates the guard time measurement status of the etu counter.
         * .GDTM
         *  This bit indicates the guard time measurement status of the etu counter.
         *    1 (R): During measurement
         *    0 (R): Idle
         */
        unsigned short      GDTM        : 1;
        /**
         * This bit indicates the wait time measurement status of the etu counter.
         * .WTTM
         *  This bit indicates the wait time measurement status of the etu counter.
         *    1 (R): During measurement
         *    0 (R): Idle
         */
        unsigned short      WTTM        : 1;
        unsigned short      reserved_1  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNINTF_REG;

/// SMCIF Ch.N Interrupt Enable Register
typedef union SMCNINTE_REG_TAG {
    unsigned short  reg;
    struct SMCNINTE_REG_BIT_TAG {
        /**
         * This bit enable transmitTransmit buffer empty interrupt.
         * .TBEIE
         *  This bit enable transmitTransmit buffer empty interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      TBEIE       : 1;
        /**
         * This bit enable Receive buffer one byte full interrupt.
         * .RB1FIE
         *  This bit enable Receive buffer one byte full interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      RB1FIE      : 1;
        /**
         * This bit enable Receive buffer two bytes full interrupt.
         * .RB2FIE
         *  This bit enable Receive buffer two bytes full interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      RB2FIE      : 1;
        /**
         * This bit enable Overrun error interrupt.
         * .OEIE
         *  This bit enable Overrun error interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      OEIE        : 1;
        /**
         * This bit enable Parity error interrupt.
         * .PEIE
         *  This bit enable Parity error interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      PEIE        : 1;
        /**
         * This bit enable Error signal detection interrupt.
         * .EDTIE
         *  This bit enable Error signal detection interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      EDTIE       : 1;
        /**
         * This bit enable End-of-transmission interrupt.
         * .TENDIE
         *  This bit enable End-of-transmission interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      TENDIE      : 1;
        /**
         * This bit enable Wait time error interrupt.
         * .WTEIE
         *  This bit enable Wait time error interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      WTEIE       : 1;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNINTE_REG;

/// SMCIF Ch.N Etu Counter Data Register 0
typedef union SMCNETU0_REG_TAG {
    unsigned short  reg;
    struct SMCNETU0_REG_BIT_TAG {
        /**
         * These bits show the current etu counter value.
         * .ETUC[15:0]
         *  These bits show the current etu counter value.
         */
        unsigned short      ETUC        :16;
    } bit;
    unsigned char   byte_reg[2];
} SMCNETU0_REG;

/// SMCIF Ch.N Etu Counter Data Register 1
typedef union SMCNETU1_REG_TAG {
    unsigned short  reg;
    struct SMCNETU1_REG_BIT_TAG {
        /**
         * These bits show the current etu counter value.
         * .ETUC[23:16]
         *  These bits show the current etu counter value.
         */
        unsigned short      ETUC        : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SMCNETU1_REG;

/// SMCIF Ch.N Etu Counter Data Register
typedef union SMCNETU_DWORD_REG_TAG {
    unsigned long   reg;
} SMCNETU_DWORD_REG;

#ifdef __cplusplus
}
#endif

#endif // SMCIF_REG_H__
