/************************************************************************//**
 *
 * @file    c17_adc12a.c
 *
 * @brief   This is ADC12A driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Change source code.
 *          - 01 Nov. 2018    1.20
 *              - Change source code.
 *
 ***************************************************************************/
#include "crt0.h"
#include "c17_misc.h"
#include "c17_adc12a.h"
#include "c17_init_config.h"

#if defined(C17_PE_ADC)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
#if defined(C17_MCUSEL_M13)
C17_INTERRUPT_HANDLER(_vector18_handler, c17intAd12);		// ADC12A interrupt
#endif
#if defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
C17_INTERRUPT_HANDLER(_vector26_handler, c17intAd12);		// ADC12A interrupt
#endif
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_M40)
C17_INTERRUPT_HANDLER(_vector25_handler, c17intAd12);		// ADC12A interrupt
#endif
#if defined(C17_MCUSEL_F63)
C17_INTERRUPT_HANDLER(_vector19_handler, c17intAd12);		// ADC12A interrupt
#endif
#endif /* CRT0_LINKED */


/* --- global variable --- */
/**
 * @todo change the number of buffer.
 */
#define ADC_BUFF_COUNT	20

/**
 * @todo change the number of elements in that array.
 */
unsigned short adVal[ADC_BUFF_COUNT];

// Interrupt counter.
/**
 * @todo c17adcIntCount = Interruption counter.
 */
volatile unsigned char c17adcIntCount = 0;	///< Interruption counter.


/* --- static variable --- */
static void setAdc12PortCh0(unsigned short start, unsigned short end);

/************************************************************************//**
 * @brief   Initialize ADC.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initAdc(void)
{
	///Stop ADC12A.
	c17stopAdc();

	/// ADC12A CH 0 Setting
	if((ADCCH0_INIT & 0x0200) == 0x0200)
	{
		c17setAdc12Ch0((ADCCH0_INIT & 0x0100)>>8, (ADCCH0_INIT & 0x00C0)>>6, (ADCCH0_INIT & 0x0038)>>3, (ADCCH0_INIT & 0x0007));

		setAdc12PortCh0((ADCCH0_INIT & 0x0038)>>3, (ADCCH0_INIT & 0x0007));

		/// Initialize the interrupt.
		ILVADC12_0 = INTERRUPT_LEVEL_ADC0;				/// - Set interrupt level.
	}

}


/************************************************************************//**
 * @brief   ADC12A set.
 *
 * @param   cnvmode      Conversion mode.
 *  @li @c  C17_ADC12A_MODE_CONT     Continuous.
 *  @li @c  C17_ADC12A_MODE_ONETIME1 One-time.
 * @param   trigger                  Trigger source.
 *  @li @c  C17_ADC12A_TRG_SOFT      Software trigger.
 *  @li @c  C17_ADC12A_TRG_T16       16-bit timer.
 *  @li @c  C17_ADC12A_TRG_EXTERNAL  External trigger.
 * @param   start                    A/D converted first.
 *  @li @c  C17_ADC12A_START0 to C17_ADC12A_START7
 * @param   end                      A/D converted last.
 *  @li @c  C17_ADC12A_END0 to C17_ADC12A_END7
 *
 * @retval     none
 ***************************************************************************/
void c17setAdc12Ch0(unsigned short cnvmode, unsigned short trigger, unsigned short start, unsigned short end)
{
	ADC12_NCTL_REG	adc12_0ctl_set;
	ADC12_NTRG_REG	adc12_0trg_set;

	/* ADC12_0CTL: ADC12A Ch.0 Control Register */
	{
		adc12_0ctl_set.reg = ADC12_0CTL;
		adc12_0ctl_set.bit.MODEN = 1;
		ADC12_0CTL = adc12_0ctl_set.reg;
	}

	/*  ADC12_0TRG :ADC12A Ch.0 Trigger/Analog Input Select Register */
	{
	    /**
	     * @todo change STMD,SMPCLK
	     */
		adc12_0trg_set.bit.ENDAIN = end;		/// -ENDAIN   : converted last selection
		adc12_0trg_set.bit.STAAIN = start;		/// -STAAIN   : converted first selection
		adc12_0trg_set.bit.STMD = 0;			/// -STMD     : right
		adc12_0trg_set.bit.CNVMD = cnvmode;		/// -CNVM     : conversion mode selection
		adc12_0trg_set.bit.CNVTRG = trigger;	/// -CNVTRG   : Trigger source selection
		adc12_0trg_set.bit.SMPCLK = 4; 			/// -SMPCLK   : 8cycle
		ADC12_0TRG = adc12_0trg_set.reg;
	}

	/**
	 * @todo change voltage range
	 */
	/// ADC12A Ch.0 Configuration Register
	ADC12_0CFG =  0x0003;						/// -VRANGE   : 1.8-3.6V

}


/************************************************************************//**
 * @brief   Setting Port(GPIO) for ADC12A CH.0.
 *
 * @param   start        A/D converted first.
 *  @li @c  C17_ADC12A_START0 to C17_ADC12A_START7
 * @param   end          A/D converted last.
 *  @li @c  C17_ADC12A_END0 to C17_ADC12A_END7
 *
 * @retval     none
 ***************************************************************************/
static void setAdc12PortCh0(unsigned short start, unsigned short end)
{
#if defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_F63)
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;
#endif
#if defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
	P1MODSEL_REG	p1modsel_set;
	P1FNCSEL_REG	p1fncsel_set;
#endif
#if defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	P2MODSEL_REG	p2modsel_set;
	P2FNCSEL_REG	p2fncsel_set;
#endif
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_M40)
	P3MODSEL_REG	p3modsel_set;
	P3FNCSEL_REG	p3fncsel_set;
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
	P7MODSEL_REG	p7modsel_set;
	P7FNCSEL_REG	p7fncsel_set;
#endif


#if defined(C17_MCUSEL_M13)
	///Assign port to ADC12 ADIN
	/*
	 *  P07 : ADIN0
	 *  P06 : ADIN1
	 *  P05 : ADIN2
	 *  P04 : ADIN3
	 *  P03 : ADIN4
	 *  P02 : ADIN5
	 *  P01 : ADIN6
	 *  P00 : ADIN7
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P07SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P07 : Enable peripheral I/O function.
		p0modsel_set.bit.P06SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P06 : Enable peripheral I/O function.
		p0modsel_set.bit.P05SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P05 : Enable peripheral I/O function.
		p0modsel_set.bit.P04SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P04 : Enable peripheral I/O function.
		p0modsel_set.bit.P03SEL = ((start <= 4) && (end >= 4)) ? 1:0;		/// - P03 : Enable peripheral I/O function.
		p0modsel_set.bit.P02SEL = ((start <= 5) && (end >= 5)) ? 1:0;		/// - P02 : Enable peripheral I/O function.
		p0modsel_set.bit.P01SEL = ((start <= 6) && (end >= 6)) ? 1:0;		/// - P01 : Enable peripheral I/O function.
		p0modsel_set.bit.P00SEL = ((start <= 7) && (end >= 7)) ? 1:0;		/// - P00 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;

	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P07MUX = ((start <= 0) && (end >= 0)) ? 2:0;		/// - P07 : UPMUX.
		p0fncsel_set.bit.P06MUX = ((start <= 1) && (end >= 1)) ? 2:0;		/// - P06 : UPMUX.
		p0fncsel_set.bit.P05MUX = ((start <= 2) && (end >= 2)) ? 2:0;		/// - P05 : UPMUX.
		p0fncsel_set.bit.P04MUX = ((start <= 3) && (end >= 3)) ? 2:0;		/// - P04 : UPMUX.
		p0fncsel_set.bit.P03MUX = ((start <= 4) && (end >= 4)) ? 2:0;		/// - P03 : UPMUX.
		p0fncsel_set.bit.P02MUX = ((start <= 5) && (end >= 5)) ? 2:0;		/// - P02 : UPMUX.
		p0fncsel_set.bit.P01MUX = ((start <= 6) && (end >= 6)) ? 2:0;		/// - P01 : UPMUX.
		p0fncsel_set.bit.P00MUX = ((start <= 7) && (end >= 7)) ? 2:0;		/// - P00 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}

	//VREFA0 = P10
	/// Set Universal Port Multiplexer (UPMUX).
	/* P1MODSEL: P1 Port Mode Select Register */
	{
		/// P1 Port Mode Select Register
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P10SEL = 1;		/// - P10 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;

	}
	/* P1FNCSEL: P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P10MUX = 2;		/// - P10 : UPMUX.
		P1FNCSEL = p1fncsel_set.reg;
	}
#endif

#if defined(C17_MCUSEL_W23)
	///Assign port to ADC12 ADIN
	/*
	 *  P07 : ADIN3
	 *  P06 : ADIN4
	 *  P05 : ADIN5
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P07SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P07 : Enable peripheral I/O function.
		p0modsel_set.bit.P06SEL = ((start <= 4) && (end >= 4)) ? 1:0;		/// - P06 : Enable peripheral I/O function.
		p0modsel_set.bit.P05SEL = ((start <= 5) && (end >= 5)) ? 1:0;		/// - P05 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;

	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P07MUX = ((start <= 3) && (end >= 3)) ? 2:0;		/// - P07 : UPMUX.
		p0fncsel_set.bit.P06MUX = ((start <= 4) && (end >= 4)) ? 2:0;		/// - P06 : UPMUX.
		p0fncsel_set.bit.P05MUX = ((start <= 5) && (end >= 5)) ? 2:0;		/// - P05 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}

	///Assign port to ADC12 ADIN,VREF
	/*
	 *  P13 : VREFA0
	 *  P12 : ADIN0
	 *  P11 : ADIN1
	 *  P10 : ADIN2
	 */

	/// Set Universal Port Multiplexer (UPMUX).
	/* P1MODSEL: P1 Port Mode Select Register */
	{
		/// P1 Port Mode Select Register
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P13SEL = 1;										/// - P13 : Enable peripheral I/O function.
		p1modsel_set.bit.P12SEL = ((start <= 0) && (end >= 0)) ? 1:0;;		/// - P12 : Enable peripheral I/O function.
		p1modsel_set.bit.P11SEL = ((start <= 1) && (end >= 1)) ? 1:0;;		/// - P11 : Enable peripheral I/O function.
		p1modsel_set.bit.P10SEL = ((start <= 2) && (end >= 2)) ? 1:0;;		/// - P10 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;

	}
	/* P1FNCSEL: P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P13MUX = 2;										/// - P13 : UPMUX.
		p1fncsel_set.bit.P12MUX = ((start <= 0) && (end >= 0)) ? 2:0;;		/// - P12 : UPMUX.
		p1fncsel_set.bit.P11MUX = ((start <= 1) && (end >= 1)) ? 2:0;;		/// - P11 : UPMUX.
		p1fncsel_set.bit.P10MUX = ((start <= 2) && (end >= 2)) ? 2:0;;		/// - P10 : UPMUX.
		P1FNCSEL = p1fncsel_set.reg;
	}
#endif

#if defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
	///Assign port to ADC12 ADIN
	/*
	 *  P17 : ADIN6
	 *  P16 : ADIN5
	 *  P15 : ADIN4
	 *  P14 : ADIN3
	 *  P13 : ADIN2
	 *  P12 : ADIN1
	 *  P11 : ADIN0
	 *  P10 : VREFA0
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P1MODSEL: P1 Port Mode Select Register */
	{
		/// P1 Port Mode Select Register
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P11SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P11 : Enable peripheral I/O function.
		p1modsel_set.bit.P12SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P12 : Enable peripheral I/O function.
		p1modsel_set.bit.P13SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P13 : Enable peripheral I/O function.
		p1modsel_set.bit.P14SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P14 : Enable peripheral I/O function.
		p1modsel_set.bit.P15SEL = ((start <= 4) && (end >= 4)) ? 1:0;		/// - P15 : Enable peripheral I/O function.
		p1modsel_set.bit.P16SEL = ((start <= 5) && (end >= 5)) ? 1:0;		/// - P16 : Enable peripheral I/O function.
		p1modsel_set.bit.P17SEL = ((start <= 6) && (end >= 6)) ? 1:0;		/// - P17 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;

	}
	/* P1FNCSEL: P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P11MUX = ((start <= 0) && (end >= 0)) ? 2:0;		/// - P11 : UPMUX.
		p1fncsel_set.bit.P12MUX = ((start <= 1) && (end >= 1)) ? 2:0;		/// - P12 : UPMUX.
		p1fncsel_set.bit.P13MUX = ((start <= 2) && (end >= 2)) ? 2:0;		/// - P13 : UPMUX.
		p1fncsel_set.bit.P14MUX = ((start <= 3) && (end >= 3)) ? 2:0;		/// - P14 : UPMUX.
		p1fncsel_set.bit.P15MUX = ((start <= 4) && (end >= 4)) ? 2:0;		/// - P15 : UPMUX.
		p1fncsel_set.bit.P16MUX = ((start <= 5) && (end >= 5)) ? 2:0;		/// - P16 : UPMUX.
		p1fncsel_set.bit.P17MUX = ((start <= 6) && (end >= 6)) ? 2:0;		/// - P17 : UPMUX.
		P1FNCSEL = p1fncsel_set.reg;
	}

	//VREFA0 = P10
	/// Set Universal Port Multiplexer (UPMUX).
	/* P1MODSEL: P1 Port Mode Select Register */
	{
		/// P1 Port Mode Select Register
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P10SEL = 1;		/// - P10 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;

	}
	/* P1FNCSEL: P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P10MUX = 2;		/// - P10 : UPMUX.
		P1FNCSEL = p1fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W16)
	///Assign port to ADC12 ADIN
	/*
	 *  P25 : ADIN3
	 *  P24 : ADIN2
	 *  P23 : ADIN1
	 *  P22 : ADIN0
	 *  P21 : VREFA0
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P2MODSEL: P2 Port Mode Select Register */
	{
		/// P2 Port Mode Select Register
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P22SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P22 : Enable peripheral I/O function.
		p2modsel_set.bit.P23SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P23 : Enable peripheral I/O function.
		p2modsel_set.bit.P24SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P24 : Enable peripheral I/O function.
		p2modsel_set.bit.P25SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P25 : Enable peripheral I/O function.
		P2MODSEL = p2modsel_set.reg;

	}
	/* P2FNCSEL: P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P22MUX = ((start <= 0) && (end >= 0)) ? 2:0;		/// - P22 : UPMUX.
		p2fncsel_set.bit.P23MUX = ((start <= 1) && (end >= 1)) ? 2:0;		/// - P23 : UPMUX.
		p2fncsel_set.bit.P24MUX = ((start <= 2) && (end >= 2)) ? 2:0;		/// - P24 : UPMUX.
		p2fncsel_set.bit.P25MUX = ((start <= 3) && (end >= 3)) ? 2:0;		/// - P25 : UPMUX.
		P2FNCSEL = p2fncsel_set.reg;
	}

	//VREFA0 = P21
	/// Set Universal Port Multiplexer (UPMUX).
	/* P2MODSEL: P2 Port Mode Select Register */
	{
		/// P2 Port Mode Select Register
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P21SEL = 1;		/// - P21 : Enable peripheral I/O function.
		P2MODSEL = p2modsel_set.reg;

	}
	/* P2FNCSEL: P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P21MUX = 2;		/// - P21 : UPMUX.
		P2FNCSEL = p2fncsel_set.reg;
	}
#endif
#if	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	///Assign port to ADC12 ADIN
	/*
	 *  P27 : ADIN0
	 *  P26 : ADIN1
	 *  P25 : ADIN2
	 *  P24 : ADIN3
	 *  P23 : ADIN4
	 *  P22 : ADIN5
	 *  P21 : ADIN6
	 *  P20 : ADIN7
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P2MODSEL: P2 Port Mode Select Register */
	{
		/// P2 Port Mode Select Register
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P27SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P27 : Enable peripheral I/O function.
		p2modsel_set.bit.P26SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P26 : Enable peripheral I/O function.
		p2modsel_set.bit.P25SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P25 : Enable peripheral I/O function.
		p2modsel_set.bit.P24SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P24 : Enable peripheral I/O function.
		p2modsel_set.bit.P23SEL = ((start <= 4) && (end >= 4)) ? 1:0;		/// - P23 : Enable peripheral I/O function.
		p2modsel_set.bit.P22SEL = ((start <= 5) && (end >= 5)) ? 1:0;		/// - P22 : Enable peripheral I/O function.
#if defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M25)
		p2modsel_set.bit.P21SEL = ((start <= 6) && (end >= 6)) ? 1:0;		/// - P21 : Enable peripheral I/O function.
		p2modsel_set.bit.P20SEL = ((start <= 7) && (end >= 7)) ? 1:0;		/// - P20 : Enable peripheral I/O function.
#endif
		P2MODSEL = p2modsel_set.reg;

	}
	/* P2FNCSEL: P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P27MUX = ((start <= 0) && (end >= 0)) ? 2:0;		/// - P27 : UPMUX.
		p2fncsel_set.bit.P26MUX = ((start <= 1) && (end >= 1)) ? 2:0;		/// - P26 : UPMUX.
		p2fncsel_set.bit.P25MUX = ((start <= 2) && (end >= 2)) ? 2:0;		/// - P25 : UPMUX.
		p2fncsel_set.bit.P24MUX = ((start <= 3) && (end >= 3)) ? 2:0;		/// - P24 : UPMUX.
		p2fncsel_set.bit.P23MUX = ((start <= 4) && (end >= 4)) ? 2:0;		/// - P23 : UPMUX.
		p2fncsel_set.bit.P22MUX = ((start <= 5) && (end >= 5)) ? 2:0;		/// - P22 : UPMUX.
#if defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M25)
		p2fncsel_set.bit.P21MUX = ((start <= 6) && (end >= 6)) ? 2:0;		/// - P21 : UPMUX.
		p2fncsel_set.bit.P20MUX = ((start <= 7) && (end >= 7)) ? 2:0;		/// - P20 : UPMUX.
#endif
		P2FNCSEL = p2fncsel_set.reg;
	}

	///Assign port to ADC12 VREF
	/*
	 *  P30 : VREFA0
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P3MODSEL: P3 Port Mode Select Register */
	{
		/// P3 Port Mode Select Register
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P30SEL = 1;		/// - P30 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;

	}
	/* P3FNCSEL: P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P30MUX = 2;		/// - P30 : UPMUX.
		P3FNCSEL = p3fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04)
	///Assign port to ADC12 ADIN
	/*
	 *  P35 : ADIN0
	 *  P34 : ADIN1
	 *  P33 : ADIN2
	 *  P32 : ADIN3
	 *  P31 : ADIN4
	 *  P30 : ADIN5
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P3MODSEL: P3 Port Mode Select Register */
	{
		/// P3 Port Mode Select Register
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P35SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P35 : Enable peripheral I/O function.
		p3modsel_set.bit.P34SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P34 : Enable peripheral I/O function.
		p3modsel_set.bit.P33SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P33 : Enable peripheral I/O function.
		p3modsel_set.bit.P32SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P32 : Enable peripheral I/O function.
		p3modsel_set.bit.P31SEL = ((start <= 4) && (end >= 4)) ? 1:0;		/// - P31 : Enable peripheral I/O function.
		p3modsel_set.bit.P30SEL = ((start <= 5) && (end >= 5)) ? 1:0;		/// - P30 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;

	}
	/* P3FNCSEL: P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P35MUX = ((start <= 0) && (end >= 0)) ? 2:0;		/// - P35 : UPMUX.
		p3fncsel_set.bit.P34MUX = ((start <= 1) && (end >= 1)) ? 2:0;		/// - P34 : UPMUX.
		p3fncsel_set.bit.P33MUX = ((start <= 2) && (end >= 2)) ? 2:0;		/// - P33 : UPMUX.
		p3fncsel_set.bit.P32MUX = ((start <= 3) && (end >= 3)) ? 2:0;		/// - P32 : UPMUX.
		p3fncsel_set.bit.P31MUX = ((start <= 4) && (end >= 4)) ? 2:0;		/// - P31 : UPMUX.
		p3fncsel_set.bit.P30MUX = ((start <= 5) && (end >= 5)) ? 2:0;		/// - P30 : UPMUX.
		P3FNCSEL = p3fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
	///Assign port to ADC12 ADIN
	/*
	 *  P72 : ADIN0
	 *  P73 : ADIN1
	 *  P74 : ADIN2
	 *  P75 : ADIN3
	 *  P76 : ADIN4
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P7MODSEL: P7 Port Mode Select Register */
	{
		/// P7 Port Mode Select Register
		p7modsel_set.reg = P7MODSEL;
		p7modsel_set.bit.P72SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P72 : Enable peripheral I/O function.
		p7modsel_set.bit.P73SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P73 : Enable peripheral I/O function.
#if defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
		p7modsel_set.bit.P74SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P74 : Enable peripheral I/O function.
		p7modsel_set.bit.P75SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P75 : Enable peripheral I/O function.
		p7modsel_set.bit.P76SEL = ((start <= 4) && (end >= 4)) ? 1:0;		/// - P76 : Enable peripheral I/O function.
#endif
		P7MODSEL = p7modsel_set.reg;

	}
	/* P7FNCSEL: P7 Port Function Select Register */
	{
		p7fncsel_set.reg = P7FNCSEL;
		p7fncsel_set.bit.P72MUX = ((start <= 0) && (end >= 0)) ? 3:0;		/// - P72 : UPMUX.
		p7fncsel_set.bit.P73MUX = ((start <= 1) && (end >= 1)) ? 3:0;		/// - P73 : UPMUX.
#if defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
		p7fncsel_set.bit.P74MUX = ((start <= 2) && (end >= 2)) ? 3:0;		/// - P74 : UPMUX.
		p7fncsel_set.bit.P75MUX = ((start <= 3) && (end >= 3)) ? 3:0;		/// - P75 : UPMUX.
		p7fncsel_set.bit.P76MUX = ((start <= 4) && (end >= 4)) ? 3:0;		/// - P76 : UPMUX.
#endif
		P7FNCSEL = p7fncsel_set.reg;
	}

	//VREFA0 = P71
	/// Set Universal Port Multiplexer (UPMUX).
	/* P7MODSEL: P7 Port Mode Select Register */
	{
		/// P7 Port Mode Select Register
		p7modsel_set.reg = P1MODSEL;
		p7modsel_set.bit.P71SEL = 1;		/// - P71 : Enable peripheral I/O function.
		P7MODSEL = p7modsel_set.reg;

	}
	/* P7FNCSEL: P7 Port Function Select Register */
	{
		p7fncsel_set.reg = P7FNCSEL;
		p7fncsel_set.bit.P71MUX = 2;		/// - P71 : UPMUX.
		P7FNCSEL = p7fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_M40)
	///Assign port to ADC12 ADIN
	/*
	 *  P34 : ADIN0
	 *  P35 : ADIN1
	 *  P36 : ADIN2
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P3MODSEL: P3 Port Mode Select Register */
	{
		/// P3 Port Mode Select Register
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P34SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P34 : Enable peripheral I/O function.
		p3modsel_set.bit.P35SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P35 : Enable peripheral I/O function.
		p3modsel_set.bit.P36SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P36 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;
	}
	/* P3FNCSEL: P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P34MUX = ((start <= 0) && (end >= 0)) ? 2:0;		/// - P34 : UPMUX.
		p3fncsel_set.bit.P35MUX = ((start <= 1) && (end >= 1)) ? 2:0;		/// - P35 : UPMUX.
		p3fncsel_set.bit.P36MUX = ((start <= 2) && (end >= 2)) ? 2:0;		/// - P36 : UPMUX.
		P3FNCSEL = p3fncsel_set.reg;
	}

	//VREFA0 = P32
	/// Set Universal Port Multiplexer (UPMUX).
	/* P3MODSEL: P3 Port Mode Select Register */
	{
		/// P3 Port Mode Select Register
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P32SEL = 1;		/// - P32 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;

	}
	/* P3FNCSEL: P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P32MUX = 2;		/// - P32 : UPMUX.
		P3FNCSEL = p3fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_F63)
	///Assign port to ADC12 ADIN
	/*
	 *  P07 : ADIN0
	 *  P06 : ADIN1
	 *  P05 : ADIN2
	 *  P04 : ADIN3
	 *  P03 : ADIN4
	 *  P02 : ADIN5
	 *  P01 : ADIN6
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P07SEL = ((start <= 0) && (end >= 0)) ? 1:0;		/// - P07 : Enable peripheral I/O function.
		p0modsel_set.bit.P06SEL = ((start <= 1) && (end >= 1)) ? 1:0;		/// - P06 : Enable peripheral I/O function.
		p0modsel_set.bit.P05SEL = ((start <= 2) && (end >= 2)) ? 1:0;		/// - P05 : Enable peripheral I/O function.
		p0modsel_set.bit.P04SEL = ((start <= 3) && (end >= 3)) ? 1:0;		/// - P04 : Enable peripheral I/O function.
		p0modsel_set.bit.P03SEL = ((start <= 4) && (end >= 4)) ? 1:0;		/// - P03 : Enable peripheral I/O function.
		p0modsel_set.bit.P02SEL = ((start <= 5) && (end >= 5)) ? 1:0;		/// - P02 : Enable peripheral I/O function.
		p0modsel_set.bit.P01SEL = ((start <= 6) && (end >= 6)) ? 1:0;		/// - P01 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;

	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P07MUX = ((start <= 0) && (end >= 0)) ? 2:0;		/// - P07 : UPMUX.
		p0fncsel_set.bit.P06MUX = ((start <= 1) && (end >= 1)) ? 2:0;		/// - P06 : UPMUX.
		p0fncsel_set.bit.P05MUX = ((start <= 2) && (end >= 2)) ? 2:0;		/// - P05 : UPMUX.
		p0fncsel_set.bit.P04MUX = ((start <= 3) && (end >= 3)) ? 2:0;		/// - P04 : UPMUX.
		p0fncsel_set.bit.P03MUX = ((start <= 4) && (end >= 4)) ? 2:0;		/// - P03 : UPMUX.
		p0fncsel_set.bit.P02MUX = ((start <= 5) && (end >= 5)) ? 2:0;		/// - P02 : UPMUX.
		p0fncsel_set.bit.P01MUX = ((start <= 6) && (end >= 6)) ? 2:0;		/// - P01 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}

	//VREFA0 = P00
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P1 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P00SEL = 1;		/// - P00 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;

	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P00MUX = 2;		/// - P00 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}
#endif
}


/************************************************************************//**
 * @brief   Start ADC.
 *
 * @param   none
 *
 * @return  none
 ***************************************************************************/
void c17startAdc(void)
{
	ADC12_NCTL_REG	adc12_0ctl_set;

	/// clear flag
	ADC12_0INTF	= 0xFFFF;			// 1111.1111.1111.1111	interrupt flag clear

	// ADST=1 MODEN=1
	/* ADC12_0CTL: ADC12A Ch.0 Control Register */
	{
		adc12_0ctl_set.reg = ADC12_0CTL;
		adc12_0ctl_set.bit.ADST = 1;
		adc12_0ctl_set.bit.MODEN = 1;
		ADC12_0CTL = adc12_0ctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Stop ADC.
 *
 * @param   none
 *
 * @return  none
 ***************************************************************************/
void c17stopAdc(void)
{
	ADC12_NCTL_REG	adc12_0ctl_set;

	// ADST=0 MODEN=0
	/* ADC12_0CTL: ADC12A Ch.0 Control Register */
	{
		adc12_0ctl_set.reg = ADC12_0CTL;
		adc12_0ctl_set.bit.ADST = 0;
		adc12_0ctl_set.bit.MODEN = 0;
		ADC12_0CTL = adc12_0ctl_set.reg;
	}
}


#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
/************************************************************************//**
 * @brief   ADC12A interrupt start.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntAdc(void)
{
	ADC12_NINTE_REG	adc12_0inte_set;

	ADC12_0INTF	= 0xFFFF;			/// -interrupt flag clear

	/**
	 * @todo Change interrupt
	 */
	// AD0OVIE,AD0CIE=1(enable)
	/*  ADC12_0INTE:ADC12A Ch.0 Interrupt Enable Register */
	{
		adc12_0inte_set.bit.AD7OVIE = 0;
		adc12_0inte_set.bit.AD6OVIE = 0;
		adc12_0inte_set.bit.AD5OVIE = 0;
		adc12_0inte_set.bit.AD4OVIE = 0;
		adc12_0inte_set.bit.AD3OVIE = 0;
		adc12_0inte_set.bit.AD2OVIE = 0;
		adc12_0inte_set.bit.AD1OVIE = 0;
		adc12_0inte_set.bit.AD0OVIE = 1;

		adc12_0inte_set.bit.AD7CIE = 0;
		adc12_0inte_set.bit.AD6CIE = 0;
		adc12_0inte_set.bit.AD5CIE = 0;
		adc12_0inte_set.bit.AD4CIE = 0;
		adc12_0inte_set.bit.AD3CIE = 0;
		adc12_0inte_set.bit.AD2CIE = 0;
		adc12_0inte_set.bit.AD1CIE = 0;
		adc12_0inte_set.bit.AD0CIE = 1;			/// Use for the ADC12A sample program.
		ADC12_0INTE = adc12_0inte_set.reg;

	}
}
#endif

#if defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_F63)
/************************************************************************//**
 * @brief   ADC12A interrupt start.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntAdc(void)
{
	ADC12_NINTE_REG	adc12_0inte_set;

	ADC12_0INTF	= 0xFFFF;			/// -interrupt flag clear

	/**
	 * @todo Change interrupt
	 */
	// AD0OVIE,AD0CIE=1(enable)
	/*  ADC12_0INTE:ADC12A Ch.0 Interrupt Enable Register */
	{
		adc12_0inte_set.bit.AD7OVIE = 0;
		adc12_0inte_set.bit.AD6OVIE = 0;
		adc12_0inte_set.bit.AD5OVIE = 0;
		adc12_0inte_set.bit.AD4OVIE = 0;
		adc12_0inte_set.bit.AD3OVIE = 0;
		adc12_0inte_set.bit.AD2OVIE = 0;
		adc12_0inte_set.bit.AD1OVIE = 0;
		adc12_0inte_set.bit.AD0OVIE = 1;

		adc12_0inte_set.bit.AD7CIE = 1;			/// Use for the TSRVR sample program.
		adc12_0inte_set.bit.AD6CIE = 0;
		adc12_0inte_set.bit.AD5CIE = 0;
		adc12_0inte_set.bit.AD4CIE = 0;
		adc12_0inte_set.bit.AD3CIE = 0;
		adc12_0inte_set.bit.AD2CIE = 0;
		adc12_0inte_set.bit.AD1CIE = 0;
		adc12_0inte_set.bit.AD0CIE = 1;			/// Use for the ADC12A sample program.
		ADC12_0INTE = adc12_0inte_set.reg;

	}
}
#endif

#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
/************************************************************************//**
 * @brief   ADC12A interrupt start.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntAdc(void)
{
	ADC12_NINTE_REG	adc12_0inte_set;

	ADC12_0INTF	= 0xFFFF;			/// -interrupt flag clear

	/**
	 * @todo Change interrupt
	 */
	// AD0OVIE,AD0CIE=1(enable)
	/*  ADC12_0INTE:ADC12A Ch.0 Interrupt Enable Register */
	{
		adc12_0inte_set.bit.AD5OVIE = 0;
#if defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
		adc12_0inte_set.bit.AD4OVIE = 0;
		adc12_0inte_set.bit.AD3OVIE = 0;
		adc12_0inte_set.bit.AD2OVIE = 0;
#endif
		adc12_0inte_set.bit.AD1OVIE = 0;
		adc12_0inte_set.bit.AD0OVIE = 1;

		adc12_0inte_set.bit.AD5CIE = 1;			/// Use for the TSRVR sample program.
#if defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
		adc12_0inte_set.bit.AD4CIE = 0;
		adc12_0inte_set.bit.AD3CIE = 0;
		adc12_0inte_set.bit.AD2CIE = 0;
#endif
		adc12_0inte_set.bit.AD1CIE = 0;
		adc12_0inte_set.bit.AD0CIE = 1;			/// Use for the ADC12A sample program.
		ADC12_0INTE = adc12_0inte_set.reg;

	}
}
#endif

#if defined(C17_MCUSEL_M40)
/************************************************************************//**
 * @brief   ADC12A interrupt start.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntAdc(void)
{
	ADC12_NINTE_REG	adc12_0inte_set;

	ADC12_0INTF	= 0xFFFF;			/// -interrupt flag clear

	/**
	 * @todo Change interrupt
	 */
	// AD0OVIE,AD0CIE=1(enable)
	/*  ADC12_0INTE:ADC12A Ch.0 Interrupt Enable Register */
	{
		adc12_0inte_set.bit.AD4OVIE = 0;
		adc12_0inte_set.bit.AD3OVIE = 0;
		adc12_0inte_set.bit.AD2OVIE = 0;
		adc12_0inte_set.bit.AD1OVIE = 0;
		adc12_0inte_set.bit.AD0OVIE = 1;

		adc12_0inte_set.bit.AD4CIE = 1;			/// Use for the TSRVR sample program.
		adc12_0inte_set.bit.AD3CIE = 0;
		adc12_0inte_set.bit.AD2CIE = 0;
		adc12_0inte_set.bit.AD1CIE = 0;
		adc12_0inte_set.bit.AD0CIE = 1;			/// Use for the ADC12A sample program.
		ADC12_0INTE = adc12_0inte_set.reg;

	}
}
#endif


/************************************************************************//**
 * @brief   ADC12A interrupt stop.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntAdc(void)
{
	ADC12_NINTE_REG	adc12_0inte_set;

	ADC12_0INTF	= 0xFFFF;			/// -interrupt flag clear

	/**
	 * @todo Change interrupt
	 */
	// AD0OVIE,AD0CIE=0(disable)
	/*  ADC12_0INTE:ADC12A Ch.0 Interrupt Enable Register */
	{
		adc12_0inte_set.bit.AD7OVIE = 0;
		adc12_0inte_set.bit.AD6OVIE = 0;
		adc12_0inte_set.bit.AD5OVIE = 0;
		adc12_0inte_set.bit.AD4OVIE = 0;
		adc12_0inte_set.bit.AD3OVIE = 0;
		adc12_0inte_set.bit.AD2OVIE = 0;
		adc12_0inte_set.bit.AD1OVIE = 0;
		adc12_0inte_set.bit.AD0OVIE = 0;

		adc12_0inte_set.bit.AD7CIE = 0;
		adc12_0inte_set.bit.AD6CIE = 0;
		adc12_0inte_set.bit.AD5CIE = 0;
		adc12_0inte_set.bit.AD4CIE = 0;
		adc12_0inte_set.bit.AD3CIE = 0;
		adc12_0inte_set.bit.AD2CIE = 0;
		adc12_0inte_set.bit.AD1CIE = 0;
		adc12_0inte_set.bit.AD0CIE = 0;
		ADC12_0INTE = adc12_0inte_set.reg;

	}
}

#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) ||  defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
/************************************************************************//**
 * @brief   Interrupt function.
 *
 * @param   none
 *
 * @return  none
 ***************************************************************************/
void c17intAd12(void)
{
	ADC12_NINTF_REG		adc12_0intf_set;

	/* Capture interrupt flag */
	adc12_0intf_set.reg = ADC12_0INTF;
	/* Clear Interrupt flag. */
	ADC12_0INTF = adc12_0intf_set.reg;

	/// Check interrupt cause occurrence status.
	if(adc12_0intf_set.bit.AD0CIF == 1)
	{
		if(c17adcIntCount < ADC_BUFF_COUNT)
		{
			// read ADC & save.
		    /**
		     * @todo change
		     */
			adVal[c17adcIntCount] = ADC12_0AD0D;
			c17adcIntCount++;
		}
	   /**
		* @todo add interrupt AD0CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1CIF == 1)
	{
	   /**
		* @todo add interrupt AD1CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2CIF == 1)
	{
	   /**
		* @todo add interrupt AD2CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3CIF == 1)
	{
	   /**
		* @todo add interrupt AD3CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4CIF == 1)
	{
	   /**
		* @todo add interrupt AD4CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD5CIF == 1)
	{
		if(c17adcIntCount < ADC_BUFF_COUNT)
		{
			// read ADC & save.
		    /**
		     * @todo change
		     */
			adVal[c17adcIntCount] = ADC12_0AD5D;
			c17adcIntCount++;
		}
	   /**
		* @todo add interrupt AD5CIF interrupt
		*/
	}

	if(adc12_0intf_set.bit.AD0OVIF == 1)
	{
		// Stop ADC12A
		c17stopAdc();

		// Restart ADC12A
		c17startAdc();
	   /**
		* @todo add interrupt AD0OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1OVIF == 1)
	{
	   /**
		* @todo add interrupt AD1OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2OVIF == 1)
	{
	   /**
		* @todo add interrupt AD2OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3OVIF == 1)
	{
	   /**
		* @todo add interrupt AD3OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4OVIF == 1)
	{
	   /**
		* @todo add interrupt AD4OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD5OVIF == 1)
	{
	   /**
		* @todo add interrupt AD5OVIF interrupt
		*/
	}
}

#elif defined(C17_MCUSEL_M40)
/************************************************************************//**
 * @brief   Interrupt function.
 *
 * @param   none
 *
 * @return  none
 ***************************************************************************/
void c17intAd12(void)
{
	ADC12_NINTF_REG		adc12_0intf_set;

	/* Capture interrupt flag */
	adc12_0intf_set.reg = ADC12_0INTF;
	/* Clear Interrupt flag. */
	ADC12_0INTF = adc12_0intf_set.reg;

	/// Check interrupt cause occurrence status.
	if(adc12_0intf_set.bit.AD0CIF == 1)
	{
		if(c17adcIntCount < ADC_BUFF_COUNT)
		{
			// read ADC & save.
		    /**
		     * @todo change
		     */
			adVal[c17adcIntCount] = ADC12_0AD0D;
			c17adcIntCount++;
		}
	   /**
		* @todo add interrupt AD0CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1CIF == 1)
	{
	   /**
		* @todo add interrupt AD1CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2CIF == 1)
	{
	   /**
		* @todo add interrupt AD2CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3CIF == 1)
	{
	   /**
		* @todo add interrupt AD3CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4CIF == 1)
	{
		if(c17adcIntCount < ADC_BUFF_COUNT)
		{
			// read ADC & save.
		    /**
		     * @todo change
		     */
			adVal[c17adcIntCount] = ADC12_0AD4D;
			c17adcIntCount++;
		}
	   /**
		* @todo add interrupt AD4CIF interrupt
		*/
	}

	if(adc12_0intf_set.bit.AD0OVIF == 1)
	{
		// Stop ADC12A
		c17stopAdc();

		// Restart ADC12A
		c17startAdc();
	   /**
		* @todo add interrupt AD0OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1OVIF == 1)
	{
	   /**
		* @todo add interrupt AD1OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2OVIF == 1)
	{
	   /**
		* @todo add interrupt AD2OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3OVIF == 1)
	{
	   /**
		* @todo add interrupt AD3OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4OVIF == 1)
	{
	   /**
		* @todo add interrupt AD4OVIF interrupt
		*/
	}
}

#elif defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_F63)
/************************************************************************//**
 * @brief   Interrupt function.
 *
 * @param   none
 *
 * @return  none
 ***************************************************************************/
void c17intAd12(void)
{
	ADC12_NINTF_REG		adc12_0intf_set;

	/* Capture interrupt flag */
	adc12_0intf_set.reg = ADC12_0INTF;
	/* Clear Interrupt flag. */
	ADC12_0INTF = adc12_0intf_set.reg;

	/// Check interrupt cause occurrence status.
	if(adc12_0intf_set.bit.AD0CIF == 1)
	{
		if(c17adcIntCount < ADC_BUFF_COUNT)
		{
			// read ADC & save.
		    /**
		     * @todo change
		     */
			adVal[c17adcIntCount] = ADC12_0AD0D;
			c17adcIntCount++;
		}
	   /**
		* @todo add interrupt AD0CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1CIF == 1)
	{
	   /**
		* @todo add interrupt AD1CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2CIF == 1)
	{
	   /**
		* @todo add interrupt AD2CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3CIF == 1)
	{
	   /**
		* @todo add interrupt AD3CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4CIF == 1)
	{
	   /**
		* @todo add interrupt AD4CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD5CIF == 1)
	{
	   /**
		* @todo add interrupt AD5CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD6CIF == 1)
	{
	   /**
		* @todo add interrupt AD6CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD7CIF == 1)
	{
		if(c17adcIntCount < ADC_BUFF_COUNT)
		{
			// read ADC & save.
		    /**
		     * @todo change
		     */
			adVal[c17adcIntCount] = ADC12_0AD7D;
			c17adcIntCount++;
		}
	   /**
		* @todo add interrupt AD7CIF interrupt
		*/
	}

	if(adc12_0intf_set.bit.AD0OVIF == 1)
	{
		// Stop ADC12A
		c17stopAdc();

		// Restart ADC12A
		c17startAdc();
	   /**
		* @todo add interrupt AD0OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1OVIF == 1)
	{
	   /**
		* @todo add interrupt AD1OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2OVIF == 1)
	{
	   /**
		* @todo add interrupt AD2OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3OVIF == 1)
	{
	   /**
		* @todo add interrupt AD3OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4OVIF == 1)
	{
	   /**
		* @todo add interrupt AD4OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD5OVIF == 1)
	{
	   /**
		* @todo add interrupt AD5OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD6OVIF == 1)
	{
	   /**
		* @todo add interrupt AD6OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD7OVIF == 1)
	{
	   /**
		* @todo add interrupt AD7OVIF interrupt
		*/
	}
}

#else
/************************************************************************//**
 * @brief   Interrupt function.
 *
 * @param   none
 *
 * @return  none
 ***************************************************************************/
void c17intAd12(void)
{
	ADC12_NINTF_REG		adc12_0intf_set;

	/* Capture interrupt flag */
	adc12_0intf_set.reg = ADC12_0INTF;
	/* Clear Interrupt flag. */
	ADC12_0INTF = adc12_0intf_set.reg;

	/// Check interrupt cause occurrence status.
	if(adc12_0intf_set.bit.AD0CIF == 1)
	{
		if(c17adcIntCount < ADC_BUFF_COUNT)
		{
			// read ADC & save.
		    /**
		     * @todo change
		     */
			adVal[c17adcIntCount] = ADC12_0AD0D;
			c17adcIntCount++;
		}
	   /**
		* @todo add interrupt AD0CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1CIF == 1)
	{
	   /**
		* @todo add interrupt AD1CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2CIF == 1)
	{
	   /**
		* @todo add interrupt AD2CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3CIF == 1)
	{
	   /**
		* @todo add interrupt AD3CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4CIF == 1)
	{
	   /**
		* @todo add interrupt AD4CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD5CIF == 1)
	{
	   /**
		* @todo add interrupt AD5CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD6CIF == 1)
	{
	   /**
		* @todo add interrupt AD6CIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD7CIF == 1)
	{
	   /**
		* @todo add interrupt AD7CIF interrupt
		*/
	}

	if(adc12_0intf_set.bit.AD0OVIF == 1)
	{
		// Stop ADC12A
		c17stopAdc();

		// Restart ADC12A
		c17startAdc();
	   /**
		* @todo add interrupt AD0OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD1OVIF == 1)
	{
	   /**
		* @todo add interrupt AD1OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD2OVIF == 1)
	{
	   /**
		* @todo add interrupt AD2OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD3OVIF == 1)
	{
	   /**
		* @todo add interrupt AD3OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD4OVIF == 1)
	{
	   /**
		* @todo add interrupt AD4OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD5OVIF == 1)
	{
	   /**
		* @todo add interrupt AD5OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD6OVIF == 1)
	{
	   /**
		* @todo add interrupt AD6OVIF interrupt
		*/
	}
	if(adc12_0intf_set.bit.AD7OVIF == 1)
	{
	   /**
		* @todo add interrupt AD7OVIF interrupt
		*/
	}
}
#endif

#endif

