/************************************************************************//**
 *
 * @file    c17_eepromc.c
 *
 * @brief   This is EEPROMC driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2019. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *          - 19 Jun. 2019    1.20
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_eepromc.h"
#include "c17_t16.h"
#include "c17_init_config.h"

#if defined(C17_PE_EEPROMC)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector21_handler, c17intEepromc);		// EEPROMC interrupt
#endif /* CRT0_LINKED */

/* --- macro definition --- */
#if defined(C17_MCUSEL_F63)
#define CH_MAX_COUNT	3
#endif
#if defined(C17_MCUSEL_M40)
#define CH_MAX_COUNT	4
#endif

/* --- static const variable --- */
#if defined(C17_MCUSEL_F63)
static volatile T16_NINTF_REG* const t16_intf_table[CH_MAX_COUNT] = {(T16_NINTF_REG *)&T16_0INTF,(T16_NINTF_REG *)&T16_1INTF,(T16_NINTF_REG *)&T16_2INTF};
#endif
#if defined(C17_MCUSEL_M40)
static volatile T16_NINTF_REG* const t16_intf_table[CH_MAX_COUNT] = {(T16_NINTF_REG *)&T16_0INTF,(T16_NINTF_REG *)&T16_1INTF,(T16_NINTF_REG *)&T16_2INTF,(T16_NINTF_REG *)&T16_3INTF};
#endif

/* --- global variable --- */
volatile unsigned int eepromAsyncState = 0;		///< Async Write State.
volatile unsigned char eepromDataAsync = 0;		///< Async Write Data.

/* --- function declaration --- */
static void enableWriteEeprom(void);
static void disableWriteEeprom(void);


/************************************************************************//**
 * @brief   This function initializes EEPROMC driver.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initEeprom(void)
{
	EPRCINTE_REG	eprcinte_set;

	/// Stop EEPROMC.
	disableWriteEeprom();

	/// Async_State Write standby.
	eepromAsyncState = C17_EEPROMC_STATE_STDNDBY;

	/// Initialize T16.

	/// Stop T16 CH timer.
	c17stopT16(EEPROMC_T16_CH);

	/**
	 * Clock Source=IOSC(700KHz)
	 * Clock Division = 1/32
	 * Ft = 4msec
	 * TR = 4[msec] * f(700KHz/32) -1 = 87
	 */
	/// Initialize T16 timer.
	c17setT16(EEPROMC_T16_CH, C17_T16_CLKSRC_IOSC, C17_T16_CLKDIV_32, C17_T16_MODE_REPEAT);

	/// Set Reload Value T16
	c17setCountT16(EEPROMC_T16_CH, 87);

	/// T16 Interrupt Disable
	c17disableIntT16(EEPROMC_T16_CH);

	/*EPRCINTE:EEPROMC Interrupt Enable Register */
	{
	    /**
	     * @todo change interrupt.
	     */
		eprcinte_set.bit.ECCERIE = 0;
		eprcinte_set.bit.RXBIE = 0;
		EPRCINTE = eprcinte_set.reg;
	}

	/// Initialize the interrupt.
	ILVEPRC = INTERRUPT_LEVEL_EEPROMC;		/// - Set interrupt level.
}


/************************************************************************//**
 * @brief   This function start EEPROMC.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void enableWriteEeprom(void)
{
	EPRCCTL0_REG	eprcctl0_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/* EPRCCTL0: EPRCCTL0 Register */
	{
		eprcctl0_set.reg = EPRCCTL0;
		eprcctl0_set.bit.EP_WMODE = 1;		// Enable Write mode.
		EPRCCTL0 = eprcctl0_set.reg;
	}

	/* EPRCCTL0: EPRCCTL0 Register */
	{
		eprcctl0_set.reg = EPRCCTL0;
		eprcctl0_set.bit.EP_PWRSET = 1;		// Programming power ON.
		EPRCCTL0 = eprcctl0_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();

	/// T16 Interrupt Disable
	c17disableIntT16(EEPROMC_T16_CH);

	/// Start T16 timer.
	c17startT16(EEPROMC_T16_CH);

	/// Check Transmit buffer empty interrupt.
	while (t16_intf_table[EEPROMC_T16_CH]->bit.UFIF == 0) {
		/// Wait underflow Interrupt.
		/**
		 * @todo Add time-out processing as needed.
		 */
	}

	/// Stop T16 timer.
	c17stopT16(EEPROMC_T16_CH);

}


/************************************************************************//**
 * @brief   This function stop EEPROMC.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void disableWriteEeprom(void)
{
	EPRCCTL0_REG	eprcctl0_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/* EPRCCTL0: EPRCCTL0 Register */
	{
		eprcctl0_set.reg = EPRCCTL0;
		eprcctl0_set.bit.EP_WMODE = 0;		// Disable Write mode.
		EPRCCTL0 = eprcctl0_set.reg;
	}

	/* EPRCCTL0: EPRCCTL0 Register */
	{
		eprcctl0_set.reg = EPRCCTL0;
		eprcctl0_set.bit.EP_PWRSET = 0;		// Programming power OFF.
		EPRCCTL0 = eprcctl0_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();

}


/************************************************************************//**
 * @brief   This function EEPROMC Read Data.
 *
 * @param   eepromAddress    Read Address
 * @param   eepromData       Read Data
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed(Write in progress).
 ***************************************************************************/
int c17readEeprom(unsigned char eepromAddress, unsigned char *eepromData)
{
	EPRCCTL0_REG	eprcctl0_set;

	unsigned char	*readData = (unsigned char *)EEPROM_ADDRESS;

	/// It doesn't do at all when writing.
	/* EPRCCTL0: EPRCCTL0 Register */
	{
		eprcctl0_set.reg = EPRCCTL0;
	}

	if(eprcctl0_set.bit.EP_WMODE == 1) {
		return EXIT_FAILURE;
	}

	readData += eepromAddress;

	*eepromData = *readData;

	return EXIT_SUCCESS;

}


/************************************************************************//**
 * @brief   This function EEPROMC Write Data.
 *
 * @param   eepromAddress    Write Address
 * @param   eepromData       Write Data
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 * @retval  C17_EEPROMC_ERROR_VERIFY    Verify error.
 ***************************************************************************/
int c17writeEepromSync(unsigned char eepromAddress, unsigned char eepromData)
{
	EPRCCTL1_REG	eprcctl1_set;
	EPRCADR_REG		eprcadr_set;
	EPRCINTF_REG	eprcintf_set;
	EPRCINTE_REG	eprcinte_set;

	unsigned char readData = 0;

	///Write Enable EEPROMC
	enableWriteEeprom();

	/// Disable write-protect.
	c17protectMiscOff();

	/// Set write address.
	/* EPRCCADR: EPRCADR Register */
	{
		eprcadr_set.bit.EP_ADDR = eepromAddress;		// Set address.
		EPRCADR = eprcadr_set.reg;
	}

	/// Set write data.
	EPRCWDAT = eepromData;


	/* EPRCINTF:EEPROMC Interrupt Flag Register */
	{
		eprcintf_set.reg = 0;
		eprcintf_set.bit.ECCERIF = 1;
		eprcintf_set.bit.RXBIF = 1;
		EPRCINTF = eprcintf_set.reg;		///- Clear interrupt flag.
	}

	/*EPRCINTE:EEPROMC Interrupt Enable Register */
	{
	    /**
	     * @todo change interrupt.
	     */
		eprcinte_set.bit.ECCERIE = 0;
		eprcinte_set.bit.RXBIE = 0;
		EPRCINTE = eprcinte_set.reg;
	}

	/* EPRCCTL1: EPRCCTL1 Register */
	{
		eprcctl1_set.bit.EP_CK = 1;
		EPRCCTL1 = eprcctl1_set.reg;
	}


	/* EPRCINTF:EEPROMC Interrupt Flag Register */
	{
		eprcintf_set.reg = EPRCINTF;
	}

	/// Check Reprogram/read completion interrupt.
	while (eprcintf_set.bit.RXBIF == 0) {
		/**
		 * Wait RXBIF Interrupt for the Reprogram/read completion interrupt.
		 * @todo Add time-out processing as needed.
		 */
		{
			eprcintf_set.reg = EPRCINTF;
		}
	}

	/* EPRCINTF:EEPROMC Interrupt Flag Register */
	{
		eprcintf_set.reg = 0;
		eprcintf_set.bit.RXBIF = 1;
		EPRCINTF = eprcintf_set.reg;		///- Clear interrupt flag.
	}

	/// Enable write-protect.
	c17protectMiscOn();

	/* Check data */
	/// Write Disable EEPROMC
	disableWriteEeprom();
	/// read data.
	c17readEeprom(eepromAddress, &readData);

	/// Check read data.
	if (eepromData != readData) {
		return C17_EEPROMC_ERROR_VERIFY;
	}

	return EXIT_SUCCESS;

}


/************************************************************************//**
 * @brief   This function EEPROMC Write Data.
 *
 * @param   eepromAddress    Write Address
 * @param   eepromData       Write Data
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17writeEepromAsync(unsigned char eepromAddress, unsigned char eepromData)
{

	EPRCCTL1_REG	eprcctl1_set;
	EPRCADR_REG		eprcadr_set;
	EPRCINTF_REG	eprcintf_set;
	EPRCINTE_REG	eprcinte_set;

	///Write Enable EEPROMC
	enableWriteEeprom();

	/// Async_State Writing.
	eepromAsyncState = C17_EEPROMC_STATE_WRITING;

	/// Disable write-protect.
	c17protectMiscOff();

	/* EPRCINTF:EEPROMC Interrupt Flag Register */
	{
		eprcintf_set.reg = 0;
		eprcintf_set.bit.ECCERIF = 1;
		eprcintf_set.bit.RXBIF = 1;
		EPRCINTF = eprcintf_set.reg;		///- Clear interrupt flag.
	}

	/*EPRCINTE:EEPROMC Interrupt Enable Register */
	{
	    /**
	     * @todo change interrupt.
	     */
		eprcinte_set.bit.ECCERIE = 0;
		eprcinte_set.bit.RXBIE = 1;
		EPRCINTE = eprcinte_set.reg;
	}

	/* EPRCCADR: EPRCADR Register */
	{
		eprcadr_set.bit.EP_ADDR = eepromAddress;		// Set address.
		EPRCADR = eprcadr_set.reg;
	}

	eepromDataAsync = eepromData;

	/// Set write data.
	EPRCWDAT = eepromDataAsync;

	/* EPRCCTL1: EPRCCTL1 Register */
	{
		eprcctl1_set.bit.EP_CK = 1;
		EPRCCTL1 = eprcctl1_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();

	return EXIT_SUCCESS;

}


/************************************************************************//**
 * @brief   This function EEPROMC Checking the end of writing.
 *
 * @param   eepromAddress    Write Address
 * @param   eepromData       Write Data
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17writeEepromAsyncCheck(unsigned char eepromAddress)
{
	EPRCINTE_REG	eprcinte_set;

	unsigned char readData = 0;

	switch(eepromAsyncState){
		case C17_EEPROMC_STATE_STDNDBY:

			return EXIT_SUCCESS;
		break;
		case C17_EEPROMC_STATE_WRITING:

			return EXIT_FAILURE;
		break;
		case C17_EEPROMC_STATE_COMPLETE:

			c17protectMiscOff();
			/*EPRCINTE:EEPROMC Interrupt Enable Register */
			{
				eprcinte_set.reg = EPRCINTE;
				eprcinte_set.bit.RXBIE = 0;
				EPRCINTE = eprcinte_set.reg;
			}

			c17protectMiscOn();

			/* Check data */
			/// Write Disable EEPROMC
			disableWriteEeprom();
			/// read data.
			c17readEeprom(eepromAddress, &readData);

			/// Check read data.
			if (eepromDataAsync != readData) {
				return C17_EEPROMC_ERROR_VERIFY;
			}

			eepromAsyncState = C17_EEPROMC_STATE_STDNDBY;

			return EXIT_SUCCESS;
		break;
	}

	return EXIT_SUCCESS;

}


/************************************************************************//**
 * @brief   EEPROMC Interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intEepromc(void)
{
	EPRCINTF_REG	eprcintf_set;

	c17protectMiscOff();
	/* EPRCINTF:EEPROMC Interrupt Flag Register read & clear */
	{
		eprcintf_set.reg = EPRCINTF;
		EPRCINTF = eprcintf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(eprcintf_set.bit.ECCERIF == 1) {
	   /**
		* @todo add interrupt function of EEPROMC
		*/
	}

	/// Check interrupt cause occurrence status.
	if(eprcintf_set.bit.RXBIF == 1) {
	   /**
		* @todo add interrupt function of EEPROMC
		*/

		/// Async_State Write completed.
		eepromAsyncState = C17_EEPROMC_STATE_COMPLETE;
	}
	c17protectMiscOn();
}
#endif

