/************************************************************************//**
 *
 * @file    c17_i2c.c
 *
 * @brief   This is I2C driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Change source code.
 *          - 01 Nov. 2018    1.20
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_i2c.h"
#include "c17_init_config.h"

#if defined(C17_PE_I2C_MASTER) || defined( C17_PE_I2C_SLAVE)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
#if defined(C17_MCUSEL_M01)
C17_INTERRUPT_HANDLER(_vector12_handler, c17intI2c);		// I2C interrupt
#endif
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
C17_INTERRUPT_HANDLER(_vector13_handler, c17intI2c);		// I2C interrupt
#endif
#endif /* CRT0_LINKED */

/* --- static variable --- */
static const unsigned char i2c_div_table[4] = {1,2,4,8};	/* I2C clock division table */

/* --- function declaration --- */
static void setStartCondition(void);
static void setStopCondition(void);
static int waitStartStopCondition(void);
static int sendData(unsigned char i2cData);
static int rcvData(unsigned char *i2cData, unsigned short i2cLastData);
static unsigned char baudrateSet(unsigned short i2cClkSrc, unsigned short i2cClkDiv, unsigned long i2cBrt);

/************************************************************************//**
 * @brief   I2C initialize.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initI2c(void)
{
	/// Stop I2C.
	c17stopI2c();

	if((I2CCH0_INIT & 0x80) == 0x80)
	{

		c17setI2C(((I2CCH0_INIT & 0x40)>>6), ((I2CCH0_INIT & 0x30)>>4), (I2CCH0_INIT & 0x03), I2C_BSP);

		if((I2CCH0_INIT & 0x40) == 0x00)
		{
			c17setSlvAddressI2C(I2C_SLAVE_ADDRESS);
		}
	}

	/// Set interrupt level.
	ILVI2C_0 = INTERRUPT_LEVEL_I2C;				/// - Set interrupt level.
}


/************************************************************************//**
 * @brief  set I2C function.
 *
 * @param   i2cMode
 *  @li @c  C17_I2C_MASTER   I2C master mode.
 *  @li @c  C17_I2C_SLAVE    I2C slave mode.
 * @param   i2cClkSrc        Clock Source.
 *  @li @c  C17_I2C_IOSC
 *  @li @c  C17_I2C_OSC1
 *  @li @c  C17_I2C_OSC3
 *  @li @c  C17_I2C_EXT
 * @param   i2cClkDiv        Clock Division
 *  @li @c  C17_I2C_CLKDIV_1
 *  @li @c  C17_I2C_CLKDIV_2
 *  @li @c  C17_I2C_CLKDIV_4
 *  @li @c  C17_I2C_CLKDIV_8
 * @param   i2cBrt          I2C BPS[Hz]
 *
 * @retval  none
 ***************************************************************************/
void c17setI2C(unsigned short i2cMode, unsigned short i2cClkSrc, unsigned short i2cClkDiv, unsigned long i2cBrt)
{
	I2CNCLK_REG		i2c0clk_set;
	I2CNCTL_REG		i2c0ctl_set;
	I2CNINTE_REG	i2c0inte_set;
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23)
	P0UPMUX1_REG	p0upmux1_set;
	P0UPMUX2_REG	p0upmux2_set;
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	P0UPMUX0_REG	p0upmux0_set;
#endif

	/// Configure operating clock.
	/* I2C0CLK:I2C Ch.0 Clock Control Register */
	{
		i2c0clk_set.bit.DBRUN = 1;					/// - Clock supplied in DEBUG mode.
		i2c0clk_set.bit.CLKDIV = i2cClkDiv;			/// - Select the division ratio(1/1).
		i2c0clk_set.bit.CLKSRC = i2cClkSrc;			/// - Select the clock source of the I2C.
		I2C0CLK = i2c0clk_set.reg;
	}
    I2C0BR =  baudrateSet(i2cClkSrc, i2cClkDiv, i2cBrt);	/// Set transfer rate

#if defined(C17_MCUSEL_M01)
	/// Initialize the I2C input/output function to the ports.
	/*
	 *  P03 : SCL
	 *  P04 : SDA
	 */
    /**
     * @todo change PORT
     */
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P04SEL = 1;		/// - P04 : Enable peripheral I/O function.
		p0modsel_set.bit.P03SEL = 1;		/// - P03 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P04MUX = 1;		/// - P04 : SDA.
		p0fncsel_set.bit.P03MUX = 1;		/// - P03 : SCL.
		P0FNCSEL = p0fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23)
	/// Initialize the I2C input/output function to the ports.
	/*
	 *  P03 : SCL
	 *  P04 : SDA
	 */
    /**
     * @todo change PORT
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P04SEL = 1;		/// - P04 : Enable peripheral I/O function.
		p0modsel_set.bit.P03SEL = 1;		/// - P03 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P04MUX = 1;		/// - P04 : UPMUX.
		p0fncsel_set.bit.P03MUX = 1;		/// - P03 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}
	/// Assign the I2C input/output function to the ports.
	/* P0UPMUX1:P02-03 Universal Port Multiplexer Setting Register */
	{
		p0upmux1_set.reg = P0UPMUX1;
		p0upmux1_set.bit.P03PPFNC = 1;		/// - P03 : SCL.
		p0upmux1_set.bit.P03PERICH = 0;		/// - P03 : Ch.0.
		p0upmux1_set.bit.P03PERISEL	= 1;	/// - P03 : I2C.
		P0UPMUX1 = p0upmux1_set.reg;
	}
	/// Assign the I2C input/output function to the ports.
	/* P0UPMUX0:P04-05 Universal Port Multiplexer Setting Register */
	{
		p0upmux2_set.reg = P0UPMUX2;
		p0upmux2_set.bit.P04PPFNC = 2;		/// - P04 : SDA.
		p0upmux2_set.bit.P04PERICH= 0;		/// - P04 : Ch.0.
		p0upmux2_set.bit.P04PERISEL	= 1;	/// - P04 : I2C.
		P0UPMUX2 = p0upmux2_set.reg;
	}
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	/// Initialize the I2C input/output function to the ports.
	/*
	 *  P00 : SCL
	 *  P01 : SDA
	 */
    /**
     * @todo change PORT
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P01SEL = 1;		/// - P01 : Enable peripheral I/O function.
		p0modsel_set.bit.P00SEL = 1;		/// - P00 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P01MUX = 1;		/// - P01 : UPMUX.
		p0fncsel_set.bit.P00MUX = 1;		/// - P00 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}
	/// Assign the I2C input/output function to the ports.
	/* P0UPMUX0:P00-01 Universal Port Multiplexer Setting Register */
	{
		p0upmux0_set.reg = P0UPMUX0;
		p0upmux0_set.bit.P01PPFNC = 2;		/// - P01 : SDA.
		p0upmux0_set.bit.P01PERICH= 0;		/// - P01 : Ch.0.
		p0upmux0_set.bit.P01PERISEL	= 1;	/// - P01 : I2C.
		p0upmux0_set.bit.P00PPFNC = 1;		/// - P00 : SCL.
		p0upmux0_set.bit.P00PERICH = 0;		/// - P00 : Ch.0.
		p0upmux0_set.bit.P00PERISEL	= 1;	/// - P00 : I2C.
		P0UPMUX0 = p0upmux0_set.reg;
	}
#endif

	/// Set the following I2C0CTL register.
	/* I2C0CTL:I2C Ch.0 Control Register */
	{
		i2c0ctl_set.reg = I2C0CTL;
		i2c0ctl_set.bit.MST = i2cMode;		/// - Set master mode(Master/Slave).
		I2C0CTL = i2c0ctl_set.reg;
	}

	/// Disable I2C interrupts.
	/*  I2C0INTE:I2C Ch.0 Interrupt Enable Register */
	{
		i2c0inte_set.bit.BYTEENDIE = 0;
		i2c0inte_set.bit.GCIE = 0;
		i2c0inte_set.bit.NACKIE = 0;
		i2c0inte_set.bit.STOPIE = 0;
		i2c0inte_set.bit.STARTIE = 0;
		i2c0inte_set.bit.ERRIE = 0;
		i2c0inte_set.bit.RBFIE = 0;
		i2c0inte_set.bit.TBEIE = 0;
		I2C0INTE = i2c0inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   I2C Slave address.
 *
 * @param   i2cAddress: Slave address
 *
 * @retval  none
 ***************************************************************************/
void c17setSlvAddressI2C(unsigned short i2cAddress)
{
	I2CNMOD_REG		i2c0mod_set;

	/* I2C0MOD: I2C Ch.0 Mode Register */
	{
	    /**
	     * @todo change OADR10,GCEN
	     */
		i2c0mod_set.bit.OADR10 = 0;			/// - Disable response to general call address.
		i2c0mod_set.bit.GCEN = 0;			/// - Set 10/7-bit address mode(0=7bit address mode / 1=10bit address mode).
		I2C0MOD = i2c0mod_set.reg;
	}
	/// - Set slave address(7bit address mode).
	I2C0OADR = i2cAddress;
}


/************************************************************************//**
 * @brief   Start I2C.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startI2c(void)
{
	I2CNCTL_REG		i2c0ctl_set;
	I2CNINTF_REG	i2c0intf_set;

	/// This variable is for dummy reading.
	/// Warning comes out, but there is no problem.
	int dummy_rd;

	///clear flag
	dummy_rd = I2C0RXD;		// RXD dummy read
	/* I2C0INTF:I2C Ch.0 Status and Interrupt Flag Register */
	{
		i2c0intf_set.reg = 0;
		i2c0intf_set.bit.BYTEENDIF = 1;
		i2c0intf_set.bit.GCIF = 1;
		i2c0intf_set.bit.NACKIF = 1;
		i2c0intf_set.bit.STOPIF = 1;
		i2c0intf_set.bit.STARTIF = 1;
		i2c0intf_set.bit.ERRIF = 1;
		I2C0INTF = i2c0intf_set.reg;
	}

	/// Execute software reset.
	/// Enable I2C Ch.0 operations.
	/* I2C0CTL:I2C Ch.0 Control Register */
	{
		i2c0ctl_set.reg = I2C0CTL;
		i2c0ctl_set.bit.SFTRST = 1;
		i2c0ctl_set.bit.MODEN = 0;
		I2C0CTL = i2c0ctl_set.reg;

		i2c0ctl_set.bit.SFTRST = 0;
		i2c0ctl_set.bit.MODEN = 1;
		I2C0CTL = i2c0ctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Stop I2C.
 *
 * @param   none
 *
 * @return  neone
 ***************************************************************************/
void c17stopI2c(void)
{
	I2CNCTL_REG		i2c0ctl_set;

	/// Disable I2C Ch.0 operations.
	/// Execute software reset.
	/* I2C0CTL:I2C Ch.0 Control Register */
	{
		i2c0ctl_set.reg = I2C0CTL;
		i2c0ctl_set.bit.SFTRST = 0;
		i2c0ctl_set.bit.MODEN = 0;
		I2C0CTL = i2c0ctl_set.reg;

		i2c0ctl_set.bit.SFTRST = 1;
		i2c0ctl_set.bit.MODEN = 0;
		I2C0CTL = i2c0ctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   I2C Interrupt enable
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntI2c(void)
{
	I2CNINTE_REG	i2c0inte_set;
	I2CNINTF_REG	i2c0intf_set;

	/* I2C0INTF:I2C Ch.0 Status and Interrupt Flag Register */
	{
		i2c0intf_set.reg = 0;
		i2c0intf_set.bit.BYTEENDIF = 1;
		i2c0intf_set.bit.GCIF = 1;
		i2c0intf_set.bit.NACKIF = 1;
		i2c0intf_set.bit.STOPIF = 1;
		i2c0intf_set.bit.STARTIF = 1;
		i2c0intf_set.bit.ERRIF = 1;
		I2C0INTF = i2c0intf_set.reg;
	}

	/// Enable I2C interrupts.
	/*  I2C0INTE:I2C Ch.0 Interrupt Enable Register */
	{
	    /**
	     * @todo change I2C0INTE
	     */
		i2c0inte_set.bit.BYTEENDIE = 1;
		i2c0inte_set.bit.GCIE = 1;
		i2c0inte_set.bit.NACKIE = 1;
		i2c0inte_set.bit.STOPIE = 1;
		i2c0inte_set.bit.STARTIE = 1;
		i2c0inte_set.bit.ERRIE = 1;
		i2c0inte_set.bit.RBFIE = 1;
		i2c0inte_set.bit.TBEIE = 1;
		I2C0INTE = i2c0inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   I2C Interrupt disable
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntI2c(void)
{
	I2CNINTE_REG	i2c0inte_set;
	/// Disable I2C interrupts.
	/*  I2C0INTE:I2C Ch.0 Interrupt Enable Register */
	{
		i2c0inte_set.bit.BYTEENDIE = 0;
		i2c0inte_set.bit.GCIE = 0;
		i2c0inte_set.bit.NACKIE = 0;
		i2c0inte_set.bit.STOPIE = 0;
		i2c0inte_set.bit.STARTIE = 0;
		i2c0inte_set.bit.ERRIE = 0;
		i2c0inte_set.bit.RBFIE = 0;
		i2c0inte_set.bit.TBEIE = 0;
		I2C0INTE = i2c0inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   I2C transmit data(Master).
 *
 * @param   i2cAddress     Slave address(6 to 0 bits).
 * @param   i2cData        Transmit data.
 * @param   i2cDataSize    Transmit data size.
 *
 * @return  EXIT_SUCCESS        I2C status : OK.
 * @return  C17_I2C_STS_NACK    I2C status : NACK reception.
 ***************************************************************************/
int c17sendI2cMstData(unsigned short i2cAddress, unsigned char *i2cData, int i2cDataSize)
{
	int i;
	unsigned char connect;
	int result;

	/// Set Start condition.
	setStartCondition();

	/// Send slave address(+Write).
	/// slave address(7 to 1 bits) + Read(1)/Write(0)).
	connect = ((i2cAddress << 1) & 0xFE);
	result = sendData(connect);

	if (result != EXIT_SUCCESS) {
		// Set STOP condition.
		setStopCondition();
		return result;
	}

	/// Send data.
	for (i = 0;i < i2cDataSize;i++)
	{
		result = sendData(*i2cData++);
		/// Check result.
		if (result != EXIT_SUCCESS) {
			/// The result is not checked at the last data.
			if (((i + 1) == i2cDataSize) && (result == C17_I2C_STS_NACK)) {
				result = EXIT_SUCCESS;
			}
			// End processing when failing.
			break;
		}
	}

	/// Set STOP condition.
	setStopCondition();

	return result;
}


/************************************************************************//**
 * @brief   I2C receive data(Master).
 *
 * @param   i2cAddress Slave address(6 to 0 bits).
 * @param   i2cData    Transmit data.
 * @param   i2cDataSize    Transmit data size.
 *
 * @return  EXIT_SUCCESS        I2C status : OK.
 * @return  C17_I2C_STS_NACK    I2C status : NACK reception.
 ***************************************************************************/
int c17rcvI2cMstData(unsigned short i2cAddress, unsigned char *i2cData, int i2cDataSize)
{
	int i;
	int nextLast = 0;
	unsigned char connect;
	int result;

	I2CNCTL_REG		i2c0ctl_set;

	if(i2cDataSize == 1){
		/* I2C0CTL:I2C Ch.0 Control Register */
		i2c0ctl_set.reg = I2C0CTL;
		i2c0ctl_set.bit.TXNACK = 1;
		I2C0CTL = i2c0ctl_set.reg;
	}

	/// Set Start condition.
	setStartCondition();

	/// Send slave address(+Read).
	/// slave address(7 to 1 bits) + Read(1)/Write(0)).
	connect = ((i2cAddress << 1) | 0x01);
	result = sendData(connect);
	if ((result != EXIT_SUCCESS) && (result != C17_I2C_STS_RCVBUF_FULL))
	{
		// Set STOP condition.
		setStopCondition();
		return result;
	}

	/// Receive data.
	for (i = 0;i < i2cDataSize;i++)
	{
	    /**
	     * When the next-to-last data is received, send a NACK. (Sample Program)
	     *
	     */
		if(i2cDataSize >= 2){
			if ((i + 2) == i2cDataSize) {
				nextLast = 1;
			} else {
				nextLast = 0;
			}
		}

		// Receive data.
		result = rcvData(i2cData++, nextLast);
		if (result != EXIT_SUCCESS) {
			break;
		}
	}

	/// Set STOP condition.
	setStopCondition();

	return result;
}


/************************************************************************//**
 * @brief	I2C transmit data(Slave).
 *
 * @param	i2cData	    Transmit data.
 * @param	i2cDataSize	Transmit data size.
 *
 * @retval	EXIT_SUCCESS         		I2C status : OK.
 * @retval	C17_I2C_STS_NACK			I2C status : NACK reception.
 * @retval	C17_I2C_STS_STOP_CONDITION	I2C status : STOP condition.
 * @retval	C17_I2C_STT_START_CONDITION	I2C status : START condition.
 ***************************************************************************/
int c17sendI2cSlvData(unsigned char *i2cData, int i2cDataSize)
{
	I2CNINTF_REG	i2c0intf_set;

	int i;
	int condition;
	int result = EXIT_SUCCESS;

	/// Wait Start Condition
	waitStartStopCondition();

	/// Check transmission mode
	{
		i2c0intf_set.reg = I2C0INTF;
	}

	if(i2c0intf_set.bit.TR == 1)
	{
		// Send data.
		for(i = 0;i <= i2cDataSize;i++)
		{
			// Transmit data.
			if(i < i2cDataSize)
			{
				result = sendData(*i2cData++);
			}
			else
			{
				result = sendData(0x00);
			}

			// Send OK.
			if(result == EXIT_SUCCESS)
			{
				continue;
			}

			// Check NACK reception interrupt.
			if(result == C17_I2C_STS_NACK)
			{
				// Wait for a START/STOP condition interrupt.
				condition = waitStartStopCondition();

				// Condition is START condition interrupt and transmission mode(1:Transmit/0:Receive).
				/// Check transmission mode
				{
					i2c0intf_set.reg = I2C0INTF;
				}
				if((condition == C17_I2C_STS_START_CONDITION) && (i2c0intf_set.bit.TR == 1))
				{
					continue;
				}

				// Condition is STOP condition interrupt and All data transmit.
				if(condition == C17_I2C_STS_STOP_CONDITION)
				{
					// send OK.
					result = C17_I2C_STS_STOP_CONDITION;
					break;
				}

				// send error(NACK).
				break;
			}

			// Check STOP condition interrupt.
			if(result == C17_I2C_STS_STOP_CONDITION)
			{
				// send error(STOP).
				break;
			}
		}
	}
	/// Wait Stop Condition
	if(result != C17_I2C_STS_STOP_CONDITION)
	{
		waitStartStopCondition();
	}
	else
	{
		result = EXIT_SUCCESS;
	}
	return result;
}


/************************************************************************//**
 * @brief	I2C receive data(Slave).
 *
 * @param	data	Receive data.
 * @param	size	Receive data size.
 *
 * @retval	EXIT_SUCCESS				I2C status : OK.
 * @retval	C17_I2C_STS_NACK			I2C status : NACK reception.
 ***************************************************************************/
int c17rcvI2cSlvData(unsigned char *i2cData, int i2cDataSize)
{
	I2CNINTF_REG	i2c0intf_set;
	I2CNCTL_REG		i2c0ctl_set;

	int i;
	int nextLast = 0;
	int result = EXIT_SUCCESS;

	if(i2cDataSize == 1){
		/* I2C0CTL:I2C Ch.0 Control Register */
		i2c0ctl_set.reg = I2C0CTL;
		i2c0ctl_set.bit.TXNACK = 1;
		I2C0CTL = i2c0ctl_set.reg;
	}

	/// Wait Start Condition
	waitStartStopCondition();

	/// Check transmission mode
	{
		i2c0intf_set.reg = I2C0INTF;
	}

	if(i2c0intf_set.bit.TR == 0)
	{
		// Receive data.
		for (i = 0;i < i2cDataSize;i++) {
			/// When the next-to-last data is received, send a NACK.
			if(i2cDataSize >= 2)
			{
				if ((i + 2) == i2cDataSize) {
					nextLast = 1;
				} else {
					nextLast = 0;
				}
			}

			result = rcvData(i2cData++, nextLast);
			// Receive NG.
			if (result != EXIT_SUCCESS) {
				break;
			}
		}
	}
	/// Wait Stop Condition
	if(result != C17_I2C_STS_STOP_CONDITION)
	{
		waitStartStopCondition();
	}

	return result;
}


/************************************************************************//**
 * @brief   First, It checks in underflow interruption.
 *
 *          Next, interruption flag is cleared.
 *
 * @param   none
 *
 * @retval none
 ***************************************************************************/
void c17intI2c(void)
{
	I2CNINTF_REG	i2c0intf_set;

	/* Capture interrupt flag */
	i2c0intf_set.reg = I2C0INTF;
	/* Clear Interrupt flag. */
	I2C0INTF = i2c0intf_set.reg;

	/// Check interrupt cause occurrence status.
	if (i2c0intf_set.bit.BYTEENDIF == 1) {
	   /**
		* @todo add interrupt End of transfer interrupt.
		*/
	}
	if (i2c0intf_set.bit.GCIF == 1) {
	   /**
		* @todo add interrupt General call address reception interrupt.
		*/
	}
	if (i2c0intf_set.bit.NACKIF == 1) {
	   /**
		* @todo add interrupt NACK reception interrupt interrupt.
		*/
	}
	if (i2c0intf_set.bit.STOPIF == 1) {
	   /**
		* @todo add interrupt STOP condition interrupt.
		*/
	}
	if (i2c0intf_set.bit.STARTIF == 1) {
	   /**
		* @todo add interrupt STOP condition interrupt.
		*/
	}
	if (i2c0intf_set.bit.ERRIF == 1) {
	   /**
		* @todo add interrupt Error detection interrupt.
		*/
	}
	if (i2c0intf_set.bit.RBFIF == 1) {
	   /**
		* @todo add interrupt Receive buffer full interrupt.
		*/
	}
	if (i2c0intf_set.bit.TBEIF == 1) {
	   /**
		* @todo add interrupt Transmit buffer empty interrupt.
		*/
	}
}


/* ----- static ----- */
/************************************************************************//**
 * @brief   Set Baudrate
 *
 * @param   i2cClkSrc    Clock Source
 * @param   i2cClkDiv    Clock Division
 * @param   i2cBrt       BPS
 *
 * @retval  BRT
 ***************************************************************************/
static unsigned char baudrateSet(unsigned short i2cClkSrc, unsigned short i2cClkDiv, unsigned long i2cBrt)
{
	unsigned long clock,temp;

	/// (BRT = fCLK_I2C / (bps * 2) - 3).
	if((i2cClkSrc == C17_I2C_IOSC) || (i2cClkSrc == C17_I2C_OSC3))
	{
		clock = I2C_CLK / i2c_div_table[i2cClkDiv];
	}
	else
	{
		clock = I2C_CLK;
	}

	temp = clock / (i2cBrt * 2) - 3 ;

	return (unsigned char)temp;
}


/************************************************************************//**
 * @brief   Issue a START condition.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void setStartCondition(void)
{
	I2CNCTL_REG		i2c0ctl_set;
	I2CNINTF_REG	i2c0intf_set;

	// Clear START condition interrupt.
	/* I2C0INTF:I2C Ch.0 Status and Interrupt Flag Register */
	{
		i2c0intf_set.reg = 0;
		i2c0intf_set.bit.STARTIF = 1;
		I2C0INTF = i2c0intf_set.reg;
	}

	// Issue a START condition.
	/* I2C0CTL:I2C Ch.0 Control Register */
	{
		i2c0ctl_set.reg = I2C0CTL;
		i2c0ctl_set.bit.TXSTART = 1;
		I2C0CTL = i2c0ctl_set.reg;
	}

	// Wait Start condition.
	waitStartStopCondition();
}


/************************************************************************//**
 * @brief   Issue a STOP condition.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void setStopCondition(void)
{
	I2CNCTL_REG		i2c0ctl_set;
	I2CNINTF_REG	i2c0intf_set;

	// Clear STOP condition interrupt.
	/* I2C0INTF:I2C Ch.0 Status and Interrupt Flag Register */
	{
		i2c0intf_set.reg = 0;
		i2c0intf_set.bit.STOPIF = 1;
		I2C0INTF = i2c0intf_set.reg;
	}

	// Issue a STOP condition.
	/* I2C0CTL:I2C Ch.0 Control Register */
	{
		i2c0ctl_set.reg = I2C0CTL;
		i2c0ctl_set.bit.TXSTOP = 1;
		I2C0CTL = i2c0ctl_set.reg;
	}

	// Wait STOP condition.
	waitStartStopCondition();
}


/************************************************************************//**
 * @brief   Wait a START/STOP condition.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static int waitStartStopCondition(void)
{
	I2CNINTF_REG	i2c0intf_set;

	int result = EXIT_SUCCESS;

	// Wait Start/Stop condition.
	while (1)
	{
		/**
		 * @todo Add time-out processing as needed.
		 */

		/// Check interrupt cause occurrence status.
		{
			i2c0intf_set.reg = I2C0INTF;
		}

		//  Check START condition interrupt.
		if(i2c0intf_set.bit.STARTIF == 1)
		{
			// Clear START condition interrupt.
			i2c0intf_set.reg = 0;
			i2c0intf_set.bit.STARTIF = 1;
			I2C0INTF = i2c0intf_set.reg;
			result = C17_I2C_STS_START_CONDITION;
			break;
		}

		// Check STOP condition interrupt.
		if(i2c0intf_set.bit.STOPIF == 1)
		{
			// Clear STOP condition interrupt.
			i2c0intf_set.reg = 0;
			i2c0intf_set.bit.STOPIF = 1;
			I2C0INTF = i2c0intf_set.reg;
			result = C17_I2C_STS_STOP_CONDITION;
			break;
		}
	}
	return result;
}


/************************************************************************//**
 * @brief   I2C transmit data write.
 *
 * @param   data    Transmit data(8-bits mode = 0x00-0xff / 7-bits mode = 0x00-0x7f).
 *
 * @retval  EXIT_SUCCESS                    I2C status : OK.
 * @retval  C17_I2C_STS_NACK                I2C status : NACK reception.
 * @retval  C17_I2C_STS_STOP_CONDITION      I2C status : STOP condition reception.
 * @retval  C17_I2C_STS_RCVBUF_FULL         I2C status : Receive buff full.
 ***************************************************************************/
static int sendData(unsigned char i2cData)
{
	I2CNINTF_REG	i2c0intf_set;

	int result = EXIT_SUCCESS;

	/// Send Data.
	I2C0TXD = i2cData;

	/* I2C0INTF: I2C Ch.0 Status and Interrupt Flag Register */
	while(1)
	{
		/**
		 * @todo Add time-out processing as needed.
		 */

		/// Check interrupt cause occurrence status.
		{
			i2c0intf_set.reg = I2C0INTF;
		}

		if(i2c0intf_set.bit.TBEIF == 1)
		{
			break;
		}
		if (i2c0intf_set.bit.RBFIF == 1)
		{
			result = C17_I2C_STS_RCVBUF_FULL;
			break;
		}
		if(i2c0intf_set.bit.NACKIF == 1)
		{
			// Clear NAK interrupt.
			/* I2C0INTF:I2C Ch.0 Status and Interrupt Flag Register */
			{
				i2c0intf_set.reg = 0;
				i2c0intf_set.bit.NACKIF = 1;
				I2C0INTF = i2c0intf_set.reg;
			}
			result = C17_I2C_STS_NACK;
			break;
		}
		if(i2c0intf_set.bit.STOPIF == 1)
		{
			result = C17_I2C_STS_STOP_CONDITION;
			break;
		}
	}
	return result;
}


/************************************************************************//**
 * @brief   I2C receive data read.
 *
 * @param   data        Pointer of place where receive data is stored.
 * @param   nextLast    The next-to-last data flag(STATUS_ENABLE/STATUS_DISABLE).
 *
 * @retval  EXIT_SUCCESS            I2C status : OK.
 * @retval  C17_I2C_STS_NACK        I2C status : NACK reception.
 ***************************************************************************/
static int rcvData(unsigned char *i2cData, unsigned short i2cLastData)
{
	I2CNCTL_REG		i2c0ctl_set;
	I2CNINTF_REG	i2c0intf_set;

	int result = EXIT_SUCCESS;

	/// Check interrupt cause occurrence status.
	{
		i2c0intf_set.reg = I2C0INTF;
	}

	/// Wait receive buff full.
	while(i2c0intf_set.bit.RBFIF == 0)
	{
		/// Wait RBFIF Interrupt for receive buff full.
		{
			i2c0intf_set.reg = I2C0INTF;
		}

		/**
		 * @todo Add time-out processing as needed.
		 */
	}

	/// Check interrupt cause occurrence status.
	{
		i2c0intf_set.reg = I2C0INTF;
	}

	// Clear NAK interrupt.
	/* I2C0INTF:I2C Ch.0 Status and Interrupt Flag Register */
	if (i2c0intf_set.bit.NACKIF == 1)
	{
		i2c0intf_set.reg = 0;
		i2c0intf_set.bit.NACKIF = 1;
		I2C0INTF = i2c0intf_set.reg;
		result = C17_I2C_STS_NACK;
	}

	/// Check interrupt cause occurrence status.
	{
		i2c0intf_set.reg = I2C0INTF;
	}

	if(i2c0intf_set.bit.RBFIF == 1)
	{
		/// When the next-to-last data is received, send a NACK.
		if (i2cLastData == 1)
		{
			/* I2C0CTL:I2C Ch.0 Control Register */
			i2c0ctl_set.reg = I2C0CTL;
			i2c0ctl_set.bit.TXNACK = 1;
			I2C0CTL = i2c0ctl_set.reg;
		}
		/// receive Data.
		*i2cData = I2C0RXD;
	}

	return result;
}

#endif

