/************************************************************************//**
 *
 * @file    c17_port.c
 *
 * @brief   This is Port program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2019. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *          - 11 Sep. 2019    1.20
 *              - Change source code.
 *
 ***************************************************************************/
#include "crt0.h"
#include "c17_misc.h"
#include "c17_port_f63.h"
#include "c17_init_config.h"

#if defined(C17_PE_PORT)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector05_handler, c17intPortInput);		// PPORT interrupt
#endif /* CRT0_LINKED */


/* --- static const variable --- */
static const unsigned short highByteTable[8] = {0x0100,0x0200,0x0400,0x0800,0x1000,0x2000,0x4000,0x8000};
static const unsigned short lowByteTable[8]  = {0x0001,0x0002,0x0004,0x0008,0x0010,0x0020,0x0040,0x0080};

/* --- macro definition --- */
#define HIGHBYTE_BIT_SET(portno)				(highByteTable[portno])			///- portno = 0 to 7
#define LOWBYTE_BIT_SET(portno)				(lowByteTable[portno])			///- portno = 0 to 7


/* --- static  function --- */
static inline void intInputP0x(P0INTF_REG* p0nif_get);
static inline void intInputP1x(P1INTF_REG* p1nif_get);
static inline void intInputP2x(P2INTF_REG* p2nif_get);

static void enableIntPxx(unsigned short portNo, unsigned short portIntEdge, P0INTCTL_REG* portRegister);
static void disableIntPxx(unsigned short portNo, P0INTCTL_REG* portRegister);
static unsigned char getOutputPxx(unsigned short portNo, P0DAT_REG* portRegister);
static void setOutputPxx(unsigned short portNo, unsigned short portData, P0DAT_REG* portRegister);
static void setOutputPxxByte(unsigned char portData, P0DAT_REG* portRegister);
static unsigned char getInputPxx(unsigned short portNo, P0DAT_REG* portRegister);
static void setChattaPxx(unsigned short portNo, unsigned short portChatta, P0CHATEN_REG* portRegister);
static void setPullUpDownPxx(unsigned short portNo, unsigned short portPull, P0RCTL_REG* portRegister);
static void setDirectionPxx(unsigned short portNo, unsigned short portDir, P0IOEN_REG* portRegister);

/* --- global variable --- */

// Interrupt flag.
/**
 * @todo c17portP0xIntInputFlag = Port P0x interrupt flag
 */
volatile unsigned short c17portP0xIntInputFlag = 0;				///< Port P0x interrupt flag.

/************************************************************************//**
 * @brief   Initialization of the PORT Register.
 *
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initPort(void)
{
	PCLK_REG		pclk_set;

	P0INTCTL_REG	p0intctl_set;
	P0IOEN_REG		p0ioen_set;
	P0CHATEN_REG	p0chaten_set;
	P0RCTL_REG		p0rctl_set;
	P0MODSEL_REG	p0modsel_set;

	P1INTCTL_REG	p1intctl_set;
	P1IOEN_REG		p1ioen_set;
	P1CHATEN_REG	p1chaten_set;
	P1RCTL_REG		p1rctl_set;
	P1MODSEL_REG	p1modsel_set;

	PDIOEN_REG		pdioen_set;
	PDRCTL_REG		pdrctl_set;
	PDMODSEL_REG	pdmodsel_set;

	/* P0INTCTL: disable port input interrupt. */
	{
		p0intctl_set.reg = 0;
		p0intctl_set.bit.P07IE = 0;
		p0intctl_set.bit.P06IE = 0;
		p0intctl_set.bit.P05IE = 0;
		p0intctl_set.bit.P04IE = 0;
		p0intctl_set.bit.P03IE = 0;
		p0intctl_set.bit.P02IE = 0;
		p0intctl_set.bit.P01IE = 0;
		p0intctl_set.bit.P00IE = 0;
		P0INTCTL = p0intctl_set.reg;
	}
	/* P1INTCTL: disable port input interrupt. */
	{
		p1intctl_set.reg = 0;
		p1intctl_set.bit.P15IE = 0;
		p1intctl_set.bit.P14IE = 0;
		p1intctl_set.bit.P13IE = 0;
		p1intctl_set.bit.P12IE = 0;
		p1intctl_set.bit.P11IE = 0;
		p1intctl_set.bit.P10IE = 0;

		P1INTCTL = p1intctl_set.reg;
	}

	/// Disable write-protect.
	c17protectMiscOff();

	/* PCLK: P Port Clock Control Register */
	{
      /**
       * @todo Change CLKSRC & CLKDIV
       */
		// When using the chattering filter, configure the PPORT operating clock
		{
			pclk_set.bit.DBRUN = 1;			/// - Clock supplied in DEBUG mode.
			pclk_set.bit.CLKDIV = 4;		/// - Set division ratio for clock source(1/16).
			pclk_set.bit.KRSTCFG = 0;		/// - Configure the key-entry reset function.
			pclk_set.bit.CLKSRC = 0;		/// - Set clock source for chattering filter(IOSC).
			PCLK = pclk_set.reg;
		}
	}

	/// Enable write-protect.
	c17protectMiscOn();

	/* P0CHATEN: P0 Port Chattering Filter Enable Register */
	{
		p0chaten_set.reg = 0;
		p0chaten_set.bit.P07CHATEN = (PORT07_INIT & 0x20) ? 1:0;
		p0chaten_set.bit.P06CHATEN = (PORT06_INIT & 0x20) ? 1:0;
		p0chaten_set.bit.P05CHATEN = (PORT05_INIT & 0x20) ? 1:0;
		p0chaten_set.bit.P04CHATEN = (PORT04_INIT & 0x20) ? 1:0;
		p0chaten_set.bit.P03CHATEN = (PORT03_INIT & 0x20) ? 1:0;
		p0chaten_set.bit.P02CHATEN = (PORT02_INIT & 0x20) ? 1:0;
		p0chaten_set.bit.P01CHATEN = (PORT01_INIT & 0x20) ? 1:0;
		p0chaten_set.bit.P00CHATEN = (PORT00_INIT & 0x20) ? 1:0;
		P0CHATEN = p0chaten_set.reg;
	}
	/* P1CHATEN: P1 Port Chattering Filter Enable Register */
	{
		p1chaten_set.reg = 0;
		p1chaten_set.bit.P15CHATEN = (PORT15_INIT & 0x20) ? 1:0;
		p1chaten_set.bit.P14CHATEN = (PORT14_INIT & 0x20) ? 1:0;
		p1chaten_set.bit.P13CHATEN = (PORT13_INIT & 0x20) ? 1:0;
		p1chaten_set.bit.P12CHATEN = (PORT12_INIT & 0x20) ? 1:0;
		p1chaten_set.bit.P11CHATEN = (PORT11_INIT & 0x20) ? 1:0;
		p1chaten_set.bit.P10CHATEN = (PORT10_INIT & 0x20) ? 1:0;
		P1CHATEN = p1chaten_set.reg;
	}

	/* P0IOEN:P0 Port Enable Register */
	{
		p0ioen_set.bit.P07IEN = (((PORT07_INIT & 0x80) == 0x80) && ((PORT07_INIT & 0x40)==0x00)) ? 1:0;
		p0ioen_set.bit.P06IEN = (((PORT06_INIT & 0x80) == 0x80) && ((PORT06_INIT & 0x40)==0x00)) ? 1:0;
		p0ioen_set.bit.P05IEN = (((PORT05_INIT & 0x80) == 0x80) && ((PORT05_INIT & 0x40)==0x00)) ? 1:0;
		p0ioen_set.bit.P04IEN = (((PORT04_INIT & 0x80) == 0x80) && ((PORT04_INIT & 0x40)==0x00)) ? 1:0;
		p0ioen_set.bit.P03IEN = (((PORT03_INIT & 0x80) == 0x80) && ((PORT03_INIT & 0x40)==0x00)) ? 1:0;
		p0ioen_set.bit.P02IEN = (((PORT02_INIT & 0x80) == 0x80) && ((PORT02_INIT & 0x40)==0x00)) ? 1:0;
		p0ioen_set.bit.P01IEN = (((PORT01_INIT & 0x80) == 0x80) && ((PORT01_INIT & 0x40)==0x00)) ? 1:0;
		p0ioen_set.bit.P00IEN = (((PORT00_INIT & 0x80) == 0x80) && ((PORT00_INIT & 0x40)==0x00)) ? 1:0;

		p0ioen_set.bit.P07OEN = (((PORT07_INIT & 0x80) == 0x80) && ((PORT07_INIT & 0x40)==0x40)) ? 1:0;
		p0ioen_set.bit.P06OEN = (((PORT06_INIT & 0x80) == 0x80) && ((PORT06_INIT & 0x40)==0x40)) ? 1:0;
		p0ioen_set.bit.P05OEN = (((PORT05_INIT & 0x80) == 0x80) && ((PORT05_INIT & 0x40)==0x40)) ? 1:0;
		p0ioen_set.bit.P04OEN = (((PORT04_INIT & 0x80) == 0x80) && ((PORT04_INIT & 0x40)==0x40)) ? 1:0;
		p0ioen_set.bit.P03OEN = (((PORT03_INIT & 0x80) == 0x80) && ((PORT03_INIT & 0x40)==0x40)) ? 1:0;
		p0ioen_set.bit.P02OEN = (((PORT02_INIT & 0x80) == 0x80) && ((PORT02_INIT & 0x40)==0x40)) ? 1:0;
		p0ioen_set.bit.P01OEN = (((PORT01_INIT & 0x80) == 0x80) && ((PORT01_INIT & 0x40)==0x40)) ? 1:0;
		p0ioen_set.bit.P00OEN = (((PORT00_INIT & 0x80) == 0x80) && ((PORT00_INIT & 0x40)==0x40)) ? 1:0;
		P0IOEN = p0ioen_set.reg;
	}
	/* P1IOEN:P1 Port Enable Register */
	{
		p1ioen_set.bit.P15IEN =  (((PORT15_INIT & 0x80) == 0x80) && ((PORT15_INIT & 0x40)==0x00)) ? 1:0;
		p1ioen_set.bit.P14IEN =  (((PORT14_INIT & 0x80) == 0x80) && ((PORT14_INIT & 0x40)==0x00)) ? 1:0;
		p1ioen_set.bit.P13IEN =  (((PORT13_INIT & 0x80) == 0x80) && ((PORT13_INIT & 0x40)==0x00)) ? 1:0;
		p1ioen_set.bit.P12IEN =  (((PORT12_INIT & 0x80) == 0x80) && ((PORT12_INIT & 0x40)==0x00)) ? 1:0;
		p1ioen_set.bit.P11IEN =  (((PORT11_INIT & 0x80) == 0x80) && ((PORT11_INIT & 0x40)==0x00)) ? 1:0;
		p1ioen_set.bit.P10IEN =  (((PORT10_INIT & 0x80) == 0x80) && ((PORT10_INIT & 0x40)==0x00)) ? 1:0;

		p1ioen_set.bit.P15OEN =  (((PORT15_INIT & 0x80) == 0x80) && ((PORT15_INIT & 0x40)==0x40)) ? 1:0;
		p1ioen_set.bit.P14OEN =  (((PORT14_INIT & 0x80) == 0x80) && ((PORT14_INIT & 0x40)==0x40)) ? 1:0;
		p1ioen_set.bit.P13OEN =  (((PORT13_INIT & 0x80) == 0x80) && ((PORT13_INIT & 0x40)==0x40)) ? 1:0;
		p1ioen_set.bit.P12OEN =  (((PORT12_INIT & 0x80) == 0x80) && ((PORT12_INIT & 0x40)==0x40)) ? 1:0;
		p1ioen_set.bit.P11OEN =  (((PORT11_INIT & 0x80) == 0x80) && ((PORT11_INIT & 0x40)==0x40)) ? 1:0;
		p1ioen_set.bit.P10OEN =  (((PORT10_INIT & 0x80) == 0x80) && ((PORT10_INIT & 0x40)==0x40)) ? 1:0;
		P1IOEN = p1ioen_set.reg;
	}
	/* PDIOEN:PD Port Enable Register */
	{
		pdioen_set.bit.PD2IEN = (((PORTD2_INIT & 0x80) == 0x80) && ((PORTD2_INIT & 0x40)==0x00)) ? 1:0;
		pdioen_set.bit.PD1IEN = (((PORTD1_INIT & 0x80) == 0x80) && ((PORTD1_INIT & 0x40)==0x00)) ? 1:0;
		pdioen_set.bit.PD0IEN = (((PORTD0_INIT & 0x80) == 0x80) && ((PORTD0_INIT & 0x40)==0x00)) ? 1:0;

		pdioen_set.bit.PD2OEN = (((PORTD2_INIT & 0x80) == 0x80) && ((PORTD2_INIT & 0x40)==0x40)) ? 1:0;
		pdioen_set.bit.PD1OEN = (((PORTD1_INIT & 0x80) == 0x80) && ((PORTD1_INIT & 0x40)==0x40)) ? 1:0;
		pdioen_set.bit.PD0OEN = (((PORTD0_INIT & 0x80) == 0x80) && ((PORTD0_INIT & 0x40)==0x40)) ? 1:0;
		PDIOEN = pdioen_set.reg;
	}

	/* P0RCTL_REG:P0 Port Pull-up/down Control Register */
	{
		p0rctl_set.bit.P07PDPU = ((PORT07_INIT & 0x03)==0x01) ? 1:0;
		p0rctl_set.bit.P06PDPU = ((PORT06_INIT & 0x03)==0x01) ? 1:0;
		p0rctl_set.bit.P05PDPU = ((PORT05_INIT & 0x03)==0x01) ? 1:0;
		p0rctl_set.bit.P04PDPU = ((PORT04_INIT & 0x03)==0x01) ? 1:0;
		p0rctl_set.bit.P03PDPU = ((PORT03_INIT & 0x03)==0x01) ? 1:0;
		p0rctl_set.bit.P02PDPU = ((PORT02_INIT & 0x03)==0x01) ? 1:0;
		p0rctl_set.bit.P01PDPU = ((PORT01_INIT & 0x03)==0x01) ? 1:0;
		p0rctl_set.bit.P00PDPU = ((PORT00_INIT & 0x03)==0x01) ? 1:0;

		p0rctl_set.bit.P07REN = ((PORT07_INIT & 0x03)==0x00) ? 0:1;
		p0rctl_set.bit.P06REN = ((PORT06_INIT & 0x03)==0x00) ? 0:1;
		p0rctl_set.bit.P05REN = ((PORT05_INIT & 0x03)==0x00) ? 0:1;
		p0rctl_set.bit.P04REN = ((PORT04_INIT & 0x03)==0x00) ? 0:1;
		p0rctl_set.bit.P03REN = ((PORT03_INIT & 0x03)==0x00) ? 0:1;
		p0rctl_set.bit.P02REN = ((PORT02_INIT & 0x03)==0x00) ? 0:1;
		p0rctl_set.bit.P01REN = ((PORT01_INIT & 0x03)==0x00) ? 0:1;
		p0rctl_set.bit.P00REN = ((PORT00_INIT & 0x03)==0x00) ? 0:1;
		P0RCTL = p0rctl_set.reg;
	}
	/* P1RCTL_REG:P1 Port Pull-up/down Control Register */
	{
		p1rctl_set.bit.P15PDPU = ((PORT15_INIT & 0x03)==0x01) ? 1:0;
		p1rctl_set.bit.P14PDPU = ((PORT14_INIT & 0x03)==0x01) ? 1:0;
		p1rctl_set.bit.P13PDPU = ((PORT13_INIT & 0x03)==0x01) ? 1:0;
		p1rctl_set.bit.P12PDPU = ((PORT12_INIT & 0x03)==0x01) ? 1:0;
		p1rctl_set.bit.P11PDPU = ((PORT11_INIT & 0x03)==0x01) ? 1:0;
		p1rctl_set.bit.P10PDPU = ((PORT10_INIT & 0x03)==0x01) ? 1:0;

		p1rctl_set.bit.P15REN = ((PORT15_INIT & 0x03)==0x00) ? 0:1;
		p1rctl_set.bit.P14REN = ((PORT14_INIT & 0x03)==0x00) ? 0:1;
		p1rctl_set.bit.P13REN = ((PORT13_INIT & 0x03)==0x00) ? 0:1;
		p1rctl_set.bit.P12REN = ((PORT12_INIT & 0x03)==0x00) ? 0:1;
		p1rctl_set.bit.P11REN = ((PORT11_INIT & 0x03)==0x00) ? 0:1;
		p1rctl_set.bit.P10REN = ((PORT10_INIT & 0x03)==0x00) ? 0:1;
		P1RCTL = p1rctl_set.reg;
	}
	/* PDRCTL_REG:PD Port Pull-up/down Control Register */
	{
		pdrctl_set.bit.PD2PDPU = ((PORTD2_INIT & 0x03)==0x01) ? 1:0;
		pdrctl_set.bit.PD1PDPU = ((PORTD1_INIT & 0x03)==0x01) ? 1:0;
		pdrctl_set.bit.PD0PDPU = ((PORTD0_INIT & 0x03)==0x01) ? 1:0;

		pdrctl_set.bit.PD2REN = ((PORTD2_INIT & 0x03)==0x00) ? 0:1;
		pdrctl_set.bit.PD1REN = ((PORTD1_INIT & 0x03)==0x00) ? 0:1;
		pdrctl_set.bit.PD0REN = ((PORTD0_INIT & 0x03)==0x00) ? 0:1;
		PDRCTL = pdrctl_set.reg;
	}

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		if(PORT07_INIT & 0x80) p0modsel_set.bit.P07SEL = 0x00;
		if(PORT06_INIT & 0x80) p0modsel_set.bit.P06SEL = 0x00;
		if(PORT05_INIT & 0x80) p0modsel_set.bit.P05SEL = 0x00;
		if(PORT04_INIT & 0x80) p0modsel_set.bit.P04SEL = 0x00;
		if(PORT03_INIT & 0x80) p0modsel_set.bit.P03SEL = 0x00;
		if(PORT02_INIT & 0x80) p0modsel_set.bit.P02SEL = 0x00;
		if(PORT01_INIT & 0x80) p0modsel_set.bit.P01SEL = 0x00;
		if(PORT00_INIT & 0x80) p0modsel_set.bit.P00SEL = 0x00;
		P0MODSEL = p0modsel_set.reg;
	}
	/* P1MODSEL:P1 Port Mode Select Register */
	{
		p1modsel_set.reg = P1MODSEL;
		if(PORT15_INIT & 0x80) p1modsel_set.bit.P15SEL = 0x00;
		if(PORT14_INIT & 0x80) p1modsel_set.bit.P14SEL = 0x00;
		if(PORT13_INIT & 0x80) p1modsel_set.bit.P13SEL = 0x00;
		if(PORT12_INIT & 0x80) p1modsel_set.bit.P12SEL = 0x00;
		if(PORT11_INIT & 0x80) p1modsel_set.bit.P11SEL = 0x00;
		if(PORT10_INIT & 0x80) p1modsel_set.bit.P10SEL = 0x00;
		P1MODSEL = p1modsel_set.reg;
	}
	/* PDMODSEL:PD Port Mode Select Register */
	{
		pdmodsel_set.reg = PDMODSEL;
		if(PORTD2_INIT & 0x80) pdmodsel_set.bit.PD2SEL = 0x00;
		if(PORTD1_INIT & 0x80) pdmodsel_set.bit.PD1SEL = 0x00;
		if(PORTD0_INIT & 0x80) pdmodsel_set.bit.PD0SEL = 0x00;
		PDMODSEL = pdmodsel_set.reg;
	}
	/// Initialize the interrupt.
	ILVPPORT = INTERRUPT_LEVEL_PORT;

}


/************************************************************************//**
 * @brief   PORT input interrupt function.(Register in the vector table.)
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intPortInput(void)
{
	P0INTF_REG	p0nif_get;
	P1INTF_REG	p1nif_get;
	P2INTF_REG	p2nif_get;

	/* P0INTF:P0 Port Interrupt Flag Register read & clear */
	{
		p0nif_get.reg = P0INTF;
		P0INTF = p0nif_get.reg;
	}
	intInputP0x(&p0nif_get);

	/* P1INTF:P1 Port Interrupt Flag Register read & clear */
	{
		p1nif_get.reg = P1INTF;
		P1INTF = p1nif_get.reg;
	}
	intInputP1x(&p1nif_get);

	/* P2INTF:P2 Port Interrupt Flag Register read & clear */
	{
		p2nif_get.reg = P2INTF;
		P2INTF = p2nif_get.reg;
	}
	intInputP2x(&p2nif_get);

}


static inline void intInputP0x(P0INTF_REG* p0nif_get)
{
	/* P0INTF:P0 Port Interrupt Flag Register */
	{
		if(p0nif_get->bit.P00IF == 1)
		{
		   /**
			* @todo add interrupt function of P00
			*/
			c17portP0xIntInputFlag |= 0x01;
		}
		if(p0nif_get->bit.P01IF == 1)
		{
		   /**
			* @todo add interrupt function of P01
			*/
			c17portP0xIntInputFlag |= 0x02;
		}
		if(p0nif_get->bit.P02IF == 1)
		{
		   /**
			* @todo add interrupt function of P02
			*/
			c17portP0xIntInputFlag |= 0x04;
		}
		if(p0nif_get->bit.P03IF == 1)
		{
		   /**
			* @todo add interrupt function of P03
			*/
			c17portP0xIntInputFlag |= 0x08;
		}
		if(p0nif_get->bit.P04IF == 1)
		{
		   /**
			* @todo add interrupt function of P04
			*/
			c17portP0xIntInputFlag |= 0x10;
		}
		if(p0nif_get->bit.P05IF == 1)
		{
		   /**
			* @todo add interrupt function of P05
			*/
			c17portP0xIntInputFlag |= 0x20;
		}
		if(p0nif_get->bit.P06IF == 1)
		{
		   /**
			* @todo add interrupt function of P06
			*/
			c17portP0xIntInputFlag |= 0x40;
		}
		if(p0nif_get->bit.P07IF == 1)
		{
		   /**
			* @todo add interrupt function of P07
			*/
			c17portP0xIntInputFlag |= 0x80;
		}
	}
}


static inline void intInputP1x(P1INTF_REG* p1nif_get)
{
	/* P1INTF:P1 Port Interrupt Flag Register */
	{
		if(p1nif_get->bit.P10IF == 1)
		{
		   /**
			* @todo add interrupt function of P10
			*/
		}
		if(p1nif_get->bit.P11IF == 1)
		{
		   /**
			* @todo add interrupt function of P11
			*/
		}
		if(p1nif_get->bit.P12IF == 1)
		{
		   /**
			* @todo add interrupt function of P12
			*/
		}
		if(p1nif_get->bit.P13IF == 1)
		{
		   /**
			* @todo add interrupt function of P13
			*/
		}
		if(p1nif_get->bit.P14IF == 1)
		{
		   /**
			* @todo add interrupt function of P14
			*/
		}
		if(p1nif_get->bit.P15IF == 1)
		{
		   /**
			* @todo add interrupt function of P15
			*/
		}
	}
}


static inline void intInputP2x(P2INTF_REG* p2nif_get)
{
	/* P2INTF:P2 Port Interrupt Flag Register */
	{
		if(p2nif_get->bit.P20IF == 1)
		{
		   /**
			* @todo add interrupt function of P20
			*/
		}
		if(p2nif_get->bit.P21IF == 1)
		{
		   /**
			* @todo add interrupt function of P21
			*/
		}
	}
}


/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP0x(unsigned short portNo, unsigned short portIntEdge)
{
	P0INTF_REG      p0intf_set;

	if(portNo > 7) {
		return;
	}

	/* P0INTF: Port Interrupt Flag Register */
	{
		p0intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P0INTF = p0intf_set.reg;
	}

	/* P0INTCTL:P0 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P0INTCTL);
	}
}


/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP1x(unsigned short portNo, unsigned short portIntEdge)
{
	P1INTF_REG      p1intf_set;

	if(portNo > 7) {
		return;
	}

	/* P1INTF: Port Interrupt Flag Register */
	{
		p1intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P1INTF = p1intf_set.reg;
	}

	/* P1INTCTL:P1 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P1INTCTL);
	}
}


static void enableIntPxx(unsigned short portNo, unsigned short portIntEdge, P0INTCTL_REG* portRegister)
{
	P0INTCTL_REG	pxintctl_set;

	pxintctl_set.reg = portRegister->reg;

	if(portIntEdge == C17_PORT_INT_EDGE_FALLING) {
		/**
		 * Set PxEDGE
		 * Interrupt will occur at a falling edge.
		 */
		pxintctl_set.reg |= HIGHBYTE_BIT_SET(portNo) ;
	} else {
		/**
		 *  Clear PxEDGE
		 * Interrupt will occur at a rising edge.
		 */
		pxintctl_set.reg &= (0xFFFF ^ HIGHBYTE_BIT_SET(portNo));
	}
	/**
	 * Set PxIE
	 * Enable port input interrupts.
	 */
	pxintctl_set.reg |=  LOWBYTE_BIT_SET(portNo) ;

	portRegister->reg = pxintctl_set.reg;
}


/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP0x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P0INTCTL:P0 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P0INTCTL);
	}
}


/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP1x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P1INTCTL:P1 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P1INTCTL);
	}
}


static void disableIntPxx(unsigned short portNo, P0INTCTL_REG* portRegister)
{
	P0INTCTL_REG	pxintctl_set;

	pxintctl_set.reg = portRegister->reg;
	/**
	 * Set PxIE
	 * Disable port input interrupts.
	 */
	pxintctl_set.reg &= (0xFFFF ^ LOWBYTE_BIT_SET(portNo)) ;

	portRegister->reg = pxintctl_set.reg;
}


/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP0xByte(void)
{
	P0DAT_REG	p0data_set;

	p0data_set.reg = P0DAT;

	// Get output data.
	return p0data_set.byte_reg[1];
}


/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP1xByte(void)
{
	P1DAT_REG	p1data_set;

	p1data_set.reg = P1DAT;

	// Get output data.
	return p1data_set.byte_reg[1];
}


/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputPDxByte(void)
{
	PDDAT_REG	pddata_set;

	pddata_set.reg = PDDAT;

	// Get output data.
	return pddata_set.byte_reg[1];
}


/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP0x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P0DAT: P0 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P0DAT);
	}
}


/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP1x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P1DAT: P1 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P1DAT);
	}
}


/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputPDx(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*PDDAT: PD Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&PDDAT);
	}
}


static unsigned char getOutputPxx(unsigned short portNo, P0DAT_REG* portRegister)
{
	// Get output data.
	unsigned char result = 0;

	/**
	 * GET PxOUT
     *    1 : Output high level from the port pin
     *    0 : Output low level from the port pin
	 *
	 */
	if((portRegister->reg & HIGHBYTE_BIT_SET(portNo)) != 0)
	{
		result = 1;
	}
	return result;
}


/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP0xByte(unsigned char portData)
{
	/* P0DAT :P0 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P0DAT);
	}
}


/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP1xByte(unsigned char portData)
{
	/* P1DAT :P1 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P1DAT);
	}
}


/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputPDxByte(unsigned char portData)
{
	/* PDDAT :PD Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&PDDAT);
	}
}


static void setOutputPxxByte(unsigned char portData, P0DAT_REG* portRegister)
{
	P0DAT_REG	pxdata_set;

	/* PxDAT :Px Port Data Register */
	{
		pxdata_set.reg = portRegister->reg;
		pxdata_set.byte_reg[1] = portData;
		// Set output data.
		portRegister->reg = pxdata_set.reg;
	}
}


/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   dportDataata    output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP0x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P0DAT: P0 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P0DAT);
	}
}


/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP1x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P1DAT: P1 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P1DAT);
	}
}


/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputPDx(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*PDDAT: PD Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&PDDAT);
	}
}


static void setOutputPxx(unsigned short portNo, unsigned short portData, P0DAT_REG* portRegister)
{
	P0DAT_REG	pxdat_set;

	pxdat_set.reg =  portRegister->reg;

	if(portData == 1) {
		/**
		 * Set PxOUT
		 * Output high level from the port pin.
		 */
		pxdat_set.reg |= HIGHBYTE_BIT_SET(portNo) ;
	} else {
		/**
		 *  Clear PxOUT
		 * Output low level from the port pin.
		 */
		pxdat_set.reg &= (0xFFFF ^ HIGHBYTE_BIT_SET(portNo));
	}

	portRegister->reg = pxdat_set.reg;
}


/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP0xByte(void)
{
	P0DAT_REG	p0data_set;

	p0data_set.reg = P0DAT;

	// Get input data.
	return p0data_set.byte_reg[0];
}


/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP1xByte(void)
{
	P1DAT_REG	p1data_set;

	p1data_set.reg = P1DAT;

	// Get input data.
	return p1data_set.byte_reg[0];
}


/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputPDxByte(void)
{
	PDDAT_REG	pddata_set;

	pddata_set.reg = PDDAT;

	// Get input data.
	return pddata_set.byte_reg[0];
}


/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP0x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P0DAT: P0 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P0DAT);
	}
}


/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP1x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P1DAT: P1 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P1DAT);
	}
}


/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputPDx(unsigned short portNo)
{
	if (portNo > 7) {
		return 0;
	}

	/*PDDAT: PD Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&PDDAT);
	}
}


unsigned char getInputPxx(unsigned short portNo, P0DAT_REG* portRegister)
{
	// Get intput data.
	unsigned char result = 0;

	/**
	 * GET PxIN
     *    1 : Output high level from the port pin
     *    0 : Output low level from the port pin
	 *
	 */
	if((portRegister->reg & LOWBYTE_BIT_SET(portNo)) != 0)
	{
		result = 1;
	}

	return result;
}


/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP0x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}

	/* P0IOEN:P0 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P0IOEN);
	}
	/* P0CHATEN: P0 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P0CHATEN);
	}
	/* P0RCTL_REG:P0 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P0RCTL);
	}
}


/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP1x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}

	/* P1IOEN:P1 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P1IOEN);
	}
	/* P1CHATEN: P1 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P1CHATEN);
	}
	/* P1RCTL_REG:P1 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P1RCTL);
	}
}


/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionPDx(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}

	/* PDIOEN:PD Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&PDIOEN);
	}

	/* PDRCTL_REG:PD Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&PDRCTL);
	}
}


static void setDirectionPxx(unsigned short portNo, unsigned short portDir, P0IOEN_REG* portRegister)
{
	P0IOEN_REG	pxioen_set;

	pxioen_set.reg =  portRegister->reg;

	if(portDir == C17_PORT_DIR_OUTPUT){
		/**
		 * Set PxOEN
         *    Enable (Data is output from the port pin.)
    	 */
		pxioen_set.reg |= LOWBYTE_BIT_SET(portNo) ;

		/**
		 * Clear PxIEN
         *    0 (R/W): Disable (Input data is fixed at 0.)
		 *
		 */
		pxioen_set.reg &= (0xFFFF ^ HIGHBYTE_BIT_SET(portNo)) ;
	} else {
		/**
		 * Claer PxOEN
         *     Disable (The port is placed into Hi-Z.)
    	 */
		pxioen_set.reg &= (0xFFFF ^ LOWBYTE_BIT_SET(portNo)) ;

		/**
		 *  Set PxIEN
         *    Enable (The port pin status is input.)
		 *
		 */
		pxioen_set.reg |= HIGHBYTE_BIT_SET(portNo) ;
	}
	portRegister->reg = pxioen_set.reg;

}


static void setChattaPxx(unsigned short portNo, unsigned short portChatta, P0CHATEN_REG* portRegister)
{
	P0CHATEN_REG	pxchaten_set;

	pxchaten_set.reg =  portRegister->reg;

	if (portChatta == 1) {
		/**
		 * Set PxCHATEN
		 * Enable (The chattering filter is used.)
		 */
		pxchaten_set.reg |= LOWBYTE_BIT_SET(portNo) ;
	} else {
		/**
		 * Clear PxCHATEN
		 * Disable (The chattering filter is bypassed.)
		 */
		pxchaten_set.reg &= (0xFFFF ^ LOWBYTE_BIT_SET(portNo));
	}
	portRegister->reg = pxchaten_set.reg;
}


static void setPullUpDownPxx(unsigned short portNo, unsigned short portPull, P0RCTL_REG* portRegister)
{
	P0RCTL_REG	pxrctl_set;

	pxrctl_set.reg =  portRegister->reg;
	switch(portPull)
	{
		case C17_PORT_PULL_DISABLE:
			/**
			 * Clear PxREN
			 * Disable (No pull-up/down control is performed.)
			 *
			 */
			pxrctl_set.reg &=  (0xFFFF ^ LOWBYTE_BIT_SET(portNo));
			break;
		case C17_PORT_PULL_UP:
			/**
			 * Set PxREN
			 * Enable (The built-in pull-up/down resistor is used.)
			 *
			 */
			pxrctl_set.reg |=  LOWBYTE_BIT_SET(portNo);

			/*
			 * Set PxPDPU
			 * Pull-up resistor
			 */
			pxrctl_set.reg |=  HIGHBYTE_BIT_SET(portNo);
			break;
		case C17_PORT_PULL_DOWN:
			/**
			 * Set PxREN
			 * Enable (The built-in pull-up/down resistor is used.)
			 *
			 */
			pxrctl_set.reg |=  LOWBYTE_BIT_SET(portNo);

			/**
			 * Clear PxPDPU
			 * Pull-down resistor
			 */
			pxrctl_set.reg &=  (0xFFFF ^ HIGHBYTE_BIT_SET(portNo));
			break;
	}
	portRegister->reg = pxrctl_set.reg;
}
#endif

