/************************************************************************//**
 *
 * @file    c17_rtclp.c
 *
 * @brief   This is RTCLP Registers setting program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2019. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 19 Jun. 2019    1.10
 *              - Change source code.
 *          - 11 Sep. 2019    1.20
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_rtclp_f63.h"
#include "c17_port_f63.h"
#include "c17_clg.h"
#include "c17_spia.h"
#include "c17_t16.h"
#include "c17_init_config.h"

#if defined(C17_PE_RTCLP)
#ifdef CRT0_LINKED
/* --- interrupt handler (GNU17v3 Only) --- */
C17_INTERRUPT_HANDLER(_vector08_handler, c17intRtclp);
#endif /* CRT0_LINKED */

/* --- global variable --- */
// Interrupt counter.
/**
 * @todo c17rtclpIntCount = Interruption counter.
 */
volatile unsigned int c17rtclpIntCount = 0;		///< Interruption counter.

/**
 * @todo c17swIntCount = Interruption counter.
 */
volatile unsigned int c17swIntCount = 0;		///< Interruption counter.

// Theoretical Regulation.
volatile unsigned long c17rtclpTrCount;			///< The theoretical regulation execution cycle time counter.

/* --- static --- */
static unsigned char numtobcd(unsigned char num);
static unsigned char bcdtonum(unsigned char bcd);

static void assertSpiSlvSel(unsigned short spiSlaveSel);
static void negateSpiSlvSel(void);

/// Transmit data buffer.
static char sndDataRtclp[10];
/// Receive data buffer.
static char rcvDataRtclp[10];

static volatile SPINMOD_REG* const spinmod_table[1] = {(SPINMOD_REG *)&SPI0MOD};


/************************************************************************//**
 * @brief   initRtclp.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17initRtclp(void)
{
	RTCCTL_REG	rtcctl_set;
	RTCSEC_REG	rtcsec_set;
	RTCMIN_REG	rtcmin_set;
	RTCHUR_REG	rtchur_set;
	RTCDAY_REG	rtcday_set;
	RTCMON_REG	rtcmon_set;
	RTCYAR_REG	rtcyar_set;
	RTCWK_REG	rtcwk_set;

	SPINMOD_REG		spinmod_set;

	P2MODSEL_REG	p2modsel_set;
	P2FNCSEL0_REG	p2fncsel_set;

	CLGOSC_REG	clgosc_set;

	/// When OSC1 doesn't start, nothing is done.
	/* CLGOSC:CLG Oscillation Control Register. */
	{
		clgosc_set.reg = CLGOSC;
	}

	if(clgosc_set.bit.OSC1EN == 0) {
		return EXIT_FAILURE;
	}

	/// The variable initialize.
	c17rtclpIntCount = 0;
	c17swIntCount = 0;
	c17rtclpTrCount = 0;

	/// SPI CH 0 Setting
	/// Stop SPIA.
	c17stopSpia(0);

	/// Setting SPIA ch.N.
	/*  SPINMOD:SPIA Ch.N Mode Register */
	{
		spinmod_set.reg = 0;
		spinmod_set.bit.CHLN = 7;				/// - length	8bit
		spinmod_set.bit.PUEN = 1;				/// - pull-up	enable
		/**
		 * @todo Change NOCLKDIV.
		 */
		spinmod_set.bit.NOCLKDIV = 1;			/// - SPICLK0   : 16-bit timer operating clock frequency.
		spinmod_set.bit.LSBFST = 0;				/// - Format	MSB First
		spinmod_set.bit.CPHA = 1;				/// - Phase     : 1.
		spinmod_set.bit.CPOL = 1;				/// - Polarity  : Active Low.
		spinmod_set.bit.MST = 1;				/// - Mode      : Master(1),Slave(0)
		spinmod_table[0]->reg = spinmod_set.reg;
	}

	/// Set interrupt level.
	ILVSPIA_0 = INTERRUPT_LEVEL_SPIA0;

	/// SPIA CH.0 Start.
	c17startSpia(0);

	// SPIA Master.
	/// Initialize T16 CH.1 timer.
	/// Use for the clock generation of SPI CH.0.
	c17setT16(1, C17_T16_CLKSRC_OSC1, C17_T16_CLKDIV_4, C17_T16_MODE_REPEAT);

	/// Stop RTCLP.
	c17stopRtclp();

	/// - RTCLP output function to the ports.
    /**
     * P20=RTC1S
     */

	/* P2MODSEL:P2 Port Mode Select Register */
	/* P2FNCSEL:P2 Port Function Select Register */
	{
		p2modsel_set.reg = 0;
		p2modsel_set.bit.P20SEL = 1;		/// - P20 : Enable peripheral I/O function.
		sndDataRtclp[1] = p2modsel_set.reg;

		p2fncsel_set.reg = 0;
		p2fncsel_set.bit.P20MUX = 0;		/// - P20 : RTC1S.
		sndDataRtclp[2] = p2fncsel_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x3C;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 3);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/// Hour counter set to 24H mode.
	/* RTCCTL: RTC Control Register. */
	{
	   /**
		* @todo change RTC24H.
		*/
		rtcctl_set.reg = 0;
		rtcctl_set.bit.RTC24H = C17_RTCLP_24H;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}

	/// "00/00/00 Sunday 00:00:00" setting.
   /**
	* @todo change default time.
	*/
	{
		rtcsec_set.reg = 0;
		rtcmin_set.reg = 0;
		rtchur_set.reg = 0;
		rtcday_set.reg = 0;
		rtcmon_set.reg = 0;
		rtcyar_set.reg = 0;
		rtcwk_set.reg = 0;
		rtcsec_set.bit.RTCSH = 0;
		rtcsec_set.bit.RTCSL = 0;
		rtcmin_set.bit.RTCMIH = 0;
		rtcmin_set.bit.RTCMIL = 0;
		rtchur_set.bit.RTCAP = 0;
		rtchur_set.bit.RTCHH = 0;
		rtchur_set.bit.RTCHL = 0;
		rtcday_set.bit.RTCDH = 0;
		rtcday_set.bit.RTCDL = 0;
		rtcmon_set.bit.RTCMOH = 0;
		rtcmon_set.bit.RTCMOL = 0;
		rtcyar_set.bit.RTCYH = 0;
		rtcyar_set.bit.RTCYL = 0;
		rtcwk_set.bit.WK = 0;
		sndDataRtclp[1] = rtcsec_set.reg;
		sndDataRtclp[2] = rtcmin_set.reg;
		sndDataRtclp[3] = rtchur_set.reg;
		sndDataRtclp[4] = rtcday_set.reg;
		sndDataRtclp[5] = rtcmon_set.reg;
		sndDataRtclp[6] = rtcyar_set.reg;
		sndDataRtclp[7] = rtcwk_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x09;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 8);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/// Initialize the interrupt.
	ILVRTCLP_0 = INTERRUPT_LEVEL_RTCLP;			/// - Set interrupt level.

	return EXIT_SUCCESS;
}


/************************************************************************//**
 * @brief   start Rtclp.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startRtclp(void)
{
	RTCCTL_REG	rtcctl_set;

	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x80;

		/// Assert SPIA CH.0.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// Negate SPIA CH.0.
		negateSpiSlvSel();
	}

	/* RTCCTL:RTC Control Register */
	{
		rtcctl_set.reg = rcvDataRtclp[0];
		rtcctl_set.bit.RTCRUN = 1;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   stop Rtclp.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopRtclp(void)
{
	RTCCTL_REG	rtcctl_set;

	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x80;

		/// Assert SPIA CH.0.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// Negate SPIA CH.0.
		negateSpiSlvSel();
	}

	/* RTCCTLL:RTC Control Register */
	{
		rtcctl_set.reg = rcvDataRtclp[0];
		rtcctl_set.bit.RTCRUN = 0;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Rtclp interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intRtclp(void)
{
	RTCINTF1_REG	rtcintf1_set;
	RTCINTF2_REG	rtcintf2_set;
#if THEORETICAL_ENABLE
	RTCTRMCTL_REG	rtctrmctl_set;
#endif

	/* RTC Interrupt Flag Register. Interrupt Flag Register read & clear */
	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x90;

		/// slave 0 is selected.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// No slave is selected.
		negateSpiSlvSel();

		rtcintf1_set.reg = rcvDataRtclp[0];
	}

	{
		sndDataRtclp[1] = rtcintf1_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x10;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}


	/* RTC Interrupt Flag Register. Interrupt Flag Register read & clear */
	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x91;

		/// slave 0 is selected.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// No slave is selected.
		negateSpiSlvSel();

		rtcintf2_set.reg = rcvDataRtclp[0];
	}

	{
		sndDataRtclp[1] = rtcintf2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x11;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}

	/// Check interrupt cause occurrence status.
	if(rtcintf2_set.bit.RTCTRMIF == 1)
	{
	   /**
		* @todo add interrupt function of Theoretical regulation completion.
		*/
	}
	if(rtcintf2_set.bit.SW1IF == 1)
	{
	   /**
		* @todo add interrupt function of  Stopwatch 1Hz.
		*/
		c17swIntCount++;
	}
	if(rtcintf2_set.bit.SW10IF == 1)
	{
	   /**
		* @todo add interrupt function of  Stopwatch 10Hz.
		*/
	}
	if(rtcintf2_set.bit.SW100IF == 1)
	{
	   /**
		* @todo add interrupt function of  Stopwatch 100Hz.
		*/
	}
	if(rtcintf2_set.bit.DCNTUFIF == 1)
	{
	   /**
		* @todo add interrupt function of  Underflow.
		*/
	}
	if(rtcintf2_set.bit.ALARMIF == 1)
	{
	   /**
		* @todo add interrupt function of  Alarm.
		*/
	}
	if(rtcintf1_set.bit._1DAYIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-day.
		*/
	}
	if(rtcintf1_set.bit._1HURIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-hour.
		*/
	}
	if(rtcintf1_set.bit._1MINIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-minute.
		*/
	}
	if(rtcintf1_set.bit._1SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-second.
		*/
		// Count up the interrupt counter(1-second).
		c17rtclpIntCount++;

#if THEORETICAL_ENABLE
		// Execute theoretical regulation.
		c17rtclpTrCount++;
		if (c17rtclpTrCount == THEORETICAL_CYCLE_TIME) {

			/* RTCCTL:RTC Control Register.*/
			{
				rtctrmctl_set.bit.RTCTRM = THEORETICAL_TRIM;
				sndDataRtclp[1] = rtctrmctl_set.reg;

				/* Transmit */
				{
					/// Write Address Set.
					sndDataRtclp[0] = 0x01;

					/// slave 0 is selected.
					assertSpiSlvSel(C17_RTCLP_SVL_0);

					/// Transmit data.
					c17sendSpiaData(0, sndDataRtclp, 2);

					/// No slave is selected.
					negateSpiSlvSel();
				}
			}
			c17rtclpTrCount = 0;
		}
#endif
	}
	if(rtcintf1_set.bit._1_2SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/2-second.
		*/

	}
	if(rtcintf1_set.bit._1_4SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/4-second.
		*/
	}
	if(rtcintf1_set.bit._1_8SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/8-second.
		*/
	}
	if(rtcintf1_set.bit._1_32SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/32-second.
		*/
	}
}


/************************************************************************//**
 * @brief   Set Rtclp Date & time.
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17setRtclpDate(ST_RTCLP_BCD* rtcBcd)
{
	RTCCTL_REG	rtcctl_set;
	RTCSEC_REG	rtcsec_set;
	RTCMIN_REG	rtcmin_set;
	RTCHUR_REG	rtchur_set;
	RTCDAY_REG	rtcday_set;
	RTCMON_REG	rtcmon_set;
	RTCYAR_REG	rtcyar_set;
	RTCWK_REG	rtcwk_set;

	/// Check busy.
	rtcctl_set.bit.RTCBSY = 1;

	while (rtcctl_set.bit.RTCBSY == 1) {
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x80;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
		rtcctl_set.reg = rcvDataRtclp[0];
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 1;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/// Set value.
	{
		rtcsec_set.bit.RTCSH = rtcBcd->second >> 4;
		rtcsec_set.bit.RTCSL = rtcBcd->second & 0x0f;
		rtcmin_set.bit.RTCMIH = rtcBcd->minute >> 4;
		rtcmin_set.bit.RTCMIL = rtcBcd->minute & 0x0f;
		rtchur_set.bit.RTCAP = rtcBcd->ampm;
		rtchur_set.bit.RTCHH = rtcBcd->hour >> 4;
		rtchur_set.bit.RTCHL = rtcBcd->hour & 0x0f;
		rtcday_set.bit.RTCDH =  rtcBcd->day >> 4;
		rtcday_set.bit.RTCDL = rtcBcd->day & 0x0f;
		rtcmon_set.bit.RTCMOH = rtcBcd->month >> 4;
		rtcmon_set.bit.RTCMOL = rtcBcd->month & 0x0f;
		rtcyar_set.bit.RTCYH = rtcBcd->year >> 4;
		rtcyar_set.bit.RTCYL = rtcBcd->year & 0x0f;
		rtcwk_set.bit.WK = rtcBcd->week;

		sndDataRtclp[1] = rtcsec_set.reg;
		sndDataRtclp[2] = rtcmin_set.reg;
		sndDataRtclp[3] = rtchur_set.reg;
		sndDataRtclp[4] = rtcday_set.reg;
		sndDataRtclp[5] = rtcmon_set.reg;
		sndDataRtclp[6] = rtcyar_set.reg;
		sndDataRtclp[7] = rtcwk_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x09;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 8);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 0;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Get Rtclp Date & time.
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17getRtclpDate(ST_RTCLP_BCD* rtcBcd)
{
	RTCCTL_REG	rtcctl_set;
	RTCSEC_REG	rtcsec_set;
	RTCMIN_REG	rtcmin_set;
	RTCHUR_REG	rtchur_set;
	RTCDAY_REG	rtcday_set;
	RTCMON_REG	rtcmon_set;
	RTCYAR_REG	rtcyar_set;
	RTCWK_REG	rtcwk_set;

	/// Check busy.
	rtcctl_set.bit.RTCBSY = 1;

	while (rtcctl_set.bit.RTCBSY == 1) {
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x80;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
		rtcctl_set.reg = rcvDataRtclp[0];
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 1;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/// Get value.
	{
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x89;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 7);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}

		rtcsec_set.reg = rcvDataRtclp[0];
		rtcmin_set.reg = rcvDataRtclp[1];
		rtchur_set.reg = rcvDataRtclp[2];
		rtcday_set.reg = rcvDataRtclp[3];
		rtcmon_set.reg = rcvDataRtclp[4];
		rtcyar_set.reg = rcvDataRtclp[5];
		rtcwk_set.reg = rcvDataRtclp[6];

		rtcBcd->second = rtcsec_set.reg;
		rtcBcd->minute = rtcmin_set.reg;
		rtcBcd->hour = rtchur_set.reg;
		rtcBcd->ampm = rtchur_set.bit.RTCAP;
		rtcBcd->day = rtcday_set.reg;
		rtcBcd->month = rtcmon_set.reg;
		rtcBcd->year = rtcyar_set.reg;
		rtcBcd->week = rtcwk_set.reg;
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 0;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Set Rtclp Alarm .
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17setRtclpAlarm(ST_RTCLP_BCD* rtcBcd)
{
	RTCCTL_REG	rtcctl_set;
	RTCALM1_REG	rtcarm1_set;
	RTCALM2_REG	rtcarm2_set;
	RTCALM3_REG	rtcarm3_set;

	/// Check busy.
	rtcctl_set.bit.RTCBSY = 1;

	while (rtcctl_set.bit.RTCBSY == 1) {
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x80;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
		rtcctl_set.reg = rcvDataRtclp[0];
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 1;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/// Set value.
	{
		rtcarm1_set.bit.RTCSHA = rtcBcd->second >> 4;
		rtcarm1_set.bit.RTCSLA = rtcBcd->second & 0x0f;
		rtcarm2_set.bit.RTCMIHA = rtcBcd->minute >> 4;
		rtcarm2_set.bit.RTCMILA = rtcBcd->minute & 0x0f;
		rtcarm3_set.bit.RTCAPA = rtcBcd->ampm;
		rtcarm3_set.bit.RTCHHA = rtcBcd->hour >> 4;
		rtcarm3_set.bit.RTCHLA = rtcBcd->hour & 0x0f;
		RTCALM1 = rtcarm1_set.reg;
		RTCALM2 = rtcarm2_set.reg;
		RTCALM3 = rtcarm3_set.reg;
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 0;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Enable Rtclp interrupt .
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntRtclp(void)
{
	RTCINTF1_REG	rtcintf1_set;
	RTCINTF2_REG	rtcintf2_set;
	RTCINTE1_REG	rtcinte1_set;
	RTCINTE2_REG	rtcinte2_set;

	/// Clear interrupt flag.
	/* RTC Interrupt Flag Register. Interrupt Flag Register clear */
	{
		rtcintf1_set.reg = 0;
		rtcintf1_set.bit._1DAYIF = 1;
		rtcintf1_set.bit._1HURIF = 1;
		rtcintf1_set.bit._1MINIF = 1;
		rtcintf1_set.bit._1SECIF = 1;
		rtcintf1_set.bit._1_2SECIF = 1;
		rtcintf1_set.bit._1_4SECIF = 1;
		rtcintf1_set.bit._1_8SECIF = 1;
		rtcintf1_set.bit._1_32SECIF = 1;
		sndDataRtclp[1] = rtcintf1_set.reg;

		rtcintf2_set.reg = 0;
		rtcintf2_set.bit.RTCTRMIF = 1;
		rtcintf2_set.bit.SW1IF = 0;
		rtcintf2_set.bit.SW10IF = 0;
		rtcintf2_set.bit.SW100IF = 0;
		rtcintf2_set.bit.DCNTUFIF = 1;
		rtcintf2_set.bit.ALARMIF = 1;
		sndDataRtclp[2] = rtcintf2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x10;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 3);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}

	/* RTCINTE:RTC Interrupt Enable Register */
	{
	    /**
	     * @todo Change Interrupt enable.
	     */
		rtcinte1_set.reg = 0;
		rtcinte1_set.bit._1DAYIE = 0;
		rtcinte1_set.bit._1HURIE = 0;
		rtcinte1_set.bit._1MINIE = 0;
		rtcinte1_set.bit._1SECIE = 1;
		rtcinte1_set.bit._1_2SECIE = 0;
		rtcinte1_set.bit._1_4SECIE = 0;
		rtcinte1_set.bit._1_8SECIE = 0;
		rtcinte1_set.bit._1_32SECIE = 0;
		sndDataRtclp[1] = rtcinte1_set.reg;

		rtcinte2_set.reg = 0;
		rtcinte2_set.bit.RTCTRMIE = 0;
		rtcinte2_set.bit.SW1IE = 0;
		rtcinte2_set.bit.SW10IE = 0;
		rtcinte2_set.bit.SW100IE = 0;
		rtcinte2_set.bit.DCNTUFIE = 0;
		rtcinte2_set.bit.ALARMIE = 0;
		sndDataRtclp[2] = rtcinte2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x12;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 3);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Disable Rtclp interrupt .
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntRtclp(void)
{
	RTCINTE1_REG	rtcinte1_set;
	RTCINTE2_REG	rtcinte2_set;

	/// Disable all interrupt.
	/* RTCINTE:RTC Interrupt Enable Register */
	{
		rtcinte1_set.reg = 0;
		rtcinte1_set.bit._1DAYIE = 0;
		rtcinte1_set.bit._1HURIE = 0;
		rtcinte1_set.bit._1MINIE = 0;
		rtcinte1_set.bit._1SECIE = 0;
		rtcinte1_set.bit._1_2SECIE = 0;
		rtcinte1_set.bit._1_4SECIE = 0;
		rtcinte1_set.bit._1_8SECIE = 0;
		rtcinte1_set.bit._1_32SECIE = 0;
		sndDataRtclp[1] = rtcinte1_set.reg;

		rtcinte2_set.reg = 0;
		rtcinte2_set.bit.RTCTRMIE = 0;
		rtcinte2_set.bit.SW1IE = 0;
		rtcinte2_set.bit.SW10IE = 0;
		rtcinte2_set.bit.SW100IE = 0;
		rtcinte2_set.bit.DCNTUFIE = 0;
		rtcinte2_set.bit.ALARMIE = 0;
		sndDataRtclp[2] = rtcinte2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x12;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 3);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief  Set RTCLP mode.
 *
 * @param   rtclpMode        12h or 24h mode.
 *  @li @c   C17_RTCLP_12H   12H mode.
 *  @li @c   C17_RTCLP_24H   24H mode.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17setRtclpMode(unsigned short rtclpMode)
{
	RTCCTL_REG	rtcctl_set;
	RTCHUR_REG	rtchur_set;
	ST_RTCLP_BCD rtcBcd;
	unsigned short currentMode;

	/// Get Current Mode
	currentMode = c17getRtclpMode();

	/// It doesn't do at all for the same mode.
	if(rtclpMode == currentMode){
		return EXIT_SUCCESS;
	}

	/// Get Current time.
	c17getRtclpDate(&rtcBcd);

	/// The set value calculate.
	// 24H mode -> 12H mode.
	if (currentMode == C17_RTCLP_24H) {
		if (rtcBcd.hour == 0) {						// 0o'clock(12AM).
			rtcBcd.hour = 0x12;
			rtcBcd.ampm = C17_RTCLP_AM;
		} else if (rtcBcd.hour == 0x12) {			// 12o'clock(12PM).
			rtcBcd.ampm = C17_RTCLP_PM;
		} else if (rtcBcd.hour >= 0x13) {			// 13 to 23o'clock(01PM to 11PM).
			rtcBcd.hour = numtobcd(bcdtonum(rtcBcd.hour) - 12);
			rtcBcd.ampm = C17_RTCLP_PM;
		} else {									// 1 to 11o'clock(01AM to 11AM).
			rtcBcd.ampm = C17_RTCLP_AM;
		}
	// 12H mode -> 24H mode.
	} else {
		// 12AM(0o'clock).
		if (rtcBcd.ampm == C17_RTCLP_AM && rtcBcd.hour == 0x12) {
			rtcBcd.hour = 0;
		// 01PM to 11PM(13 to 23o'clock).
		} else 	if (rtcBcd.ampm == C17_RTCLP_PM && rtcBcd.hour < 0x12) {
			rtcBcd.hour = numtobcd(bcdtonum(rtcBcd.hour) + 12);
		} else {
			// Non.
		}
	}

	/// Check busy.
	rtcctl_set.bit.RTCBSY = 1;

	while (rtcctl_set.bit.RTCBSY == 1) {
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x80;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
		rtcctl_set.reg = rcvDataRtclp[0];
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 1;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	// Set mode.
	/* RTCCTL: RTC Control Register. */
	{
		rtcctl_set.bit.RTC24H = rtclpMode;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/// Set hour.
	/* RTCHUR: RTC Hour Register. */
	{
		rtchur_set.bit.RTCAP = rtcBcd.ampm;
		rtchur_set.bit.RTCHH = rtcBcd.hour >> 4;
		rtchur_set.bit.RTCHL = rtcBcd.hour & 0x0f;
		sndDataRtclp[1] = rtchur_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x0B;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCLP count-up.
		rtcctl_set.bit.RTCHLD = 0;
		sndDataRtclp[1] = rtcctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x00;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	return EXIT_SUCCESS;
}


/************************************************************************//**
 * @brief  Get RTCLP mode.
 *
 * @param   none
 *
 * @retval  C17_RTCLP_12H    12H mode.
 * @retval  C17_RTCLP_24H    24H mode.
 ***************************************************************************/
unsigned short c17getRtclpMode(void)
{
	RTCCTL_REG	rtcctl_set;

	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x80;

		/// Assert SPIA CH.0.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// Negate SPIA CH.0.
		negateSpiSlvSel();
	}

	rtcctl_set.reg = rcvDataRtclp[0];

	return rtcctl_set.bit.RTC24H;
}


/************************************************************************//**
 * @brief   START SW.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startSw(void)
{
	RTCSWCTL_REG	rtcswctl_set;

	/* RTCSWCTL:RTC Stopwatch Control Register */
	{
		rtcswctl_set.bit.SWRST = 1;
		rtcswctl_set.bit.SWRUN = 1;
		sndDataRtclp[1] = rtcswctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x06;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   STOP SW.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopSw(void)
{
	RTCSWCTL_REG	rtcswctl_set;

	/* RTCSWCTL:RTC Stopwatch Control Register */
	{
		rtcswctl_set.reg = 0;
		rtcswctl_set.bit.SWRUN = 0;
		sndDataRtclp[1] = rtcswctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x06;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Enable SW interrupt.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntSW(void)
{
	RTCINTF2_REG	rtcintf2_set;
	RTCINTE2_REG	rtcinte2_set;

	/// Clear interrupt flag.
	/* RTC Interrupt Flag Register. Interrupt Flag Register clear */
	{
		rtcintf2_set.reg = 0;
		rtcintf2_set.bit.SW1IF = 1;
		rtcintf2_set.bit.SW10IF = 1;
		rtcintf2_set.bit.SW100IF = 1;
		sndDataRtclp[1] = rtcintf2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x11;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}

	/* RTCINTE:RTC Interrupt Enable Register */
	{
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x93;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}

	    /**
	     * @todo Change Interrupt enable.
	     */
		rtcinte2_set.reg = rcvDataRtclp[0];
		rtcinte2_set.bit.SW1IE = 1;
		rtcinte2_set.bit.SW10IE = 0;
		rtcinte2_set.bit.SW100IE = 0;
		sndDataRtclp[1] = rtcinte2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x13;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Disable SW interrupt.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntSW(void)
{
	RTCINTE2_REG	rtcinte2_set;

	/* RTCINTE:RTC Interrupt Enable Register */
	{
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x93;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}

	    /**
	     * @todo Change Interrupt enable.
	     */
		rtcinte2_set.reg = rcvDataRtclp[0];
		rtcinte2_set.bit.SW1IE = 0;
		rtcinte2_set.bit.SW10IE = 0;
		rtcinte2_set.bit.SW100IE = 0;
		sndDataRtclp[1] = rtcinte2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x13;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Read SW count .
 *
 * @param   none
 *
 * @retval  BCD10 & BCD100
 ***************************************************************************/
unsigned short c17readSwCount(void)
{
	RTCSWCNT_REG	rtcswcnt_set[3];

	/// Read RTCSWCTL register consecutively.
	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x87;

		/// Assert SPIA CH.0.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// Negate SPIA CH.0.
		negateSpiSlvSel();
	}
	rtcswcnt_set[0].reg = rcvDataRtclp[0];

	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x87;

		/// Assert SPIA CH.0.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// Negate SPIA CH.0.
		negateSpiSlvSel();
	}
	rtcswcnt_set[1].reg =  rcvDataRtclp[0];

	/* Receive */
	{
		/// Receive Address Set.
		sndDataRtclp[0] = 0x87;

		/// Assert SPIA CH.0.
		assertSpiSlvSel(C17_RTCLP_SVL_0);

		/// Transmit data.
		c17sendSpiaData(0, sndDataRtclp, 1);

		/// Receive data.
		c17rcvSpiaData(0, rcvDataRtclp, 1);

		/// Negate SPIA CH.0.
		negateSpiSlvSel();
	}
	rtcswcnt_set[2].reg =  rcvDataRtclp[0];

	/// Compare read results and return their matched results.
	if(rtcswcnt_set[0].reg == rtcswcnt_set[1].reg){

		return ((rtcswcnt_set[0].bit.BCD10 << 4) + rtcswcnt_set[0].bit.BCD100);
	}
	else if(rtcswcnt_set[1].reg == rtcswcnt_set[2].reg){
		return ((rtcswcnt_set[1].bit.BCD10 << 4) + rtcswcnt_set[2].bit.BCD100);
	}
	else{
		return -1;
	}
	return 0;
}


/************************************************************************//**
 * @brief   Set RTCLP Down Counter.
 *
 * @param   mode                 Mode
 *  @li @c  C17_RTCLP_DCNT_MODE_REPEAT     Repeat.
 *  @li @c  C17_RTCLP_DCNT_MODE_ONESHOT    One shot.
 * @param   reload   Reload Data.
 *
 * @retval  none
 ***************************************************************************/
void c17setRtclpDcnt(unsigned short mode, unsigned short reload)
{

	RTCDCNTCTL_REG	rtcdcntctl_set;
	RTCDCNTMOD_REG	rtcdcntmd_set;
	RTCDCNTRLDL_REG	rtcdcntrldl_set;
	RTCDCNTRLDH_REG	rtcdcntrldh_set;

	/* RTCDCNTCTL:RTC Down Counter Control Register */
	{
		rtcdcntctl_set.reg = 0;
		rtcdcntctl_set.bit.RUN = 0;
		sndDataRtclp[1] = rtcdcntctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x18;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/* RTCDCNTMOD:RTC Down Counter Mode Register */
	{
		rtcdcntmd_set.bit.MODE = 1;
		sndDataRtclp[1] = rtcdcntmd_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x19;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/* RTCDCNTL:RTC Down Counter Lower Data Register */
	/* RTCDCNTH:RTC Down Counter Upper Data Register */
	{
		rtcdcntrldl_set.reg = reload;
		rtcdcntrldh_set.reg = reload >> 8;

		sndDataRtclp[1] = rtcdcntrldl_set.reg;
		sndDataRtclp[2] = rtcdcntrldh_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x1C;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 3);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}

	/* RTCDCNTCTL:RTC Down Counter Control Register */
	{
		rtcdcntctl_set.reg = 0;
		rtcdcntctl_set.bit.PRESET = 1;
		sndDataRtclp[1] = rtcdcntctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x18;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Start RTCLP Down Counter.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startRtclpDcnt(void)
{
	RTCDCNTCTL_REG	rtcdcntctl_set;

	/* RTCDCNTCTL:RTC Down Counter Control Register */
	{
		rtcdcntctl_set.bit.PRESET = 1;
		rtcdcntctl_set.bit.RUN = 1;
		sndDataRtclp[1] = rtcdcntctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x18;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Stop RTCLP Down Counter.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopRtclpDcnt(void)
{
	RTCDCNTCTL_REG	rtcdcntctl_set;

	/* RTCDCNTCTL:RTC Down Counter Control Register */
	{
		rtcdcntctl_set.reg = 0;
		rtcdcntctl_set.bit.RUN = 1;
		sndDataRtclp[1] = rtcdcntctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x18;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Enable Down Counter interrupt.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntRtclpDcnt(void)
{
	RTCINTF2_REG	rtcintf2_set;
	RTCINTE2_REG	rtcinte2_set;

	/// Clear interrupt flag.
	/* RTC Interrupt Flag Register. Interrupt Flag Register clear */
	{
		rtcintf2_set.reg = 0;
		rtcintf2_set.bit.DCNTUFIF = 1;
		sndDataRtclp[1] = rtcintf2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x11;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}

	/* RTCINTE:RTC Interrupt Enable Register */
	{
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x93;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}

		rtcinte2_set.reg = rcvDataRtclp[0];
		rtcinte2_set.bit.DCNTUFIE = 1;
		sndDataRtclp[1] = rtcinte2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x13;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   Disable Down Counter interrupt.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntRtclpDcnt(void)
{
	RTCINTE2_REG	rtcinte2_set;

	/* RTCINTE:RTC Interrupt Enable Register */
	{
		/* Receive */
		{
			/// Receive Address Set.
			sndDataRtclp[0] = 0x93;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 1);

			/// Receive data.
			c17rcvSpiaData(0, rcvDataRtclp, 1);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}

		rtcinte2_set.reg = rcvDataRtclp[0];
		rtcinte2_set.bit.DCNTUFIE = 0;
		sndDataRtclp[1] = rtcinte2_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x13;

			/// slave 0 is selected.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// No slave is selected.
			negateSpiSlvSel();
		}
	}
}


/************************************************************************//**
 * @brief   MCU power down control.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopMcu(void)
{
	PWRCTL_REG	pwrctl_set;

	/// Initialize and start OSC1.
	c17startClgOSC1();								// start OSC1.
	c17switchClgSystemClockOSC1();
	c17stopClgOSC3();								// stop OSC3
	c17stopClgIOSC();								// stop IOSC.

	/* PWRCTL:MCU Power Down Control Register */
	{
		pwrctl_set.bit.MCUPD = 1;
		sndDataRtclp[1] = pwrctl_set.reg;

		/* Transmit */
		{
			/// Write Address Set.
			sndDataRtclp[0] = 0x20;

			/// Assert SPIA CH.0.
			assertSpiSlvSel(C17_RTCLP_SVL_0);

			/// Transmit data.
			c17sendSpiaData(0, sndDataRtclp, 2);

			/// Negate SPIA CH.0.
			negateSpiSlvSel();
		}
	}
}


static unsigned char numtobcd(unsigned char num)
{

	unsigned char bcd;

	bcd = (num / 10) << 4 ;
	bcd += (num  % 10);

	return bcd;
}

static unsigned char bcdtonum(unsigned char bcd)
{
	unsigned char num;

	num = (bcd >> 4) * 10;
	num += (bcd & 0x0f);

	return num;
}

static void assertSpiSlvSel(unsigned short spiSlaveSel)
{
	SPISLVSEL_REG	spislvsel_set;

	/* SPISLVSEL:SPI slave select Register */
	{
		spislvsel_set.reg = spiSlaveSel;
		SPISLVSEL = spislvsel_set.reg;
	}
}

static void negateSpiSlvSel(void)
{
	SPISLVSEL_REG	spislvsel_set;

	/* SPISLVSEL:SPI slave select Register */
	{
		spislvsel_set.reg = 0;
		SPISLVSEL = spislvsel_set.reg;
	}
}
#endif

