/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is I2C sample program for S1C17F63 demonstration.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @main page   S1C17F63 I2C
 *
 * @section     intro Introduction
 * Sample code transmit data using I2C.
 * The code receive data using I2C.
 *
 * @subsection  step1   Step 1: Start I2C.
 *                      Start I2C.
 *                      Initialize I2C.
 *                      Transfer rate =  400000bps.
 * @subsection  step2   Step 2: Transmit data.
 *                      Slave address : 0x2A(0101010).
 *                      Transmit "A" to "Z".
 * @subsection  step3   Step 3: Receive data.
 *                      Slave address : 0x2A(1010101).
 *                      Receive data from I2C port.
 * @subsection  step4   Step 4: Compare receive data with the transmission data.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17f63_sample" project.
 *
 *******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_init_config.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_i2c.h"

#if defined(C17_PE_I2C_MASTER) || defined(C17_PE_I2C_SLAVE)
/* --- definition --- */

/// Slave address.
#define SAMPLE_SLAVE_ADDRESS			(0x2A)				///< Slave address.



/* --- global variable --- */

/// Transmit data buffer.
static unsigned char sndData[27] =
{
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
	'\0'
};

/// Receive data buffer.
static unsigned char rcvData[27];


/* --- function declaration --- */
#ifdef C17_PE_I2C_MASTER
static int runMaster(void);
#else
static int runSlave(void);
#endif

int main_i2c(void);

/************************************************************************//**
 * main function.
 *
 * @brief	Send Data & Receive.
 *
 *
 * @return 	EXIT_SUCCESS	Success.
 * @return 	EXIT_FAILURE	Failed.
 ***************************************************************************/
int main_i2c(void)
{
	int i;
	int result;

	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();
	c17stopClgOSC3();

	/// start OSC3(internal oscillator)
	c17startClgIntOSC3(C17_CLG_OSC3_16MHZ);

	/// the system clock set in OSC3.
	c17switchClgSystemClockOSC3();
	c17stopClgIOSC();


	// Clear receive data buffer.
	for (i = 0;i < sizeof(rcvData);i++) {
		rcvData[i] = 0xAA;
	}

	/* <<< I2C demonstration start >>> */

	/// Initialize I2C.
	c17initI2c();

	/// I2C Start.
	c17startI2c();

#ifdef C17_PE_I2C_MASTER
	// I2C Master.
	result = runMaster();
#else
	// I2C Slave.
	result = runSlave();
#endif

	/// Stop I2C.
	c17stopI2c();

	/* <<< I2C demonstration finish >>> */

	return result;
}


#ifdef C17_PE_I2C_MASTER
/************************************************************************//**
 * runMaster function.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int runMaster(void)
{
	int i;
	/// Transmit data.
	if(c17sendI2cMstData(SAMPLE_SLAVE_ADDRESS, sndData, sizeof(sndData)) != EXIT_SUCCESS)
	{
		// Transmit data : NG
		asm("nop");
		return EXIT_FAILURE;
	}

	/// Receive data.
	if(c17rcvI2cMstData(SAMPLE_SLAVE_ADDRESS, rcvData, sizeof(rcvData)) != EXIT_SUCCESS) {
		// Receive data : NG
		asm("nop");
		return EXIT_FAILURE;
	}

	// Check receive data buffer.
	for (i = 0;i < sizeof(rcvData);i++) {
		if (rcvData[i] != sndData[i]) {
			asm("nop");
			return EXIT_FAILURE;
		}
	}
	return EXIT_SUCCESS;
}
#else
/************************************************************************//**
 * runSlave function.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int runSlave(void)
{
	int i;

	/// Receive data.
	if(c17rcvI2cSlvData(rcvData, sizeof(rcvData)) != EXIT_SUCCESS)
	{
		/// Check on receive result.
		// Receive data : NG
		asm("nop");
		return EXIT_FAILURE;
	}

	/// Transmit data.
	if(c17sendI2cSlvData(sndData, sizeof(sndData)) != EXIT_SUCCESS)
	{
		/// Check on receive result.
		// Receive data : NG
		asm("nop");
		return EXIT_FAILURE;
	}

	/// Stop I2C.
	c17stopI2c();


	// Check receive data buffer.
	for (i = 0; i < sizeof(rcvData); i++){
		if (rcvData[i] != sndData[i]) {
			asm("nop");
			return EXIT_FAILURE;
		}
	}

	return EXIT_SUCCESS;
}
#endif

#endif
