/************************************************************************//**
 *
 * @file    clg_f63_reg.h
 *
 * @brief   Definition of the Clock Generator(CLG) Register for S1C17F63.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018-2019. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 19 Jun. 2019    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef CLG_REG_H__
#define CLG_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// CLG System Clock Control Register
typedef union CLGSCLK_REG_TAG {
    unsigned short  reg;
    struct CLGSCLK_REG_BIT_TAG {
        /**
         * These bits select the SYSCLK clock source.
         * .CLKSRC[1:0]
         *  These bits select the SYSCLK clock source.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .CLKDIV[1:0]
         *  These bits set the division ratio of the clock source to determine the SYSCLK frequency.
         * +---------------+----------------------------------------+
         * |               |                CLKSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  CLKDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |  1/2    |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |  1/1    |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits set the division ratio of the clock source to determine the SYSCLK frequency.
         * .CLKSRC[1:0]
         *  These bits select the SYSCLK clock source.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .CLKDIV[1:0]
         *  These bits set the division ratio of the clock source to determine the SYSCLK frequency.
         * +---------------+----------------------------------------+
         * |               |                CLKSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  CLKDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |  1/2    |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |  1/1    |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * These bits select the SYSCLK clock source for resetting the CLGSCLK.CLKSRC[1:0] bits at wake-up.
         * .WUPSRC[1:0]
         *  These bits select the SYSCLK clock source for resetting the CLGSCLK.CLKSRC[1:0] bits at wake-up.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input
         *  at wake-up. However, this setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .WUPDIV[1:0]
         *  These bits select the SYSCLK division ratio for resetting the CLGSCLK.CLKDIV[1:0] bits at wake-up.
         *  This setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         * +---------------+----------------------------------------+
         * |               |                WUPSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  WUPDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |   1/2   |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |   1/1   |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      WUPSRC      : 2;
        unsigned short      reserved_3  : 2;    ///< Reserved.
        /**
         * These bits select the SYSCLK division ratio for resetting the CLGSCLK.CLKDIV[1:0] bits at wake-up.
         * .WUPSRC[1:0]
         *  These bits select the SYSCLK clock source for resetting the CLGSCLK.CLKSRC[1:0] bits at wake-up.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input
         *  at wake-up. However, this setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .WUPDIV[1:0]
         *  These bits select the SYSCLK division ratio for resetting the CLGSCLK.CLKDIV[1:0] bits at wake-up.
         *  This setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         * +---------------+----------------------------------------+
         * |               |                WUPSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  WUPDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |   1/2   |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |   1/1   |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      WUPDIV      : 2;
        unsigned short      reserved_4  : 1;    ///< Reserved.
        /**
         * This bit enables the SYSCLK switching function at wake-up.
         * .WUPMD
         *  This bit enables the SYSCLK switching function at wake-up.
         *    1 (R/WP): Enable
         *    0 (R/WP): Disable
         */
        unsigned short      WUPMD       : 1;
    } bit;
    unsigned char   byte_reg[2];
} CLGSCLK_REG;

/// CLG Oscillation Control Register.
typedef union CLGOSC_REG_TAG {
    unsigned short  reg;
    struct CLGOSC_REG_BIT_TAG {
        /**
         * This bit control the clock source(IOSC) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         *
         * .IOSCEN: IOSC oscillator circuit
         */
        unsigned short      IOSCEN      : 1;
        /**
         * This bit control the clock source(OSC1) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         *
         * .OSC1EN: OSC1 oscillator circuit
         */
        unsigned short      OSC1EN      : 1;
        /**
         * This bit control the clock source(OSC3) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         *
         * .OSC3EN: OSC3 oscillator circuit
         */
        unsigned short      OSC3EN      : 1;
        /**
         * This bit control the clock source(EXOSC) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         *
         * .EXOSCEN: EXOSC clock input
         */
        unsigned short      EXOSCEN     : 1;
        unsigned short      reserved_1  : 4;    ///< Reserved.
        /**
         * This bit control the clock source(IOSC) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         *
         * .IOSCSLPC: IOSC oscillator circuit
         */
        unsigned short      IOSCSLPC    : 1;
        /**
         * This bit control the clock source(OSC1) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         *
         * .OSC1SLPC: OSC1 oscillator circuit
         */
        unsigned short      OSC1SLPC    : 1;
        /**
         * This bit control the clock source(OSC3) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         *
         * .OSC3SLPC: OSC3 oscillator circuit
         */
        unsigned short      OSC3SLPC    : 1;
        /**
         * This bit control the clock source(EXOSC) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         *
         * .EXOSCSLPC: EXOSC clock input
         */
        unsigned short      EXOSCSLPC   : 1;
        unsigned short      reserved_2  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGOSC_REG;

/// CLG IOSC Control Register.
typedef union CLGIOSC_REG_TAG {
    unsigned short  reg;
    struct CLGIOSC_REG_BIT_TAG {
        unsigned short      reserved_1  : 4;    ///< Reserved.
        /**
         * This bit controls the IOSCCLK auto-trimming function.
         * .IOSCSTM
         *  This bit controls the IOSCCLK auto-trimming function.
         *    1 (WP): Start trimming
         *    0 (WP): Stop trimming
         *    1 (R): Trimming is executing.
         *    0 (R): Trimming has finished. (Trimming operation inactivated.)
         */
        unsigned short      IOSCSTM     : 1;
        unsigned short      reserved_2  :11;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGIOSC_REG;

/// CLG OSC1 Trimming Register.
typedef union CLGOSC1TRIM_REG_TAG {
    unsigned short  reg;
    struct CLGOSC1TRIM_REG_BIT_TAG {
        /**
         * These bits indicate the adjustment value of the VOSC regulator included in the independent lowpower real-time clock that was set at factory shipment.
         * .XFIB[5:0]
         *  These bits indicate the adjustment value of the VOSC regulator included in the independent lowpower real-time clock that was set at factory shipment.
         */
        unsigned short      XFIB        : 6;
        unsigned short      reserved_1  : 10;    ///< Reserved.

    } bit;
    unsigned char   byte_reg[2];
} CLGOSC1TRIM_REG;

/// CLG OSC3 Control Register.
typedef union CLGOSC3_REG_TAG {
    unsigned short  reg;
    struct CLGOSC3_REG_BIT_TAG {
        /**
         * These bits set the oscillation stabilization waiting time for the OSC3 oscillator circuit.
         * .OSC3WT[2:0]
         *  These bits set the oscillation stabilization waiting time for the OSC3 oscillator circuit.
         * +-------------+---------------+
         * |             | Oscillation   |
         * | OSC3WT[2:0] | stabilization |
         * |             | waiting time  |
         * +-------------+---------------+
         * |     0x7     | 65,536 clocks |
         * +-------------+---------------+
         * |     0x6     | 16,384 clocks |
         * +-------------+---------------+
         * |     0x5     |  4,096 clocks |
         * +-------------+---------------+
         * |     0x4     |  1,024 clocks |
         * +-------------+---------------+
         * |     0x3     |    256 clocks |
         * +-------------+---------------+
         * |     0x2     |     64 clocks |
         * +-------------+---------------+
         * |     0x1     |     16 clocks |
         * +-------------+---------------+
         * |     0x0     |      4 clocks |
         * +-------------+---------------+
         */
        unsigned short      OSC3WT      : 3;
        unsigned short      reserved_1  : 7;    ///< Reserved.
        /**
         * These bits set the oscillation frequency of the OSC3 internal oscillator circuit.
         * .OSC3FQ[2:0]
         *  These bits set the oscillation frequency of the OSC3 internal oscillator circuit.
         * +--------------+----------------------------+
         * | OSC3FQ[1:0]  | OSC3 oscillation frequency |
         * +--------------+----------------------------+
         * |      0x7     |          Reserved          |
         * +--------------+----------------------------+
         * |      0x6     |           16 MHz           |
         * +--------------+----------------------------+
         * |      0x5     |           12 MHz           |
         * +--------------+----------------------------+
         * |      0x4     |            8 MHz           |
         * +--------------+----------------------------+
         * |      0x3     |            4 MHz           |
         * +--------------+----------------------------+
         * |      0x2     |            2 MHz           |
         * +--------------+----------------------------+
         * |      0x1     |            1 MHz           |
         * +--------------+----------------------------+
         * |      0x0     |          500 kHz           |
         * +--------------+----------------------------+
         */
        unsigned short      OSC3FQ      : 3;
        unsigned short      reserved_2  : 3;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGOSC3_REG;

/// CLG Interrupt Flag Register.
typedef union CLGINTF_REG_TAG {
    unsigned short  reg;
    struct CLGINTF_REG_BIT_TAG {
        /**
         * This bit indicate the oscillation stabilization waiting completion interrupt cause occurrence status in each clock source(IOSC).
         *  This bit indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .IOSCSTAIF: IOSC oscillation stabilization waiting completion interrupt
         */
        unsigned short      IOSCSTAIF   : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit indicate the oscillation stabilization waiting completion interrupt cause occurrence status in each clock source(OSC3).
         *  This bit indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .OSC3STAIF: OSC3 oscillation stabilization waiting completion interrupt
         */
        unsigned short      OSC3STAIF   : 1;
        unsigned short      reserved_2  : 1;    ///< Reserved.
        /**
         * This bit indicate the IOSC oscillation auto-trimming completion interrupt.
         *  This bit indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .IOSC oscillation auto-trimming completion interrupt
         */
        unsigned short      IOSCTEDIF   : 1;
        unsigned short      reserved_3  :11;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGINTF_REG;

/// CLG Interrupt Enable Register.
typedef union CLGINTE_REG_TAG {
    unsigned short  reg;
    struct CLGINTE_REG_BIT_TAG {
        /**
         * This bit enable the oscillation stabilization waiting completion interrupt of each clock source(IOSC).
         *  This bit enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .IOSCSTAIE: IOSC oscillation stabilization waiting completion interrupt
         */
        unsigned short      IOSCSTAIE   : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit enable the oscillation stabilization waiting completion interrupt of each clock source(OSC3).
         *  This bit enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .OSC3STAIE: OSC3 oscillation stabilization waiting completion interrupt
         */
        unsigned short      OSC3STAIE   : 1;
        unsigned short      reserved_2  : 1;    ///< Reserved.
        /**
         * This bit enable the IOSC oscillation auto-trimming completion interrupt.
         *  This bit enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .IOSCTEDIE: IOSC oscillation auto-trimming completion interrupt
         */
        unsigned short      IOSCTEDIE   : 1;
        unsigned short      reserved_3  :11;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGINTE_REG;

/// CLG FOUT Control Register 0.
typedef union CLGFOUT0_REG_TAG {
    unsigned short  reg;
    struct CLGFOUT0_REG_BIT_TAG {
        /**
         * This bit controls the FOUT clock external output.
         * .FOUTEN
         * This bit controls the FOUT clock external output.
         *    1 (R/W): Enable external output
         *    0 (R/W): Disable external output
         */
        unsigned short      FOUTEN      : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * These bits select the FOUT clock source.
         * .FOUTSRC[1:0]
         *  These bits select the FOUT clock source.
         * .FOUTDIV[2:0]
         *  These bits set the FOUT clock division ratio.
         * +---------------+-----------------------------------------+
         * |               |                FOUTSRC[1:0]             |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | FOUTDIV[2:0]  +---------+----------+---------+----------+
         * |               | IOSCCLK |  OSC1CLK | OSC3CLK |  SYSCLK  |
         * +---------------+---------+----------+---------+----------+
         * |      0x7      |  1/128  | 1/32,768 |  1/128  | Reserved |
         * +---------------+---------+----------+---------+          +
         * |      0x6      |   1/64  |  1/4,096 |  1/64   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x5      |   1/32  |  1/1,024 |  1/32   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x4      |   1/16  |  1/256   |  1/16   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x3      |   1/8   |   1/8    |  1/8    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x2      |   1/4   |   1/4    |  1/4    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x1      |   1/2   |   1/2    |  1/2    |          |
         * +---------------+---------+----------+---------+----------+
         * |      0x0      |   1/1   |   1/1    |  1/1    |   1/1    |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      FOUTSRC     : 2;
        /**
         * These bits set the FOUT clock division ratio.
         * .FOUTSRC[1:0]
         *  These bits select the FOUT clock source.
         * .FOUTDIV[2:0]
         *  These bits set the FOUT clock division ratio.
         * +---------------+-----------------------------------------+
         * |               |                FOUTSRC[1:0]             |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | FOUTDIV[2:0]  +---------+----------+---------+----------+
         * |               | IOSCCLK |  OSC1CLK | OSC3CLK |  SYSCLK  |
         * +---------------+---------+----------+---------+----------+
         * |      0x7      |  1/128  | 1/32,768 |  1/128  | Reserved |
         * +---------------+---------+----------+---------+          +
         * |      0x6      |   1/64  |  1/4,096 |  1/64   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x5      |   1/32  |  1/1,024 |  1/32   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x4      |   1/16  |  1/256   |  1/16   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x3      |   1/8   |   1/8    |  1/8    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x2      |   1/4   |   1/4    |  1/4    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x1      |   1/2   |   1/2    |  1/2    |          |
         * +---------------+---------+----------+---------+----------+
         * |      0x0      |   1/1   |   1/1    |  1/1    |   1/1    |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      FOUTDIV     : 3;
        unsigned short      reserved_2  : 9;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGFOUT0_REG;

/// CLG Oscillation Frequency Trimming Register.
typedef union CLGTRIM_REG_TAG {
    unsigned short  reg;
    struct CLGTRIM_REG_BIT_TAG {
        /**
         * These bits sets the frequency trimming value for the IOSC internal oscillator circuit.
         * .IOSCAJ[5:0]
         *  These bits sets the frequency trimming value for the IOSC internal oscillator circuit.
         * +--------------+-------------------------------------+
         * | IOSCAJ[5:0]  | IOSC internal oscillation frequency |
         * +--------------+-------------------------------------+
         * |      0x3f    |           High                      |
         * +--------------+-------------------------------------+
         * |      :       |           :                         |
         * +--------------+-------------------------------------+
         * |      0x00    |           Low                       |
         * +--------------+-------------------------------------+
         */
        unsigned short      IOSCAJ      : 6;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits sets the frequency trimming value for the OSC3 internal oscillator circuit.
         * .OSC3AJ[5:0]
         *  These bits sets the frequency trimming value for the OSC3 internal oscillator circuit.
         *  This setting affects all the frequencies from 500 kHz to 16MHz.
         * +--------------+-------------------------------------+
         * | OSC3AJ[5:0]  | OSC3 internal oscillation frequency |
         * +--------------+-------------------------------------+
         * |      0x3f    |           High                      |
         * +--------------+-------------------------------------+
         * |      :       |           :                         |
         * +--------------+-------------------------------------+
         * |      0x00    |           Low                       |
         * +--------------+-------------------------------------+
         */
        unsigned short      OSC3AJ      : 6;
        unsigned short      reserved_2  : 2;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGTRIM_REG;



#ifdef __cplusplus
}
#endif

#endif // CLG_REG_H__
