/************************************************************************//**
 *
 * @file    epdc_reg.h
 *
 * @brief   Definition of the EPD Controller/Driver (EPDC) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018-2019. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 19 Jun. 2019    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef EPDC_REG_H__
#define EPDC_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// EPDC Timing Clock Control Register.
typedef union EPDTIMCLK_REG_TAG {
    unsigned short  reg;
    struct EPDTIMCLK_REG_BIT_TAG {
        /**
         * These bits select the EPDC timing clock source.
         * .CLKSRC[1:0]
         *  These bits select the EPDC timing clock source.
         * .CLKDIV[2:0]
         *  These bits select the division ratio of the EPDC timing clock.
         * +-------------+-------------------------------------------+
         * |             |                 CLKSRC[1:0]               |
         * |             +----------+----------+----------+----------+
         * | CLKDIV[2:0] |    0x0   |    0x1   |    0x2   |    0x3   |
         * |             +----------+----------+----------+----------+
         * |             |   IOSC   |   OSC1   |   OSC3   |   EXOSC  |
         * +-------------+----------+----------+----------+----------+
         * |     0x7     | 1/16,384 |  1/128   | 1/16,384 |    1/1   |
         * +-------------+----------+----------+----------+          +
         * |     0x6     |  1/8,192 |   1/64   |  1/8,192 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x5     |  1/4,096 |   1/32   |  1/4,096 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x4     |  1/2,048 |   1/16   |  1/2,048 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x3     |  1/1,024 |    1/8   |  1/1,024 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x2     |   1/512  |    1/4   |   1/512  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x1     |   1/256  |    1/2   |   1/256  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x0     |   1/128  |    1/1   |   1/128  |          |
         * +-------------+----------+----------+----------+----------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the EPDC timing clock.
         * .CLKSRC[1:0]
         *  These bits select the EPDC timing clock source.
         * .CLKDIV[2:0]
         *  These bits select the division ratio of the EPDC timing clock.
         * +-------------+-------------------------------------------+
         * |             |                 CLKSRC[1:0]               |
         * |             +----------+----------+----------+----------+
         * | CLKDIV[2:0] |    0x0   |    0x1   |    0x2   |    0x3   |
         * |             +----------+----------+----------+----------+
         * |             |   IOSC   |   OSC1   |   OSC3   |   EXOSC  |
         * +-------------+----------+----------+----------+----------+
         * |     0x7     | 1/16,384 |  1/128   | 1/16,384 |    1/1   |
         * +-------------+----------+----------+----------+          +
         * |     0x6     |  1/8,192 |   1/64   |  1/8,192 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x5     |  1/4,096 |   1/32   |  1/4,096 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x4     |  1/2,048 |   1/16   |  1/2,048 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x3     |  1/1,024 |    1/8   |  1/1,024 |          |
         * +-------------+----------+----------+----------+          +
         * |     0x2     |   1/512  |    1/4   |   1/512  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x1     |   1/256  |    1/2   |   1/256  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x0     |   1/128  |    1/1   |   1/128  |          |
         * +-------------+----------+----------+----------+----------+
         */
        unsigned short      CLKDIV      : 3;
        unsigned short      reserved_2  : 1;    ///< Reserved.
        /**
         * This bit sets whether the EPDC timing clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the EPDC timing clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDTIMCLK_REG;

/// EPDC Doubler Clock Control Register.
typedef union EPDDBLCLK_REG_TAG {
    unsigned short  reg;
    struct EPDDBLCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of the doubler in the EPD power supply circuit.
         * .CLKSRC[1:0]
         *  These bits select the EPDC timing clock source.
         * .CLKDIV[2:0]
         *  These bits select the operating clock division ratio of the doubler in the EPD power supply circuit.
         * +-------------+-------------------------------------------+
         * |             |                 CLKSRC[1:0]               |
         * |             +----------+----------+----------+----------+
         * | CLKDIV[2:0] |    0x0   |    0x1   |    0x2   |    0x3   |
         * |             +----------+----------+----------+----------+
         * |             |   IOSC   |   OSC1   |   OSC3   |   EXOSC  |
         * +-------------+----------+----------+----------+----------+
         * |     0x7     | Reserved | Reserved | Reserved |    1/1   |
         * +-------------+          +          +          +          +
         * |     0x6     |          |          |          |          |
         * +-------------+          +          +          +          +
         * |     0x5     |          |          |          |          |
         * +-------------+----------+          +----------+          +
         * |     0x4     |   1/256  |          |   1/256  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x3     |   1/128  |    1/8   |   1/128  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x2     |    1/64  |    1/4   |    1/64  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x1     |    1/32  |    1/2   |    1/32  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x0     |    1/16  |    1/1   |    1/16  |          |
         * +-------------+----------+----------+----------+----------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the operating clock division ratio of the doubler in the EPD power supply circuit.
         * .CLKSRC[1:0]
         *  These bits select the EPDC timing clock source.
         * .CLKDIV[2:0]
         *  These bits select the operating clock division ratio of the doubler in the EPD power supply circuit.
         * +-------------+-------------------------------------------+
         * |             |                 CLKSRC[1:0]               |
         * |             +----------+----------+----------+----------+
         * | CLKDIV[2:0] |    0x0   |    0x1   |    0x2   |    0x3   |
         * |             +----------+----------+----------+----------+
         * |             |   IOSC   |   OSC1   |   OSC3   |   EXOSC  |
         * +-------------+----------+----------+----------+----------+
         * |     0x7     | Reserved | Reserved | Reserved |    1/1   |
         * +-------------+          +          +          +          +
         * |     0x6     |          |          |          |          |
         * +-------------+          +          +          +          +
         * |     0x5     |          |          |          |          |
         * +-------------+----------+          +----------+          +
         * |     0x4     |   1/256  |          |   1/256  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x3     |   1/128  |    1/8   |   1/128  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x2     |    1/64  |    1/4   |    1/64  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x1     |    1/32  |    1/2   |    1/32  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x0     |    1/16  |    1/1   |    1/16  |          |
         * +-------------+----------+----------+----------+----------+
         */
        unsigned short      CLKDIV      : 3;
        unsigned short      reserved_2  : 1;    ///< Reserved.
        /**
         * This bit sets whether the operating clock of the doubler in the EPD power supply circuit is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the EPDC timing clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDDBLCLK_REG;

/// EPDC Booster Clock Control Register.
typedef union EPDBSTCLK_REG_TAG {
    unsigned short  reg;
    struct EPDBSTCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of the booster in the EPD power supply circuit.
         * .CLKSRC[1:0]
         *  These bits select the EPDC timing clock source.
         * .CLKDIV[2:0]
         *  These bits select the operating clock division ratio of the booster in the EPD power supply circuit.
         * +-------------+-------------------------------------------+
         * |             |                 CLKSRC[1:0]               |
         * |             +----------+----------+----------+----------+
         * | CLKDIV[2:0] |    0x0   |    0x1   |    0x2   |    0x3   |
         * |             +----------+----------+----------+----------+
         * |             |   IOSC   |   OSC1   |   OSC3   |   EXOSC  |
         * +-------------+----------+----------+----------+----------+
         * |     0x7     | Reserved | Reserved | Reserved |    1/1   |
         * +-------------+          +          +          +          +
         * |     0x6     |          |          |          |          |
         * +-------------+          +          +          +          +
         * |     0x5     |          |          |          |          |
         * +-------------+----------+          +----------+          +
         * |     0x4     |   1/256  |          |   1/256  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x3     |   1/128  |    1/8   |   1/128  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x2     |    1/64  |    1/4   |    1/64  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x1     |    1/32  |    1/2   |    1/32  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x0     |    1/16  |    1/1   |    1/16  |          |
         * +-------------+----------+----------+----------+----------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the operating clock division ratio of the booster in the EPD power supply circuit.
         * .CLKSRC[1:0]
         *  These bits select the EPDC timing clock source.
         * .CLKDIV[2:0]
         *  These bits select the operating clock division ratio of the booster in the EPD power supply circuit.
         * +-------------+-------------------------------------------+
         * |             |                 CLKSRC[1:0]               |
         * |             +----------+----------+----------+----------+
         * | CLKDIV[2:0] |    0x0   |    0x1   |    0x2   |    0x3   |
         * |             +----------+----------+----------+----------+
         * |             |   IOSC   |   OSC1   |   OSC3   |   EXOSC  |
         * +-------------+----------+----------+----------+----------+
         * |     0x7     | Reserved | Reserved | Reserved |    1/1   |
         * +-------------+          +          +          +          +
         * |     0x6     |          |          |          |          |
         * +-------------+          +          +          +          +
         * |     0x5     |          |          |          |          |
         * +-------------+----------+          +----------+          +
         * |     0x4     |   1/256  |          |   1/256  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x3     |   1/128  |    1/8   |   1/128  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x2     |    1/64  |    1/4   |    1/64  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x1     |    1/32  |    1/2   |    1/32  |          |
         * +-------------+----------+----------+----------+          +
         * |     0x0     |    1/16  |    1/1   |    1/16  |          |
         * +-------------+----------+----------+----------+----------+
         */
        unsigned short      CLKDIV      : 3;
        unsigned short      reserved_2  : 1;    ///< Reserved.
        /**
         * This bit sets whether the operating clock of the booster in the EPD power supply circuit is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the EPDC timing clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDBSTCLK_REG;

/// EPDC Control Register.
typedef union EPDCTL_REG_TAG {
    unsigned short  reg;
    struct EPDCTL_REG_BIT_TAG {
        /**
         * This bit enables the EPDC operations.
         * .MODEN
         *  This bit enables the EPDC operations.
         *    1 (R/W): Enable EPDC operations (The timing clock is supplied.)
         *    0 (R/W): Disable EPDC operations (The timing clock is stopped.)
         */
        unsigned short      MODEN       : 1;
        /**
         * This bit enables the EPD drive output pin discharge operation.
         * .DIS
         *  This bit enables the EPD drive output pin discharge operation.
         *    1 (R/W): Enable discharge operation
         *    0 (R/W): Disable discharge operation
         */
        unsigned short      DIS         : 1;
        unsigned short      reserved_1  :14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDCTL_REG;

/// EPDC power supply Control Register 0.
typedef union EPDPWR0_REG_TAG {
    unsigned short  reg;
    struct EPDPWR0_REG_BIT_TAG {
        /**
         * This bit turns the VE regulator on or off.
         * .VEON
         *  This bit turns the VE regulator on or off.
         *    1 (R/W): VE regulator On
         *    0 (R/W): VE regulator Off
         */
        unsigned short      VEON        : 1;
        /**
         * This bit selects the VE regulator output voltage.
         * .VESEL
         *  This bit selects the VE regulator output voltage (reference voltage for boosting).
         *  Select either VE1 or VE2 to be generated by the VE regulator according to the VDD value.
         *    1 (R/W): VE2
         *    0 (R/W): VE1
         * +--------------------------+-------+---------------------+
         * | Power supply voltage VDD | VESEL | VE regulator output |
         * +--------------------------+-------+---------------------+
         * |   VDD >= VE1 + 0.3 V     |   0   |         VE1         |
         * +--------------------------+-------+---------------------+
         * |   VDD >= VE2 + 0.2 V     |   1   |         VE2         |
         * +--------------------------+-------+---------------------+
         */
        unsigned short      VESEL       : 1;
        /**
         * This bit sets the VE regulator into heavy load protection mode.
         * .HVLDVE
         *  This bit sets the VE regulator into heavy load protection mode.
         *    1 (R/W): Heavy load protection On
         *    0 (R/W): Heavy load protection Off
         */
        unsigned short      HVLDVE      : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * These bits switch the VE regulator output voltage value to adjust the EPD contrast.
         * .VECONT[3:0]
         *  These bits switch the VE regulator output voltage value to adjust the EPD contrast.
         * +--------------+----------------------------+
         * | VECONT[3:0]  | VE regulator output level  |
         * +--------------+----------------------------+
         * |     0xf      |  Level 15 (high contrast)  |
         * +--------------+----------------------------+
         * |      :       |           :                |
         * +--------------+----------------------------+
         * |     0x0      |  Level 0 (low contrast)    |
         * +--------------+----------------------------+
         */
        unsigned short      VECONT      : 4;
        /**
         * This bit turns the doubler on or off.
         * .DBON
         *  This bit turns the doubler on or off.
         *    1 (R/W): Doubler On
         *    0 (R/W): Doubler Off
         */
        unsigned short      DBON        : 1;
        /**
         * This bit short-circuits between the doubler input and output.
         * .DBSRT
         *  This bit short-circuits between the doubler input and output.
         *    1 (R/W): Short
         *    0 (R/W): Open
         */
        unsigned short      DBSRT       : 1;
        unsigned short      reserved_2  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDPWR0_REG;

/// EPDC power supply Control Register 1.
typedef union EPDPWR1_REG_TAG {
    unsigned short  reg;
    struct EPDPWR1_REG_BIT_TAG {
        /**
         * This bit turns the booster on or off.
         * .BSTON
         *  This bit turns the booster on or off.
         *    1 (R/W): Booster On
         *    0 (R/W): Booster Off
         */
        unsigned short      BSTON       : 1;
        /**
         * This bit pulls down the booster output to VSS.
         * .BSTPLD
         *  This bit pulls down the booster output to VSS.
         *    1 (R/W): Pull-down On
         *    0 (R/W): Pull-down Off
         */
        unsigned short      BSTPLD      : 1;
        unsigned short      reserved_1  : 6;    ///< Reserved.
        /**
         * This bit turns the HVD regulator on or off.
         * .HVDON
         *  This bit turns the HVD regulator on or off.
         *    1 (R/W): HVD regulator On
         *    0 (R/W): HVD regulator Off
         */
        unsigned short      HVDON       : 1;
        /**
         * This bit sets the HVD regulator into heavy load protection mode.
         * .HVLDHVD
         *  This bit sets the HVD regulator into heavy load protection mode.
         *    1 (R/W): Heavy load protection On
         *    0 (R/W): Heavy load protection Off
         */
        unsigned short      HVLDHVD     : 1;
        /**
         * These bits select the HVD regulator output voltage value.
         * .HVDSEL[1:0]
         *  These bits select the HVD regulator output voltage value (VEPD) in accordance with the EPD specification.
         * +-------------+--------------+
         * | HVDSEL[1:0] | VEPD voltage |
         * +-------------+--------------+
         * |     0x3     |   Reserved   |
         * +-------------+--------------+
         * |     0x2     |    9V type   |
         * +-------------+--------------+
         * |     0x1     |   12V type   |
         * +-------------+--------------+
         * |     0x0     |   15V type   |
         * +-------------+--------------+
         */
        unsigned short      HVDSEL      : 2;
        /**
         * These bits switch the HVD regulator output voltage value to adjust the EPD contrast.
         * .HVDCONT[3:0]
         *  These bits switch the HVD regulator output voltage value to adjust the EPD contrast.
         * +--------------+----------------------------+
         * | HVDCONT[3:0] | HVD regulator output level |
         * +--------------+----------------------------+
         * |     0xf      |  Level 15 (high contrast)  |
         * +--------------+----------------------------+
         * |      :       |           :                |
         * +--------------+----------------------------+
         * |     0x0      |  Level 0 (low contrast)    |
         * +--------------+----------------------------+
         */
        unsigned short      HVDCONT     : 4;
    } bit;
    unsigned char   byte_reg[2];
} EPDPWR1_REG;

/// EPD Display Control Register.
typedef union EPDDSP_REG_TAG {
    unsigned short  reg;
    struct EPDDSP_REG_BIT_TAG {
        /**
         * This bit starts display update sequence.
         * .UPDTRG
         *  This bit starts display update sequence.
         *    1 (W): Display update trigger
         *    0 (W): Ineffective
         */
        unsigned short      UPDTRG      : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * These bits select a display mode in waveform mode.
         * .DSPMD[1:0]
         *  These bits select a display mode in waveform mode.
         * +-------------+-------------------+
         * | DSPMD[1:0]  |    Display mode   |
         * +-------------+-------------------+
         * |     0x3     | All black display |
         * +-------------+-------------------+
         * |     0x2     | All white display |
         * +-------------+-------------------+
         * |     0x1     |  Reverse display  |
         * +-------------+-------------------+
         * |     0x0     |  Normal display   |
         * +-------------+-------------------+
         */
        unsigned short      DSPMD       : 2;
        /**
         * This bit sets the EPD controller into waveform or direct mode.
         * .DIRCTL
         *  This bit sets the EPD controller into waveform or direct mode.
         *    1 (R/W): Direct mode
         *    0 (R/W): Waveform mode
         */
        unsigned short      DIRCTL      : 1;
        /**
         * This bit selects the access destination through the display data register.
         * .SELCUR
         *  This bit selects the access destination through the display data register.(waveform mode)
         *    1 (R/W): Current display data
         *    0 (R/W): Next display data
         */
        unsigned short      SELCUR      : 1;
        unsigned short      reserved_2  :10;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDDSP_REG;

/// EPDC Pin Assignment Select Register.
typedef union EPDPOS_REG_TAG {
    unsigned short  reg;
    struct EPDPOS_REG_BIT_TAG {
        /**
         * These bits select the driver output pin assignment.
         * .PINPOS[1:0]
         */
        unsigned short      PINPOS      : 2;
        unsigned short      reserved_1  : 14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDPOS_REG;

/// EPDC interrupt flag status Register.
typedef union EPDINTF_REG_TAG {
    unsigned short  reg;
    struct EPDINTF_REG_BIT_TAG {
        /**
         * This bit indicates the EPDC display update interrupt cause occurrence status.
         * .DUPDIF
         *  This bit indicates the EPDC display update interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      DUPDIF      : 1;
        unsigned short      reserved_1  : 7;    ///< Reserved.
        /**
         * This bit indicates the display update status in waveform mode.
         * .UPDST
         *  This bit indicates the display update status in waveform mode.(waveform mode)
         *    1 (R): Display is being updated
         *    0 (R): Idle
         */
        unsigned short      UPDST       : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDINTF_REG;

/// EPDC interrupt flag enable Register.
typedef union EPDINTE_REG_TAG {
    unsigned short  reg;
    struct EPDINTE_REG_BIT_TAG {
        /**
         * This bit enables the EPDC display update interrupt.
         * .DUPDIE
         *  This bit enables the EPDC display update interrupt.
         *    1 (R/W): Enable interrupt
         *    0 (R/W): Disable interrupt
         */
        unsigned short      DUPDIE      : 1;
        unsigned short      reserved_1  :15;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDINTE_REG;

/// EPDC Top/Back Plane Data Register.
typedef union EPDTPBP_REG_TAG {
    unsigned short  reg;
    struct EPDTPBP_REG_BIT_TAG {
        /**
         * This bit sets the back plane display data/output level.
         * .BP
         *  This bit sets the back plane display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      BP          : 1;
        unsigned short      reserved_1  : 3;    ///< Reserved.
        /**
         * This bit sets the top plane output level.
         * .TP
         *  This bit sets the top plane output level. (direct mode)
         *    1 (R/W): High level
         *    0 (R/W): Low level
         */
        unsigned short      TP          : 1;
        unsigned short      reserved_2  : 3;    ///< Reserved.
        /**
         * This bit sets the top plane outputs into high-impedance state.
         * .TPHZ
         *  This bit sets the top plane outputs into high-impedance state.(direct mode)
         *    1 (R/W): High-impedance
         *    0 (R/W): Normal output
         */
        unsigned short      TPHZ        : 1;
        /**
         * This bit sets the segment/back plane outputs into high-impedance state.
         * .SEGHZ
         *  This bit sets the segment/back plane outputs into high-impedance state.(direct mode)
         *    1 (R/W): High-impedance
         *    0 (R/W): Normal output
         */
        unsigned short      SEGHZ       : 1;
        unsigned short      reserved_3  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDTPBP_REG;

/// EPD Segment Data Register 0.
typedef union EPDSEG0_REG_TAG {
    unsigned short  reg;
    struct EPDSEG0_REG_BIT_TAG {
        /**
         * This bits set the segment display data/output level.
         * .SEG00
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG00       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG01
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG01       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG02
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG02       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG03
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG03       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG04
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG04       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG05
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG05       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG06
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG06       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG07
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG07       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG08
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG08       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG09
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode
         */
        unsigned short      SEG09       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG10
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG10       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG11
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG11       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG12
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG12       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG13
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG13       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG14
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG14       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG15
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG15       : 1;
    } bit;
    unsigned char   byte_reg[2];
} EPDSEG0_REG;

/// EPD Segment Data Register 1.
typedef union EPDSEG1_REG_TAG {
    unsigned short  reg;
    struct EPDSEG1_REG_BIT_TAG {
        /**
         * This bits set the segment display data/output level.
         * .SEG16
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG16       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG17
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG17       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG18
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG18       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG19
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG19       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG20
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG20       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG21
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG21       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG22
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG22       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG23
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG23       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG24
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG24       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG25
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG25       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG26
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG26       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG27
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG27       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG28
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG28       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG29
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG29       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG30
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG30       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG31
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG31       : 1;
    } bit;
    unsigned char   byte_reg[2];
} EPDSEG1_REG;

/// EPD Segment Data Register 2.
typedef union EPDSEG2_REG_TAG {
    unsigned short  reg;
    struct EPDSEG2_REG_BIT_TAG {
        /**
         * This bits set the segment display data/output level.
         * .SEG32
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG32       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG33
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG33       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG34
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG34       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG35
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG35       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG36
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG36       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG37
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG37       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG38
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG38       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG39
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG39       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG40
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG40       : 1;
        /**
         * This bits set the segment display data/output level.
         * .SEG41
         *  This bits set the segment display data/output level.
         *    1 (R/W): Black (waveform mode)/High level (direct mode)
         *    0 (R/W): White (waveform mode)/Low level (direct mode)
         */
        unsigned short      SEG41       : 1;
        unsigned short      reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDSEG2_REG;

/// EPD Top/Back Plane Enable Register.
typedef union EPDTPBPEN_REG_TAG {
    unsigned short  reg;
    struct EPDTPBPEN_REG_BIT_TAG {
        /**
         * This bit enables the back plane pin output.
         * .BPEN
         *  This bit enables the back plane pin output.
         *    1 (R/W): Enable back plane pin output
         *    0 (R/W): Disable back plane pin output
         */
        unsigned short      BPEN        : 1;
        unsigned short      reserved_1  : 3;    ///< Reserved.
        /**
         * This bit enables the top plane pin output.
         * .TPEN
         *  This bit enables the top plane pin output.
         *    1 (R/W): Enable top plane pin output
         *    0 (R/W): Disable top plane pin output
         */
        unsigned short      TPEN        : 1;
        unsigned short      reserved_2  :11;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDTPBPEN_REG;

/// EPD Segment Enable Register 0.
typedef union EPDSEGEN0_REG_TAG {
    unsigned short  reg;
    struct EPDSEGEN0_REG_BIT_TAG {
        /**
         * This bits enable the ESEG00 pin outputs.
         * .SEG00EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG00EN     : 1;
        /**
         * This bits enable the ESEG01 pin outputs.
         * .SEG01EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG01EN     : 1;
        /**
         * This bits enable the ESEG02 pin outputs.
         * .SEG02EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG02EN     : 1;
        /**
         * This bits enable the ESEG03 pin outputs.
         * .SEG03EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG03EN     : 1;
        /**
         * This bits enable the ESEG04 pin outputs.
         * .SEG04EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG04EN     : 1;
        /**
         * This bits enable the ESEG05 pin outputs.
         * .SEG05EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG05EN     : 1;
        /**
         * This bits enable the ESEG06 pin outputs.
         * .SEG06EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG06EN     : 1;
        /**
         * This bits enable the ESEG07 pin outputs.
         * .SEG07EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG07EN     : 1;
        /**
         * This bits enable the ESEG02 pin outputs.
         * .SEG02EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG08EN     : 1;
        /**
         * This bits enable the ESEG02 pin outputs.
         * .SEG02EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG09EN     : 1;
        /**
         * This bits enable the ESEG10 pin outputs.
         * .SEG10EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG10EN     : 1;
        /**
         * This bits enable the ESEG11 pin outputs.
         * .SEG11EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG11EN     : 1;
        /**
         * This bits enable the ESEG12 pin outputs.
         * .SEG12EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG12EN     : 1;
        /**
         * This bits enable the ESEG13 pin outputs.
         * .SEG13EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG13EN     : 1;
        /**
         * This bits enable the ESEG14 pin outputs.
         * .SEG14EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG14EN     : 1;
        /**
         * This bits enable the ESEG15 pin outputs.
         * .SEG15EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG15EN     : 1;
    } bit;
    unsigned char   byte_reg[2];
} EPDSEGEN0_REG;

/// EPD Segment Enable Register 1.
typedef union EPDSEGEN1_REG_TAG {
    unsigned short  reg;
    struct EPDSEGEN1_REG_BIT_TAG {
        /**
         * This bits enable the ESEG16 pin outputs.
         * .SEG16EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG16EN     : 1;
        /**
         * This bits enable the ESEG17 pin outputs.
         * .SEG17EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG17EN     : 1;
        /**
         * This bits enable the ESEG18 pin outputs.
         * .SEG18EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG18EN     : 1;
        /**
         * This bits enable the ESEG19 pin outputs.
         * .SEG19EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG19EN     : 1;
        /**
         * This bits enable the ESEG20 pin outputs.
         * .SEG20EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG20EN     : 1;
        /**
         * This bits enable the ESEG21 pin outputs.
         * .SEG21EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG21EN     : 1;
        /**
         * This bits enable the ESEG22 pin outputs.
         * .SEG22EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG22EN     : 1;
        /**
         * This bits enable the ESEG23 pin outputs.
         * .SEG23EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG23EN     : 1;
        /**
         * This bits enable the ESEG24 pin outputs.
         * .SEG24EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG24EN     : 1;
        /**
         * This bits enable the ESEG25 pin outputs.
         * .SEG25EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG25EN     : 1;
        /**
         * This bits enable the ESEG26 pin outputs.
         * .SEG26EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG26EN     : 1;
        /**
         * This bits enable the ESEG27 pin outputs.
         * .SEG27EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG27EN     : 1;
        /**
         * This bits enable the ESEG28 pin outputs.
         * .SEG28EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG28EN     : 1;
        /**
         * This bits enable the ESEG29 pin outputs.
         * .SEG29EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG29EN     : 1;
        /**
         * This bits enable the ESEG30 pin outputs.
         * .SEG30EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG30EN     : 1;
        /**
         * This bits enable the ESEG31 pin outputs.
         * .SEG31EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG31EN     : 1;
    } bit;
    unsigned char   byte_reg[2];
} EPDSEGEN1_REG;

/// EPD Segment Enable Register 2.
typedef union EPDSEGEN2_REG_TAG {
    unsigned short  reg;
    struct EPDSEGEN2_REG_BIT_TAG {
        /**
         * This bits enable the ESEG32 pin outputs.
         * .SEG32EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG32EN     : 1;
        /**
         * This bits enable the ESEG33 pin outputs.
         * .SEG33EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG33EN     : 1;
        /**
         * This bits enable the ESEG34 pin outputs.
         * .SEG34EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG34EN     : 1;
        /**
         * This bits enable the ESEG35 pin outputs.
         * .SEG35EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG35EN     : 1;
        /**
         * This bits enable the ESEG36 pin outputs.
         * .SEG36EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG36EN     : 1;
        /**
         * This bits enable the ESEG37 pin outputs.
         * .SEG37EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG37EN     : 1;
        /**
         * This bits enable the ESEG38 pin outputs.
         * .SEG38EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG38EN     : 1;
        /**
         * This bits enable the ESEG39 pin outputs.
         * .SEG39EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG39EN     : 1;
        /**
         * This bits enable the ESEG40 pin outputs.
         * .SEG40EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG40EN     : 1;
        /**
         * This bits enable the ESEG41 pin outputs.
         * .SEG41EN
         *  These bits enable the ESEGxx pin outputs.
         *    1 (R/W): Enable ESEGxx output
         *    0 (R/W): Disable ESEGxx pin output
         */
        unsigned short      SEG41EN     : 1;
        unsigned short      reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} EPDSEGEN2_REG;

/// EPD Waveform Timing Set Registers 0.
typedef union EPDWAVEN_REG_TAG {
    unsigned short  reg;
    struct EPDWAVEN_REG_BIT_TAG {
        /**
         * These bits specify the timing set period in a number of CLK_EPDC clocks.
         * .INTV[7:0]
         *  These bits specify the timing set period in a number of CLK_EPDC clocks.
         *  Time [s] = (INTV[7:0] + 1)/CLK_EPDC frequency
         */
        unsigned short      INTV        : 8;
        /**
         * This bit sets the waveform output from the segment and back plane pins to maintain white display during update (white to white).
         * .WW
         *  This bit sets the waveform output from the segment and back plane pins to maintain white display during update (white to white).
         *    1 (R/W): High
         *    0 (R/W): Low
         */
        unsigned short      WW          : 1;
        /**
         * This bit sets the waveform output from the segment and back plane pins to switch white display to black during update.
         * .WB
         *  This bit sets the waveform output from the segment and back plane pins to switch white display to black during update.
         *    1 (R/W): High
         *    0 (R/W): Low
         */
        unsigned short      WB          : 1;
        /**
         * This bit sets the waveform output from the segment and back plane pins to switch black display to white during update.
         * .BW
         *  This bit sets the waveform output from the segment and back plane pins to switch black display to white during update.
         *    1 (R/W): High
         *    0 (R/W): Low
         */
        unsigned short      BW          : 1;
        /**
         * This bit sets the waveform output from the segment and back plane pins to maintain black display during update (black to black).
         * .BB
         *  This bit sets the waveform output from the segment and back plane pins to maintain black display during update (black to black).
         *    1 (R/W): High
         *    0 (R/W): Low
         */
        unsigned short      BB          : 1;
        /**
         * This bit sets the waveform output from the top plane pins.
         * .TP
         *  This bit sets the waveform output from the top plane pins.
         *    1 (R/W): High
         *    0 (R/W): Low
         */
        unsigned short      TP          : 1;
        /**
         * This bit sets the segment and back plane pins into high-impedance state.
         * .HIZ
         *  This bit sets the segment and back plane pins into high-impedance state.
         *    1 (R/W): High-impedance
         *    0 (R/W): High/low output
         */
        unsigned short      HIZ         : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit specifies the end of waveform.
         * .EOW
         *  This bit specifies the end of waveform.
         *    1 (R/W): End
         *    0 (R/W): Continued
         */
        unsigned short      EOW         : 1;
    } bit;
    unsigned char   byte_reg[2];
} EPDWAVEN_REG;

#ifdef __cplusplus
}
#endif

#endif // EPDC_REG_H__
