/************************************************************************//**
 *
 * @file    rtclp_f63_reg.h
 *
 * @brief   Definition of the Independent Low-Power Real-Time Clock (RTCLP) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018-2019. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 19 Jun. 2019    1.10
 *             - Change source code.
 *
 ******************************************************************************/
#ifndef RTCLP_REG_H__
#define RTCLP_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// RTC Control Register
typedef union RTCCTL_REG_TAG {
    unsigned char   reg;
    struct RTCCTL_REG_BIT_TAG {
        /**
         * This bit starts/stops the real-time clock counter.
         * .RTCRUN
         *  This bit starts/stops the real-time clock counter.
         *    1 (R/W): Running/start control
         *    0 (R/W): Idle/stop control
         */
        unsigned char       RTCRUN      : 1;
        /**
         * This bit resets the divider counter, the RTCCTL.RTCADJ bit, and the RTCCTL.RTCHLD bit.
         * .RTCRST
         *  This bit resets the divider counter, the RTCCTL.RTCADJ bit, and the RTCCTL.RTCHLD bit.
         *    1 (W): Reset
         *    0 (W): Ineffective
         *    1 (R): Reset is being executed.
         *    0 (R): Reset has finished. (Normal operation)
         */
        unsigned char       RTCRST      : 1;
        /**
         * This bit executes the 30-second correction time adjustment function.
         * .RTCADJ
         * .This bit executes the 30-second correction time adjustment function.
         *    1 (W): Execute 30-second correction
         *    0 (W): Ineffective
         *    1 (R): 30-second correction is executing.
         *    0 (R): 30-second correction has finished. (Normal operation)
         */
        unsigned char       RTCADJ      : 1;
        unsigned char       reserved_1  : 1;    ///< Reserved.
        /**
         * This bit sets the hour counter to 24H mode or 12H mode.
         * .RTC24H
         *  This bit sets the hour counter to 24H mode or 12H mode.
         *    1 (R/W): 24H mode
         *    0 (R/W): 12H mode
         */
        unsigned char       RTC24H      : 1;
        /**
         * This bit halts the count-up operation of the real-time clock counter.
         * .RTCHLD
         *  This bit halts the count-up operation of the real-time clock counter.
         *    1 (R/W): Halt real-time clock counter count-up operation
         *    0 (R/W): Normal operation
         */
        unsigned char       RTCHLD      : 1;
        /**
         * This bit indicates whether the counter is performing count-up operation or not.
         * .RTCBSY
         *  This bit indicates whether the counter is performing count-up operation or not.
         *    1 (R): In count-up operation
         *    0 (R): Idle (ready to rewrite real-time clock counter)
         */
        unsigned char       RTCBSY      : 1;
        unsigned char       reserved_2  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCCTL_REG;

/// RTC Trimming  Control Register
typedef union RTCTRMCTL_REG_TAG {
    unsigned char   reg;
    struct RTCTRMCTL_REG_BIT_TAG {
        /**
         * Write the correction value for adjusting the 1 Hz frequency to these bits to execute theoretical regulation.
         *  .RTCTRM[6:0]
         *  Write the correction value for adjusting the 1 Hz frequency to these bits to execute theoretical regulation.
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |       RTCTRM[6:0]        |   Correction   | Correction |       RTCTRM[6:0]        |   Correction   | Correction |
         * | bits (two's-complement)  | value (decimal)| rate [ppm] | bits (two's-complement)  | value (decimal)| rate [ppm] |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |          0x00            |       0        |     0.0    |          0x40            |      -64       |   -61.0    |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |          0x01            |       1        |     1.0    |          0x41            |      -63       |   -60.1    |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |          0x02            |       2        |     1.9    |          0x42            |      -62       |   -59.1    |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |          0x03            |       3        |     2.9    |          0x43            |      -61       |   -58.2    |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |          ...             |      ...       |     ...    |          ...             |      ...       |    ...     |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |          0x3e            |      62        |    59.1    |          0x7e            |       -2       |    -1.9    |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         * |          0x3f            |      63        |    60.1    |          0x7f            |       -1       |    -1.0    |
         * +--------------------------+----------------+------------+--------------------------+----------------+------------+
         */
        unsigned char       RTCTRM      : 7;
        /**
         * This bit indicates whether the theoretical regulation is currently executed or not.
         * .RTCTRMBSY
         *  This bit indicates whether the theoretical regulation is currently executed or not.
         *    1 (R): Theoretical regulation is executing.
         *    0 (R): Theoretical regulation has finished (or not executed).
         */
        unsigned char       RTCTRMBSY   : 1;
    } bit;
    unsigned char   byte_reg[1];
} RTCTRMCTL_REG;

/// RTC Second Alarm Register
typedef union RTCALM1_REG_TAG {
    unsigned char   reg;
    struct RTCALM1_REG_BIT_TAG {
        /**
         * There bits set the 1-second digit of the alarm time.
         * .RTCSLA[3:0]
         *  There bits set the 1-second digit of the alarm time.
         * .RTCSHA[2:0]
         *  There bits set the 10-second digit of the alarm time.
         * +----------------------------------------------------+
         * | Setting value in BCD code |                        |
         * +-------------+-------------+                        |
         * | RTCSHA[2:0] | RTCSLA[3:0] | Alarm (second) setting |
         * +-------------+-------------+------------------------+
         * |     0x0     |     0x0     |  00 seconds            |
         * +-------------+-------------+------------------------+
         * |     0x0     |     0x1     |  01 seconds            |
         * +-------------+-------------+------------------------+
         * |     ...     |    ...      |  ...                   |
         * +-------------+-------------+------------------------+
         * |     0x0     |     0x9     |  09 seconds            |
         * +-------------+-------------+------------------------+
         * |     0x1     |     0x0     |  10 seconds            |
         * +-------------+-------------+------------------------+
         * |     ...     |    ...      |  ...                   |
         * +-------------+-------------+------------------------+
         * |     0x5     |     0x9     |  59 seconds            |
         * +-------------+-------------+------------------------+
         */
        unsigned char       RTCSLA      : 4;
        /**
         * There bits set the 10-second digit of the alarm time.
         * .RTCSLA[3:0]
         *  There bits set the 1-second digit of the alarm time.
         * .RTCSHA[2:0]
         *  There bits set the 10-second digit of the alarm time.
         * +----------------------------------------------------+
         * | Setting value in BCD code |                        |
         * +-------------+-------------+                        |
         * | RTCSHA[2:0] | RTCSLA[3:0] | Alarm (second) setting |
         * +-------------+-------------+------------------------+
         * |     0x0     |     0x0     |  00 seconds            |
         * +-------------+-------------+------------------------+
         * |     0x0     |     0x1     |  01 seconds            |
         * +-------------+-------------+------------------------+
         * |     ...     |    ...      |  ...                   |
         * +-------------+-------------+------------------------+
         * |     0x0     |     0x9     |  09 seconds            |
         * +-------------+-------------+------------------------+
         * |     0x1     |     0x0     |  10 seconds            |
         * +-------------+-------------+------------------------+
         * |     ...     |    ...      |  ...                   |
         * +-------------+-------------+------------------------+
         * |     0x5     |     0x9     |  59 seconds            |
         * +-------------+-------------+------------------------+
         */
        unsigned char       RTCSHA      : 3;
        unsigned char       reserved_1  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCALM1_REG;

/// RTC Minute Alarm Register
typedef union RTCALM2_REG_TAG {
    unsigned char   reg;
    struct RTCALM2_REG_BIT_TAG {
        /**
         * There bits set the 1-minute digit of the alarm time.
         * .RTCMILA[3:0]
         *  There bits set the 1-minute digit of the alarm time.
         *  The RTCALM2.RTCMIHA[2:0] bits and the RTCALM2.RTCMILA[3:0] bits set the 10-minute digit
         *  and 1-minute digit of the alarm time, respectively. A value within 0 to 59 minutes can be set in BCD
         *  code.
         */
        unsigned char       RTCMILA     : 4;
        /**
         * There bits set the 10-minute digit of the alarm time.
         * .RTCMIHA[2:0]
         *  There bits set the 10-minute digit of the alarm time.
         *  The RTCALM2.RTCMIHA[2:0] bits and the RTCALM2.RTCMILA[3:0] bits set the 10-minute digit
         *  and 1-minute digit of the alarm time, respectively. A value within 0 to 59 minutes can be set in BCD
         *  code.
         */
        unsigned char       RTCMIHA     : 3;
        unsigned char       reserved_1  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCALM2_REG;

/// RTC Hour Alarm Register
typedef union RTCALM3_REG_TAG {
    unsigned char   reg;
    struct RTCALM3_REG_BIT_TAG {
        /**
         * There bits set the 1-hour digit of the alarm time.
         * .RTCHLA[3:0]
         *  There bits set the 1-hour digit of the alarm time.
         *  The RTCALM2.RTCHHA[1:0] bits and the RTCALM2.RTCHLA[3:0] bits set the 10-hour digit and
         *  1-hour digit of the alarm time, respectively. A value within 1 to 12 ofclock in 12H mode or 0 to 23 in
         *  24H mode can be set in BCD code.
         */
        unsigned char       RTCHLA      : 4;
        /**
         * There bits set the 10-hour digit of the alarm time.
         * .RTCHHA[1:0]
         *  There bits set the 10-hour digit of the alarm time.
         *  The RTCALM2.RTCHHA[1:0] bits and the RTCALM2.RTCHLA[3:0] bits set the 10-hour digit and
         *  1-hour digit of the alarm time, respectively. A value within 1 to 12 ofclock in 12H mode or 0 to 23 in
         *  24H mode can be set in BCD code.
         */
        unsigned char       RTCHHA      : 2;
        /**
         * This bit set A.M. or P.M. of the alarm time in 12H mode (RTCCTL.RTC24H bit = 0).
         * .RTCAPA
         *  This bit set A.M. or P.M. of the alarm time in 12H mode (RTCCTL.RTC24H bit = 0).
         *    1 (R/W): P.M.
         *    0 (R/W): A.M.
         */
        unsigned char       RTCAPA      : 1;
        unsigned char       reserved_1  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCALM3_REG;

/// RTC Stopwatch Control Register
typedef union RTCSWCTL_REG_TAG {
    unsigned char   reg;
    struct RTCSWCTL_REG_BIT_TAG {
        /**
         * This bit starts/stops the stop watch counter.
         * .SWRUN
         *  This bit starts/stops the stop watch counter.
         *    1 (R/W): Running/start control
         *    0 (R/W): Idle/stop control
         */
        unsigned char       SWRUN       : 1;
        unsigned char       reserved_1  : 3;    ///< Reserved.
        /**
         * This bit resets the stop watch counter to 0x00.
         * .SWRST
         *  This bit resets the stop watch counter to 0x00.
         *    1 (W): Reset
         *    0 (W): Ineffective
         *    0 (R): Always 0 when being read
         */
        unsigned char       SWRST       : 1;
        unsigned char       reserved_2  : 3;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCSWCTL_REG;

/// RTC Stopwatch counter Register
typedef union RTCSWCNT_REG_TAG {
    unsigned char   reg;
    struct RTCSWCNT_REG_BIT_TAG {
        /**
         * The 1/100-second digits of the stop watch counter can be read as a BCD code.
         * .BCD100[3:0]
         *  The 1/100-second digits of the stop watch counter can be read as a BCD code.
         */
        unsigned char       BCD100      : 4;
        /**
         * The 1/10-second digits of the stop watch counter can be read as a BCD code.
         * .BCD10[3:0]
         *  The 1/10-second digits of the stop watch counter can be read as a BCD code.
         */
        unsigned char       BCD10       : 4;
    } bit;
    unsigned char   byte_reg[1];
} RTCSWCNT_REG;

/// RTC divider Register
typedef union RTCDIV_REG_TAG {
    unsigned char   reg;
    struct RTCDIV_REG_BIT_TAG {
        /**
         * The divider counter data can be read from these bits(128Hz).
         *  .RTC128HZ
         *  The divider counter data can be read from these bits(128Hz).
         */
        unsigned char       RTC128HZ    : 1;
        /**
         * The divider counter data can be read from these bits(64Hz).
         *  .RTC64HZ
         *  The divider counter data can be read from these bits(64Hz).
         */
        unsigned char       RTC64HZ     : 1;
        /**
         * The divider counter data can be read from these bits(32Hz).
         *  .RTC32HZ
         *  The divider counter data can be read from these bits(32Hz).
         */
        unsigned char       RTC32HZ     : 1;
        /**
         * The divider counter data can be read from these bits(16Hz).
         *  .RTC16HZ
         *  The divider counter data can be read from these bits(16Hz).
         */
        unsigned char       RTC16HZ     : 1;
        /**
         * The divider counter data can be read from these bits(8Hz).
         *  .RTC8HZ
         *  The divider counter data can be read from these bits(8Hz).
         */
        unsigned char       RTC8HZ      : 1;
        /**
         * The divider counter data can be read from these bits(4Hz).
         *  .RTC4HZ
         *  The divider counter data can be read from these bits(4Hz).
         */
        unsigned char       RTC4HZ      : 1;
        /**
         * The divider counter data can be read from these bits(2Hz).
         *  .RTC2HZ
         *  The divider counter data can be read from these bits(2Hz).
         */
        unsigned char       RTC2HZ      : 1;
        /**
         * The divider counter data can be read from these bits(1Hz).
         *  .RTC1HZ
         *  The divider counter data can be read from these bits(1Hz).
         */
        unsigned char       RTC1HZ      : 1;
    } bit;
    unsigned char   byte_reg[1];
} RTCDIV_REG;

/// RTC second Register
typedef union RTCSEC_REG_TAG {
    unsigned char   reg;
    struct RTCSEC_REG_BIT_TAG {
        /**
         * There bits are used to set and read the 1-second digit of the second counter.
         * .RTCSL[3:0]
         *  There bits are used to set and read the 1-second digit of the second counter.
         *  The RTCSEC.RTCSH[2:0] bits and the RTCSEC.RTCSL[3:0] bits are used to set and read the 10-second
         *  digit and the 1-second digit of the second counter, respectively. The setting/read values are a BCD
         *  code within the range from 0 to 59.
         */
        unsigned char       RTCSL       : 4;
        /**
         * There bits are used to set and read the 10-second digit of the second counter.
         * .RTCSH[2:0]
         *  There bits are used to set and read the 10-second digit of the second counter.
         *  The RTCSEC.RTCSH[2:0] bits and the RTCSEC.RTCSL[3:0] bits are used to set and read the 10-second
         *  digit and the 1-second digit of the second counter, respectively. The setting/read values are a BCD
         *  code within the range from 0 to 59.
         */
        unsigned char       RTCSH       : 3;
        unsigned char       reserved_1  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCSEC_REG;

/// RTC Minute Register
typedef union RTCMIN_REG_TAG {
    unsigned char   reg;
    struct RTCMIN_REG_BIT_TAG {
        /**
         * There bits are used to set and read the 1-minute digit of the minute counter.
         * .RTCMIL[3:0]
         *  There bits are used to set and read the 1-minute digit of the minute counter.
         *  The RTCHUR.RTCMIH[2:0] bits and the RTCHUR.RTCMIL[3:0] bits are used to set and read the
         *  10-minute digit and the 1-minute digit of the minute counter, respectively. The setting/read values are
         *  a BCD code within the range from 0 to 59.
         */
        unsigned char       RTCMIL      : 4;
        /**
         * There bits are used to set and read the 10-minute digit of the minute counter.
         * .RTCMIH[2:0]
         *  There bits are used to set and read the 10-minute digit of the minute counter.
         *  The RTCHUR.RTCMIH[2:0] bits and the RTCHUR.RTCMIL[3:0] bits are used to set and read the
         *  10-minute digit and the 1-minute digit of the minute counter, respectively. The setting/read values are
         *  a BCD code within the range from 0 to 59.
         */
        unsigned char       RTCMIH      : 3;
        unsigned char       reserved_1  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCMIN_REG;

/// RTC Hour Register
typedef union RTCHUR_REG_TAG {
    unsigned char   reg;
    struct RTCHUR_REG_BIT_TAG {
        /**
         * There bits are used to set and read the 1-hour digit of the hour counter.
         * .RTCHL[3:0]
         *  There bits are used to set and read the 1-hour digit of the hour counter.
         *  The RTCHUR.RTCHH[1:0] bits and the RTCHUR.RTCHL[3:0] bits are used to set and read the 10-
         *  hour digit and the 1-hour digit of the hour counter, respectively. The setting/read values are a BCD
         *  code within the range from 1 to 12 in 12H mode or 0 to 23 in 24H mode.
         */
        unsigned char       RTCHL       : 4;
        /**
         * There bits are used to set and read the 10-hour digit of the hour counter.
         * .RTCHH[1:0]
         *  There bits are used to set and read the 10-hour digit of the hour counter.
         *  The RTCHUR.RTCHH[1:0] bits and the RTCHUR.RTCHL[3:0] bits are used to set and read the 10-
         *  hour digit and the 1-hour digit of the hour counter, respectively. The setting/read values are a BCD
         *  code within the range from 1 to 12 in 12H mode or 0 to 23 in 24H mode.
         */
        unsigned char       RTCHH       : 2;
        /**
         * This bit is used to set and read A.M. or P.M. data in 12H mode (RTCCTL.RTC24H bit = 0).
         * .RTCAP
         *  This bit is used to set and read A.M. or P.M. data in 12H mode (RTCCTL.RTC24H bit = 0).
         *    1 (R/W): P.M.
         *    0 (R/W): A.M.
         */
        unsigned char       RTCAP       : 1;
        unsigned char       reserved_1  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCHUR_REG;

/// RTC Day Register
typedef union RTCDAY_REG_TAG {
    unsigned char   reg;
    struct RTCDAY_REG_BIT_TAG {
        /**
         * There bits are used to set and read the 1-day digit of the day counter.
         * .RTCDL[3:0]
         *  There bits are used to set and read the 1-day digit of the day counter.
         *  The RTCMON.RTCDH[1:0] bits and the RTCMON.RTCDL[3:0] bits are used to set and read the 10-
         *  day digit and the 1-day digit of the day counter, respectively. The setting/read values are a BCD code
         *  within the range from 1 to 31 (to 28 for February in a common year, to 29 for February in a leap year,
         *  or to 30 for April/June/September/November).
         */
        unsigned char       RTCDL       : 4;
        /**
         * There bits are used to set and read the 10-day digit of the day counter.
         * .RTCDH[1:0]
         *  There bits are used to set and read the 10-day digit of the day counter.
         *  The RTCMON.RTCDH[1:0] bits and the RTCMON.RTCDL[3:0] bits are used to set and read the 10-
         *  day digit and the 1-day digit of the day counter, respectively. The setting/read values are a BCD code
         *  within the range from 1 to 31 (to 28 for February in a common year, to 29 for February in a leap year,
         *  or to 30 for April/June/September/November).
         */
        unsigned char       RTCDH       : 2;
        unsigned char       reserved_1  : 2;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCDAY_REG;

/// RTC Month Register
typedef union RTCMON_REG_TAG {
    unsigned char   reg;
    struct RTCMON_REG_BIT_TAG {
        /**
         * There bits are used to set and read the 1-month digit of the month counter.
         * .RTCMOL[3:0]
         *  There bits are used to set and read the 1-month digit of the month counter.
         *  The RTCMON.RTCMOH bit and the RTCMON.RTCMOL[3:0] bits are used to set and read the
         *  10-month digit and the 1-month digit of the month counter, respectively. The setting/read values are a
         *  BCD code within the range from 1 to 12.
         */
        unsigned char       RTCMOL      : 4;
        /**
         * There bits are used to set and read the 10-month digit of the month counter.
         * .RTCMOH
         *  There bits are used to set and read the 10-month digit of the month counter.
         *  The RTCMON.RTCMOH bit and the RTCMON.RTCMOL[3:0] bits are used to set and read the
         *  10-month digit and the 1-month digit of the month counter, respectively. The setting/read values are a
         *  BCD code within the range from 1 to 12.
         */
        unsigned char       RTCMOH      : 1;
        unsigned char       reserved_1  : 3;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCMON_REG;

/// RTC Year Register
typedef union RTCYAR_REG_TAG {
    unsigned char   reg;
    struct RTCYAR_REG_BIT_TAG {
        /**
         * There bits are used to set and read the 1-year digit of the year counter.
         * .RTCYL[3:0]
         *  There bits are used to set and read the 1-year digit of the year counter.
         *  The RTCYAR.RTCYH[3:0] bits and the RTCYAR.RTCYL[3:0] bits are used to set and read the 10-
         *  year digit and the 1-year digit of the year counter, respectively. The setting/read values are a BCD
         *  code within the range from 0 to 99.
         */
        unsigned char       RTCYL       : 4;
        /**
         * There bits are used to set and read the 10-year digit of the year counter.
         * .RTCYH[3:0]
         *  There bits are used to set and read the 10-year digit of the year counter.
         *  The RTCYAR.RTCYH[3:0] bits and the RTCYAR.RTCYL[3:0] bits are used to set and read the 10-
         *  year digit and the 1-year digit of the year counter, respectively. The setting/read values are a BCD
         *  code within the range from 0 to 99.
         */
        unsigned char       RTCYH       : 4;
    } bit;
    unsigned char   byte_reg[1];
} RTCYAR_REG;

/// RTC Week Register
typedef union RTCWK_REG_TAG {
    unsigned char   reg;
    struct RTCWK_REG_BIT_TAG {
        /**
         * These bits are used to set and read day of the week.
         * .WK[2:0]
         *  These bits are used to set and read day of the week.
         *  The day of the week counter is a base-7 counter and the setting/read values are 0x0 to 0x6.
         * +------------+-----------------+
         * |   WK[2:0]  | Day of the week |
         * +------------+-----------------+
         * |    0x6     |    Saturday     |
         * +------------+-----------------+
         * |    0x5     |     Friday      |
         * +------------+-----------------+
         * |    0x4     |    Thursday     |
         * +------------+-----------------+
         * |    0x3     |    Wednesday    |
         * +------------+-----------------+
         * |    0x2     |     Tuesda      |
         * +------------+-----------------+
         * |    0x1     |     Monday      |
         * +------------+-----------------+
         * |    0x0     |     Sunday      |
         * +------------+-----------------+
         */
        unsigned char       WK          : 3;
        unsigned char       reserved_1  : 5;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCWK_REG;

/// RTC Interrupt Flag Register 1.
typedef union RTCINTF1_REG_TAG {
    unsigned char   reg;
    struct RTCINTF1_REG_BIT_TAG {
        /**
         * This bit indicate 1/32-second interrupt cause occurrence status.
         * .1_32SECIF
         *  This bit indicate 1/32-second interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1_32SECIF  : 1;
        /**
         * This bit indicate 1/8-second interrupt cause occurrence status.
         * .1_8SECIF
         *  This bit indicate 1/8-second interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1_8SECIF   : 1;
        /**
         * This bit indicate 1/4-second interrupt cause occurrence status.
         * .1_4SECIF
         *  This bit indicate 1/4-second interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1_4SECIF   : 1;
        /**
         * This bit indicate 1/2-second interrupt cause occurrence status.
         * .1_2SECIF
         *  This bit indicate 1/2-second interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1_2SECIF   : 1;
        /**
         * This bit indicate 1-second interrupt cause occurrence status.
         * .1SECIF
         *  This bit indicate 1-second interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1SECIF     : 1;
        /**
         * This bit indicate 1-minute interrupt cause occurrence status.
         * .1MINIF
         *  This bit indicate 1-minute interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1MINIF     : 1;
        /**
         * This bit indicate 1-hour interrupt cause occurrence status.
         * .1HURIF
         *  This bit indicate 1-hour interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1HURIF     : 1;
        /**
         * This bit indicate 1-day interrupt cause occurrence status.
         * .1DAYIF
         *  This bit indicate 1-day interrupt cause occurrence status.
         * This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       _1DAYIF     : 1;
    } bit;
    unsigned char   byte_reg[1];
} RTCINTF1_REG;

/// RTC Interrupt Flag Register 2.
typedef union RTCINTF2_REG_TAG {
    unsigned char   reg;
    struct RTCINTF2_REG_BIT_TAG {
        /**
         * This bit indicate Alarm interrupt cause occurrence status.
         * .ALARMIF
         *  This bit indicate Alarm interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       ALARMIF     : 1;
        /**
         * This bit indicate Down counter underflow interrupt cause occurrence status.
         * .DCNTUIF
         *  This bit indicate Down counter underflow interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       DCNTUFIF    : 1;
        unsigned char       reserved_1  : 2;    ///< Reserved.
        /**
         * This bit indicate Stop watch 100 Hz interrupt cause occurrence status.
         * .SW100IF
         *  This bit indicate Stop watch 100 Hz interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       SW100IF     : 1;
        /**
         * This bit indicate Stop watch 10 Hz interrupt cause occurrence status.
         * .SW10IF
         *  This bit indicate Stop watch 10 Hz interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       SW10IF      : 1;
        /**
         * This bit indicate Stop watch 1 Hz interrupt cause occurrence status.
         * .SW1IF
         *  This bit indicate Stop watch 1 Hz interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       SW1IF       : 1;
        /**
         * This bit indicate Theoretical regulation completion interrupt cause occurrence status.
         * .RTCTRMIF
         *  This bit indicate Theoretical regulation completion interrupt cause occurrence status.
         *  This bit indicate the real-time clock interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       RTCTRMIF    : 1;
    } bit;
    unsigned char   byte_reg[1];
} RTCINTF2_REG;

/// RTC Interrupt Enable Register 1
typedef union RTCINTE1_REG_TAG {
    unsigned char   reg;
    struct RTCINTE1_REG_BIT_TAG {
        /**
         * This bits enable 1/32-second interrupt.
         * .1_32SECIE
         *  This bits enable 1/32-second interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1_32SECIE  : 1;
        /**
         * This bits enable 1/8-second interrupt.
         * .1_8SECIE
         *  This bits enable 1/8-second interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1_8SECIE   : 1;
        /**
         * This bits enable 1/4-second interrupt.
         * .1_4SECIE
         *  This bits enable 1/4-second interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1_4SECIE   : 1;
        /**
         * This bits enable 1/2-second interrupt.
         * .1_2SECIE
         *  This bits enable 1/2-second interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1_2SECIE   : 1;
        /**
         * This bits enable 1-second interrupt.
         * .1SECIE
         *  This bits enable 1-second interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1SECIE     : 1;
        /**
         * This bits enable 1-minute interrupt.
         * .1MINIE
         *  This bits enable 1-minute interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1MINIE     : 1;
        /**
         * This bits enable 1-hour interrupt.
         * .1HURIE
         *  This bits enable 1-hour interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1HURIE     : 1;
        /**
         * This bits enable 1-day interrupt.
         * .1DAYIE
         *  This bits enable 1-day interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       _1DAYIE     : 1;
    } bit;
    unsigned char   byte_reg[1];
} RTCINTE1_REG;

/// RTC Interrupt Enable Register 2
typedef union RTCINTE2_REG_TAG {
    unsigned char   reg;
    struct RTCINTE2_REG_BIT_TAG {
        /**
         * This bits enable Alarm interrupt.
         * .ALARMIE
         *  This bits enable Alarm interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       ALARMIE     : 1;
        /**
         * This bits enable Down counter underflow interrupt.
         * .DCNTUFIE
         *  This bits enable Down counter underflow interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       DCNTUFIE    : 1;
        unsigned char       reserved_1  : 2;    ///< Reserved.
        /**
         * This bit enable Stop watch 100 Hz interrupt.
         * .SW100IE
         *  This bit enable Stop watch 100 Hz interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       SW100IE     : 1;
        /**
         * This bit enable Stop watch 10 Hz interrupt.
         * .SW10IE
         *  This bit enable Stop watch 10 Hz interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       SW10IE      : 1;
        /**
         * This bit enable Stop watch 1 Hz interrupt.
         * .SW1IE
         *  This bit enable Stop watch 1 Hz interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       SW1IE       : 1;
        /**
         * This bit enable Theoretical regulation completion interrupt.
         * .RTCTRMIE
         *  This bit enable Theoretical regulation completion interrupt.
         *  This bit enable real-time clock interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       RTCTRMIE    : 1;
    } bit;
    unsigned char   byte_reg[1];
} RTCINTE2_REG;

/// RTC Down Counter Control Register
typedef union RTCDCNTCTL_REG_TAG {
    unsigned char   reg;
    struct RTCDCNTCTL_REG_BIT_TAG {
        /**
         * This bit starts/stops the down counter.
         * .RUN
         *  This bit starts/stops the down counter.
         *    1 (W): Start down counter
         *    0 (W): Stop down counter
         *    1 (R): Down counter is running
         *    0 (R): Down counter is idle
         */
        unsigned char       RUN         : 1;
        unsigned char       reserved_1  : 3;    ///< Reserved.
        /**
         * This bit presets the reload data stored in the RTCDCNTRLDL and RTCDCNTRLDH registers to the  down counter.
         * .PRESET
         *  This bit presets the reload data stored in the RTCDCNTRLDL and RTCDCNTRLDH registers to the down counter.
         *    1 (W): Preset
         *    0 (W): Ineffective
         *    1 (R): Presetting in progress
         *    0 (R): Presetting finished or normal operation
         */
        unsigned char       PRESET      : 1;
        unsigned char       reserved_2  : 3;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCDCNTCTL_REG;

/// RTC Down Counter Mode Register
typedef union RTCDCNTMOD_REG_TAG {
    unsigned char   reg;
    struct RTCDCNTMOD_REG_BIT_TAG {
        /**
         * This bit selects the down counter operation mode.
         * .MODE
         *  This bit selects the down counter operation mode.
         *    1 (R/W): One-shot mode
         *    0 (R/W): Repeat mode
         */
        unsigned char       MODE        : 1;
        unsigned char       reserved_1  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} RTCDCNTMOD_REG;

/// RTC Down Counter Lower Reload Data Register.
typedef union RTCDCNTRLDL_REG_TAG {
    unsigned char   reg;
    struct RTCDCNTRLDL_REG_BIT_TAG {
        /**
         * These bits are used to set the initial value to be preset to the down counter.
         * .RLD[7:0]
         *  These bits are used to set the initial value to be preset to the down counter.
         *  The value set to this register will be preset to the down counter when 1 is written to the
         *  RTCDCNTCTL.PRESET bit or when the down counter underflows.
         */
        unsigned char       RLD         : 8;
    } bit;
    unsigned char   byte_reg[1];
} RTCDCNTRLDL_REG;

/// RTC Down Counter Upper Reload Data Register.
typedef union RTCDCNTRLDH_REG_TAG {
    unsigned char   reg;
    struct RTCDCNTRLDH_REG_BIT_TAG {
        /**
         * These bits are used to set the initial value to be preset to the down counter.
         * .RLD[15:8]
         *  These bits are used to set the initial value to be preset to the down counter.
         *  The value set to this register will be preset to the down counter when 1 is written to the
         *  RTCDCNTCTL.PRESET bit or when the down counter underflows.
         */
        unsigned char       RLD         : 8;
    } bit;
    unsigned char   byte_reg[1];
} RTCDCNTRLDH_REG;

/// RTC Down Counter Lower Data Register.
typedef union RTCDCNTL_REG_TAG {
    unsigned char   reg;
    struct RTCDCNTL_REG_BIT_TAG {
        /**
         * The current down counter value can be read out from these bits.
         * .CNT[7:0]
         *  The current down counter value can be read out from these bits.
         */
        unsigned char       CNT         : 8;
    } bit;
    unsigned char   byte_reg[1];
} RTCDCNTL_REG;

/// RTC Down Counter Upper Data Register.
typedef union RTCDCNTH_REG_TAG {
    unsigned char   reg;
    struct RTCDCNTH_REG_BIT_TAG {
        /**
         * The current down counter value can be read out from these bits.
         * .CNT[15:8]
         *  The current down counter value can be read out from these bits.
         */
        unsigned char       CNT         : 8;
    } bit;
    unsigned char   byte_reg[1];
} RTCDCNTH_REG;

/// Power Control Register
typedef union PWRCTL_REG_TAG {
    unsigned char   reg;
    struct PWRCTL_REG_BIT_TAG {
        /**
         * This bit turns the MCU core operating voltage VD1M on and of
         * .MCUPD
         *  This bit turns the MCU core operating voltage VD1M on and of
         *    1 (R/W): Off
         *    0 (R/W): On
         */
        unsigned char       MCUPD       : 1;
        unsigned char       reserved_1  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} PWRCTL_REG;

/// P2 Input Data Register
typedef union P2IN_REG_TAG {
    unsigned char   reg;
    struct P2IN_REG_BIT_TAG {
        /**
         * The GPIO port pin status can be read out from this bit.
         * .P20IN
         *  The GPIO port pin status can be read out from this bit.
         *    1 (R): Port pin = High level
         *    0 (R): Port pin = Low level
         */
        unsigned char       P20IN       : 1;
        /**
         * The GPIO port pin status can be read out from this bit.
         * .P21IN
         *  The GPIO port pin status can be read out from this bit.
         *    1 (R): Port pin = High level
         *    0 (R): Port pin = Low level
         */
        unsigned char       P21IN       : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2IN_REG;

/// P2 Output Data Register
typedef union P2OUT_REG_TAG {
    unsigned char   reg;
    struct P2OUT_REG_BIT_TAG {
        /**
         * This bit is used to set data to be output from the GPIO port pin.
         * .P20OUT
         *  This bit is used to set data to be output from the GPIO port pin.
         *    1 (R/W): Output high level from the port pin
         *    0 (R/W): Output low level from the port pin
         */
        unsigned char       P20OUT      : 1;
        /**
         * This bit is used to set data to be output from the GPIO port pin.
         * .P21OUT
         *  This bit is used to set data to be output from the GPIO port pin.
         *    1 (R/W): Output high level from the port pin
         *    0 (R/W): Output low level from the port pin
         */
        unsigned char       P21OUT      : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2OUT_REG;

/// P2 Output Enable Register
typedef union P2OEN_REG_TAG {
    unsigned char   reg;
    struct P2OEN_REG_BIT_TAG {
        /**
         * This bit enable/disable the GPIO port output.
         * .P20OEN
         *  This bit enable/disable the GPIO port output.
         *    1 (R/W): Enable (Data is output from the port pin.)
         *    0 (R/W): Disable (The port is placed into Hi-Z.)
         */
        unsigned char       P20OEN      : 1;
        /**
         * This bit enable/disable the GPIO port output.
         * .P21OEN
         *  This bit enable/disable the GPIO port output.
         *    1 (R/W): Enable (Data is output from the port pin.)
         *    0 (R/W): Disable (The port is placed into Hi-Z.)
         */
        unsigned char       P21OEN      : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2OEN_REG;

/// P2 Input Enable Register
typedef union P2IEN_REG_TAG {
    unsigned char   reg;
    struct P2IEN_REG_BIT_TAG {
        /**
         * This bit enable/disable the GPIO port input.
         * .P20IEN
         *  This bit enable/disable the GPIO port input.
         *    1 (R/W): Enable (The port pin status is input.)
         *    0 (R/W): Disable (Input data is fixed at 0.)
         */
        unsigned char       P20IEN      : 1;
        /**
         * This bit enable/disable the GPIO port input.
         * .P21IEN
         *  This bit enable/disable the GPIO port input.
         *    1 (R/W): Enable (The port pin status is input.)
         *    0 (R/W): Disable (Input data is fixed at 0.)
         */
        unsigned char       P21IEN      : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2IEN_REG;

/// P2 Pull-up/down Enable Register
typedef union P2REN_REG_TAG {
    unsigned char   reg;
    struct P2REN_REG_BIT_TAG {
        /**
         * This bit enable/disable the port pull-up/down control.
         * .P20REN
         *  This bit enable/disable the port pull-up/down control.
         *    1 (R/W): Enable (The built-in pull-up/down resistor is used.)
         *    0 (R/W): Disable (No pull-up/down control is performed.)
         */
        unsigned char       P20REN      : 1;
        /**
         * This bit enable/disable the port pull-up/down control.
         * .P21REN
         *  This bit enable/disable the port pull-up/down control.
         *    1 (R/W): Enable (The built-in pull-up/down resistor is used.)
         *    0 (R/W): Disable (No pull-up/down control is performed.)
         */
        unsigned char       P21REN      : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2REN_REG;

/// P2 Pull-up/down Control Register
typedef union P2PDPU_REG_TAG {
    unsigned char   reg;
    struct P2PDPU_REG_BIT_TAG {
        /**
         * This bit select either the pull-up resistor or the pull-down resistor when using a resistor built into the port.
         * .P20PDPU
         *  This bit select either the pull-up resistor or the pull-down resistor when using a resistor built into
         *  the port.
         *    1 (R/W): Pull-up resistor
         *    0 (R/W): Pull-down resistor
         */
        unsigned char       P20PDPU     : 1;
        /**
         * This bit select either the pull-up resistor or the pull-down resistor when using a resistor built into the port.
         * .P21PDPU
         *  This bit select either the pull-up resistor or the pull-down resistor when using a resistor built into
         *  the port.
         *    1 (R/W): Pull-up resistor
         *    0 (R/W): Pull-down resistor
         */
        unsigned char       P21PDPU     : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2PDPU_REG;

/// P2 Port Interrupt Flag Register
typedef union P2INTF_REG_TAG {
    unsigned char   reg;
    struct P2INTF_REG_BIT_TAG {
        /**
         * This bit indicate the port input interrupt cause occurrence status.
         * .P20IF
         *  This bit indicate the port input interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       P20IF       : 1;
        /**
         * This bit indicate the port input interrupt cause occurrence status.
         * .P21IF
         *  This bit indicate the port input interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned char       P21IF       : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2INTF_REG;

/// P2 Port Interrupt Control Register
typedef union P2INTE_REG_TAG {
    unsigned char   reg;
    struct P2INTE_REG_BIT_TAG {
        /**
         * This bit enable port input interrupt.
         * .P20IE
         *  This bit enable port input interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       P20IE       : 1;
        /**
         * This bit enable port input interrupt.
         * .P21IE
         *  This bit enable port input interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned char       P21IE       : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2INTE_REG;

/// P2 Port Interrupt Edge Control Register
typedef union P2IEDGE_REG_TAG {
    unsigned char   reg;
    struct P2IEDGE_REG_BIT_TAG {
        /**
         * This bit select the input signal edge to generate a port input interrupt.
         * .P20EDGE
         *  This bit select the input signal edge to generate a port input interrupt.
         *    1 (R/W): An interrupt will occur at a falling edge.
         *    0 (R/W): An interrupt will occur at a rising edge.
         */
        unsigned char       P20EDGE     : 1;
        /**
         * This bit select the input signal edge to generate a port input interrupt.
         * .P21EDGE
         *  This bit select the input signal edge to generate a port input interrupt.
         *    1 (R/W): An interrupt will occur at a falling edge.
         *    0 (R/W): An interrupt will occur at a rising edge.
         */
        unsigned char       P21EDGE     : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2IEDGE_REG;

/// P2 Port Chattering Filter Enable Register
typedef union P2CHATEN_REG_TAG {
    unsigned char   reg;
    struct P2CHATEN_REG_BIT_TAG {
        /**
         * This bit enable/disable the chattering filter function.
         * .P20CHATEN
         *  This bit enable/disable the chattering filter function.
         *    1 (R/W): Enable (The chattering filter is used.)
         *    0 (R/W): Disable (The chattering filter is bypassed.)
         */
        unsigned char       P20CHATEN   : 1;
        /**
         * This bit enable/disable the chattering filter function.
         * .P21CHATEN
         *  This bit enable/disable the chattering filter function.
         *    1 (R/W): Enable (The chattering filter is used.)
         *    0 (R/W): Disable (The chattering filter is bypassed.)
         */
        unsigned char       P21CHATEN   : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2CHATEN_REG;

/// P2 Port Mode Select Register
typedef union P2MODSEL_REG_TAG {
    unsigned char   reg;
    struct P2MODSEL_REG_BIT_TAG {
        /**
         * This bit select whether each port is used for the GPIO function or a peripheral I/O function.
         * .P20SEL
         *  This bit select whether each port is used for the GPIO function or a peripheral I/O function.
         *    1 (R/W): Use peripheral I/O function
         *    0 (R/W): Use GPIO function
         */
        unsigned char       P20SEL      : 1;
        /**
         * This bit select whether each port is used for the GPIO function or a peripheral I/O function.
         * .P21SEL
         *  This bit select whether each port is used for the GPIO function or a peripheral I/O function.
         *    1 (R/W): Use peripheral I/O function
         *    0 (R/W): Use GPIO function
         */
        unsigned char       P21SEL      : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2MODSEL_REG;

/// P2 Port Function Select Register
typedef union P2FNCSEL0_REG_TAG {
    unsigned char   reg;
    struct P2FNCSEL0_REG_BIT_TAG {
        /**
         * These bits select the peripheral I/O function to be assigned to each port pin.
         * .P20MUX[1:0]
         *  These bits select the peripheral I/O function to be assigned to each port pin.
         * +-------------+-------------------------+
         * | P2nMUX[1:0] | Peripheral I/O function |
         * +-------------+-------------------------+
         * |     0x3     |       Function 3        |
         * +-------------+-------------------------+
         * |     0x2     |       Function 2        |
         * +-------------+-------------------------+
         * |     0x1     |       Function 1        |
         * +-------------+-------------------------+
         * |     0x0     |       Function 0        |
         * +-------------+-------------------------+
         */
        unsigned char       P20MUX      : 2;
        /**
         * These bits select the peripheral I/O function to be assigned to each port pin.
         * .P21MUX[1:0]
         *  These bits select the peripheral I/O function to be assigned to each port pin.
         * +-------------+-------------------------+
         * | P2nMUX[1:0] | Peripheral I/O function |
         * +-------------+-------------------------+
         * |     0x3     |       Function 3        |
         * +-------------+-------------------------+
         * |     0x2     |       Function 2        |
         * +-------------+-------------------------+
         * |     0x1     |       Function 1        |
         * +-------------+-------------------------+
         * |     0x0     |       Function 0        |
         * +-------------+-------------------------+
         */
        unsigned char       P21MUX      : 2;
        unsigned char       reserved_1  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} P2FNCSEL0_REG;

/// OSC1 Load Capacitance 1 Control Register
typedef union OSC1CL1_REG_TAG {
    unsigned char   reg;
    struct OSC1CL1_REG_BIT_TAG {
        /**
         * These bits adjust the load capacitance 1 (CG1, CD1) built into the OSC1 oscillator circuit.
         * .CLADJ[7:0]
         *  These bits adjust the load capacitance 1 (CG1, CD1) built into the OSC1 oscillator circuit.
         * +--------------------+-------------+
         * | OSC1CL1.CLADJ[7:0] | Capacitance |
         * +--------------------+-------------+
         * |     0x7F           |     Max.    |
         * +--------------------+             +
         * |     0x7E           |             |
         * +--------------------+             +
         * |     ...            |             |
         * +--------------------+             +
         * |     0x00           |             |
         * +--------------------+             +
         * |     0xFF           |             |
         * +--------------------+             +
         * |     ...            |             |
         * +--------------------+             +
         * |     0x81           |             |
         * +--------------------+             +
         * |     0x80           |    Min.     |
         * +--------------------+-------------+
         */
        unsigned char       CLADJ       : 8;
    } bit;
    unsigned char   byte_reg[1];
} OSC1CL1_REG;

/// OSC1 Load Capacitance 2 Control Register
typedef union OSC1CL2_REG_TAG {
    unsigned char   reg;
    struct OSC1CL2_REG_BIT_TAG {
        /**
         * This bit disconnects the load capacitance 2 (CG2, CD2) built into the OSC1 oscillator circuit.
         * .CLCUT
         *   This bit disconnects the load capacitance 2 (CG2, CD2) built into the OSC1 oscillator circuit.
         *    1 (R/W): Normal
         *    0 (R/W): Disconnect
         */
        unsigned char       CLCUT       : 1;
        unsigned char       reserved_1  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} OSC1CL2_REG;

/// VOSC Regulator Control Register
typedef union VOSCCTL_REG_TAG {
    unsigned char   reg;
    struct VOSCCTL_REG_BIT_TAG {
        /**
         * This bit controls the load current of the VOSC regulator.
         * .IUP
         *   This bit controls the load current of the VOSC regulator.
         *    1 (R/W): Normal
         *    0 (R/W): Load current On
         */
        unsigned char       IUP         : 1;
        /**
         * This bit controls the driving capability of the VOSC regulator.
         * .ION
         *   This bit controls the driving capability of the VOSC regulator.
         *    1 (R/W): Improve driving capability
         *    0 (R/W): Normal
         */
        unsigned char       ION         : 1;
        unsigned char       reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} VOSCCTL_REG;

/// VOSC Reference Current Adjustment Register
typedef union VOSCTRIM_REG_TAG {
    unsigned char   reg;
    struct VOSCTRIM_REG_BIT_TAG {
        /**
         * These bits adjust the reference current amount of the VOSC regulator.
         * .XFIB[5:0]
         *   Set the value read from the CLGOSC1TRIM.XFIB[5:0] bits of the clock generator to these bits.
         */
        unsigned char       XFIB        : 6;
        unsigned char       reserved_1  : 2;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} VOSCTRIM_REG;

/// RTCLPCLK Control Register
typedef union F32KCTL_REG_TAG {
    unsigned char   reg;
    struct F32KCTL_REG_BIT_TAG {
        /**
         * This bit controls the RTCLPCLK output to the MCU core and FOUT32K pin.
         * .OEN
         *   This bit controls the driving capability of the VOSC regulator.
         *    1 (R/W): Output enable
         *    0 (R/W): Output disable
         */
        unsigned char       OEN         : 1;
        unsigned char       reserved_1  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[1];
} F32KCTL_REG;

#ifdef __cplusplus
}
#endif

#endif // RTCLP_REG_H__
