/************************************************************************//**
 *
 * @file    c17_epdc.c
 *
 * @brief   This is EPDC driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#include "crt0.h"
#include "c17_misc.h"
#include "c17_epdc.h"
#include "c17_init_config.h"

#if defined(C17_PE_EPDC)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector20_handler, c17intEpdc);		// EPDC interrupt
#endif /* CRT0_LINKED */

/* --- global variable --- */
volatile unsigned int c17epdcDisplayUpdate = 0;				///< Display Update interrupt.

/* --- function declaration --- */
static void initEpdcSegData(void);
static void initEpdcWaveTiming(void);
static void waitUseSystemClock(unsigned short x, unsigned long sysClk);

/************************************************************************//**
 * @brief   This function initializes EPDC driver.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initEpdc(void)
{
	/// EPDC Setting
	if((EPDC_SET_INIT & 0x20) == 0x20)
	{
		/// Initialize EPDC.
		c17setEpdc((EPDC_SET_INIT & 0x18)>>3, (EPDC_SET_INIT & 0x07));

		/// Initialize the interrupt.
		ILVEPDC = INTERRUPT_LEVEL_EPDC;				/// - Set interrupt level.
	}
}


/************************************************************************//**
 * @brief   Enable EPDC interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntEpdc(void)
{
	EPDINTF_REG		epdintf_set;
	EPDINTE_REG		epdinte_set;

	/* EPDINTF:EPDC Interrupt Flag Register */
	{
		epdintf_set.reg = 0;
		epdintf_set.bit.DUPDIF = 1;  			///- Clear interrupt flag.
		EPDINTF = epdintf_set.reg;
	}
	/*EPDINTE:EPDC Interrupt Enable Register */
	{
		epdinte_set.reg = 0;
		epdinte_set.bit.DUPDIE = 1;  			///- Disable Display update interrupts.
		EPDINTE = epdinte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Disable EPDC interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntEpdc(void)
{
	EPDINTF_REG		epdintf_set;
	EPDINTE_REG		epdinte_set;

	/* EPDINTF:EPDC Interrupt Flag Register */
	{
		epdintf_set.reg = 0;
		epdintf_set.bit.DUPDIF = 1;  			///- Clear interrupt flag.
		EPDINTF = epdintf_set.reg;
	}
	/*EPDINTE:EPDC Interrupt Enable Register */
	{
		epdinte_set.reg = 0;
		epdinte_set.bit.DUPDIE = 0;  			///- Disable Display update interrupts.
		EPDINTE = epdinte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Enable EPDC Module.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableEpdcModule(void)
{
	EPDCTL_REG	epdctl_set;

	/* EPDCTL:EPDC Control Register */
	{
		epdctl_set.reg = EPDCTL;
		epdctl_set.bit.DIS = 1;				/// - Enable discharging of drive pins
		epdctl_set.bit.MODEN = 1;			/// - EPDC timing clock enable
		EPDCTL = epdctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   disable EPDC Timing Clock.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableEpdcModule(void)
{
	EPDCTL_REG	epdctl_set;

	/* EPDCTL:EPDC Control Register */
	{
		epdctl_set.reg = EPDCTL;
		epdctl_set.bit.DIS = 0;				/// - Disable discharging of drive pins
		epdctl_set.bit.MODEN = 0;			/// - EPDC timing clock disable
		EPDCTL = epdctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function changes Operating Mode.
 *
 * @param   state
 *  @li @c  C17_EPDC_CNT_WAVE        Mode : Waveform mode
 *  @li @c  C17_EPDC_CNT_DIRECT      Mode : Direct control mode
 *
 * @retval  none
 ***************************************************************************/
void c17setEpdcOperatingMode(unsigned char mode)
{
	EPDDSP_REG		epddsp_set;

	/* EPDDSP:EPDC Display Control Register. */
	{
		epddsp_set.reg = EPDDSP;
		epddsp_set.bit.DIRCTL = mode;		/// Operating Mode Select.
		EPDDSP = epddsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function changes Display Mode.
 *
 * @param   state
 *  @li @c  C17_EPDC_DSPMD_NORMAL   Mode : Normal display
 *  @li @c  C17_EPDC_DSPMD_REV      Mode : Reverse display
 *  @li @c  C17_EPDC_DSPMD_WHITE    Mode : All white display
 *  @li @c  C17_EPDC_DSPMD_BLACK    Mode : All black display
 *
 * @retval  none
 ***************************************************************************/
void c17setEpdcDisplayMode(unsigned char mode)
{
	EPDDSP_REG		epddsp_set;

	/* EPDDSP:EPDC Display Control Register. */
	{
		epddsp_set.reg = EPDDSP;
		epddsp_set.bit.DSPMD = mode;		/// Display Mode Select.
		EPDDSP = epddsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function gets Display Mode.
 *
 * @param   none
 *
 * @retval  C17_EPDC_DSPMD_NORMAL   Mode : Normal display
 * @retval  C17_EPDC_DSPMD_REV      Mode : Reverse display
 * @retval  C17_EPDC_DSPMD_WHITE    Mode : All white display
 * @retval  C17_EPDC_DSPMD_BLACK    Mode : All black display
 ***************************************************************************/
unsigned char c17getEpdcDisplayMode(void)
{
	EPDDSP_REG		epddsp_set;

	/* EPDDSP:EPDC Display Control Register. */
	{
		epddsp_set.reg = EPDDSP;
	}

	/// return EPDC display mode.
	return epddsp_set.bit.DSPMD;
}


/************************************************************************//**
 * @brief   This function changes Display Mode.
 *
 * @param   seghz    Segment/back plane Hi-Z control
 * @param   tphz     Top plane Hi-Z control
 * @param   tp       Top plane control data
 * @param   bp       Back plane display data
 *
 * @retval  none
 ***************************************************************************/
void c17setEpdcTopBackPlane(unsigned short seghz,  unsigned short tphz, unsigned short tp, unsigned short bp)
{
	EPDTPBP_REG		epdtpbp_set;

	/* EPDC_PLNDAT:EPDC Top/Back Plane Data Register. */
	{
		epdtpbp_set.reg = EPDTPBP;
		epdtpbp_set.bit.SEGHZ = seghz;			/// Segment/back plane Hi-Z control.
		epdtpbp_set.bit.TPHZ = tphz;			/// Top plane Hi-Z control.
		epdtpbp_set.bit.TP = tp;				/// Top plane control data.
		epdtpbp_set.bit.BP = bp;				/// Back plane display data.
		EPDTPBP = epdtpbp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function set Segment output data.
 *
 * @param   addr
 *  @li @c  EPDC_SEGDATA0_ADDR   Display Data Memory 0.
 *  @li @c  EPDC_SEGDATA1_ADDR   Display Data Memory 1.
 *  @li @c  EPDC_SEGDATA2_ADDR   Display Data Memory 2.
 *  @li @c  EPDC_SEGDATA3_ADDR   Display Data Memory 3.
 * @param   data                Segment output data.
 *
 * @retval	none
 ***************************************************************************/
void c17setEpdcSegData(unsigned short addr, unsigned short data)
{
	unsigned short *segdata = (unsigned short *)EPDC_SEGDATA_TOP_ADDR + (addr/2);

	/* EPDC_SEGDATN:EPDC Segment Data Registers. */
	{
		*segdata = data;
	}
}


/************************************************************************//**
 * @brief   This function gets Segment output data.
 *
 * @param   addr
 *  @li @c  EPDC_SEGDATA0_ADDR   Display Data Memory 0.
 *  @li @c  EPDC_SEGDATA1_ADDR   Display Data Memory 1.
 *  @li @c  EPDC_SEGDATA2_ADDR   Display Data Memory 2.
 *  @li @c  EPDC_SEGDATA3_ADDR   Display Data Memory 3.
 *
 * @retval   data               Segment output data.
 ***************************************************************************/
unsigned short c17getEpdcSegData(unsigned short addr)
{
	unsigned short *segdata = (unsigned short *)EPDC_SEGDATA_TOP_ADDR + (addr/2);

	/* EPDC_SEGDATN:EPDC Segment Data Registers. */
	return *segdata;

}


/************************************************************************//**
 * @brief   This function set check pattern.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17setEpdcCheckPattern_100(void)
{
	unsigned short *segdata = (unsigned short *)EPDC_SEGDATA_TOP_ADDR;

	/// Display Data "100%".
	*segdata++ = 0xCFFF;
	*segdata++ = 0xE3FF;
	*segdata   = 0x0203;
}


/************************************************************************//**
 * @brief   This function set check pattern.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17setEpdcCheckPattern_000(void)
{
	unsigned short *segdata = (unsigned short *)EPDC_SEGDATA_TOP_ADDR;

	/// Display Data "000%".
	*segdata++ = 0xCFFF;
	*segdata++ = 0xF3FF;
	*segdata   = 0x02FF;
}


/************************************************************************//**
 * @brief   This function set waveform data.
 *
 * @param   addr
 *  @li @c  EPDC_WAVE0_ADDR   	Display waveform memory 0.
 *              :
 *  @li @c  EPDC_WAVE31_ADDR   	Display waveform memory 31.
 * @param   data                waveform data.
 *
 * @retval	none
 ***************************************************************************/
void c17setEpdcWaveform(unsigned short addr, unsigned short data)
{
	unsigned short *wavefome = (unsigned short *)EPDC_WAVE_TOP_ADDR + (addr/2);

	/* EPDC_WAVEN:EPDC Waveform Timing Set N Registers. */
	{
		*wavefome = data;
	}
}


/************************************************************************//**
 * @brief   This function gets waveform data.
 *
 * @param   addr
 *  @li @c  EPDC_WAVE0_ADDR   	Display waveform memory 0.
 *              :
 *  @li @c  EPDC_WAVE31_ADDR   	Display waveform memory 31.
 *
 * @retval   data               waveform data.
 ***************************************************************************/
unsigned short c17getEpdcWaveform(unsigned short addr)
{
	unsigned short *wavefome = (unsigned short *)EPDC_WAVE_TOP_ADDR + (addr/2);

	/* EPDC_WAVEN:EPDC Waveform Timing Set N Registers. */
	return *wavefome;

}


/************************************************************************//**
 * @brief   This function Display Update Trigger.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startEpdc(void)
{
	EPDDSP_REG		epddsp_set;
	EPDINTF_REG		epdintf_set;

	/// Wait display update operation.
	/* EPDINTF:EPDC Interrupt Flag Register */
	{
		epdintf_set.reg = EPDINTF;
	}

	while(epdintf_set.bit.UPDST == 1)
	{
		/**
		 * Wait display update status idle in wave mode.
		 * @todo Add time-out processing as needed.
		 */
		{
			epdintf_set.reg = EPDINTF;
		}
	}

	/* EPDDSP:EPDC Display Control Register. */
	{
		epddsp_set.reg = EPDDSP;
		epddsp_set.bit.UPDTRG = 1;		/// Starts display update sequence.
		EPDDSP = epddsp_set.reg;
	}

}


/************************************************************************//**
 * @brief   This function sets EPDC.
 *
 * @param   epdTClkSrc      Clock Source.
 *  @li @c  C17_EPDC_CLKSRC_IOSC
 *  @li @c  C17_EPDC_CLKSRC_OSC1
 *  @li @c  C17_EPDC_CLKSRC_OSC3
 *  @li @c  C17_EPDC_CLKSRC_EXOSC
 * @param   epdTClkDiv      Clock Division
 *  @li @c  C17_EPDC_CLKDIV_OSC1_1
 *  @li @c  C17_EPDC_CLKDIV_OSC1_2
 *  @li @c  C17_EPDC_CLKDIV_OSC1_4
 *  @li @c  C17_EPDC_CLKDIV_OSC1_8
 *  @li @c  C17_EPDC_CLKDIV_OSC1_16
 *  @li @c  C17_EPDC_CLKDIV_OSC1_32
 *  @li @c  C17_EPDC_CLKDIV_OSC1_64
 *  @li @c  C17_EPDC_CLKDIV_OSC1_128
 *  @li @c  C17_EPDC_CLKDIV_TIM_128
 *  @li @c  C17_EPDC_CLKDIV_TIM_256
 *  @li @c  C17_EPDC_CLKDIV_TIM_512
 *  @li @c  C17_EPDC_CLKDIV_TIM_1024
 *  @li @c  C17_EPDC_CLKDIV_TIM_2048
 *  @li @c  C17_EPDC_CLKDIV_TIM_4096
 *  @li @c  C17_EPDC_CLKDIV_TIM_8192
 *  @li @c  C17_EPDC_CLKDIV_TIM_16384
 *
 * @retval  none
 ***************************************************************************/
void c17setEpdc(unsigned short epdcTClkSrc, unsigned short epdcTClkDiv)
{
	EPDTIMCLK_REG	epdtimclk_set;
	EPDDBLCLK_REG	epddblclk_set;
	EPDBSTCLK_REG	epdbstclk_set;
	EPDPWR0_REG		epdpwr0_set;
	EPDPWR1_REG		epdpwr1_set;
	EPDDSP_REG		epddsp_set;
	EPDINTF_REG		epdintf_set;
	EPDINTE_REG		epdinte_set;
	EPDTPBP_REG		epdtpbp_set;

	P0FNCSEL_REG	p0fncsel_set;
	P0MODSEL_REG	p0modsel_set;

	/// - EPDC output function to the ports.
	/*
	 *  P00 : EPDCLK
	 *  P01 : EPDTRG
	 */
	/* P0FNCSEL:P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P01MUX = 0;		/// - P03 : EPDTRG.
		p0fncsel_set.bit.P00MUX = 0;		/// - P02 : EPDCLK.
		P0FNCSEL = p0fncsel_set.reg;
	}

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P01SEL = 1;		/// - P01 : Enable peripheral I/O function.
		p0modsel_set.bit.P00SEL = 1;		/// - P00 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}

	/* EPDTIMCLK:EPDC Timing Clock Control Register */
	{
	    /**
	     * @todo Change clock source & Clock division ratio selection
	     */
		epdtimclk_set.bit.DBRUN = 1;				/// - Clock supplied in DEBUG mode.
		epdtimclk_set.bit.CLKDIV = epdcTClkDiv;		/// - EPDC timing clock division ratio select
		epdtimclk_set.bit.CLKSRC = epdcTClkSrc;		/// - EPDC timing clock source select
		EPDTIMCLK = epdtimclk_set.reg;
	}

	/* EPDDSP:EPDC Display Control Register. */
	{
		epddsp_set.reg = 0;
		epddsp_set.bit.SELCUR = 0;					/// Next SEG/BP data select
		epddsp_set.bit.DIRCTL = 0;					/// Wave/direct mode select
		epddsp_set.bit.DSPMD = 0;					/// Set normal Display Mode
		EPDDSP = epddsp_set.reg;
	}

	/* EPDINTF:EPDC Interrupt Flag Register */
	{
		epdintf_set.reg = 0;
		epdintf_set.bit.DUPDIF = 1;  			///- Clear interrupt flag.
		EPDINTF = epdintf_set.reg;
	}
	/*EPDINTE:EPDC Interrupt Enable Register */
	{
		epdinte_set.reg = 0;
		epdinte_set.bit.DUPDIE = 0;  			///- Disable Display update interrupts.
		EPDINTE = epdinte_set.reg;
	}

	/* EPDC_PLNDAT:EPDC Top/Back Plane Data Register. */
	{
		epdtpbp_set.reg = 0;
		epdtpbp_set.bit.SEGHZ = 0;			/// Segment/back plane Hi-Z control.
		epdtpbp_set.bit.TPHZ = 0;			/// Top plane Hi-Z control.
		epdtpbp_set.bit.TP = 0;				/// Top plane control data.
		epdtpbp_set.bit.BP = 0;				/// Back plane display data.
		EPDTPBP = epdtpbp_set.reg;
	}

	/* EPDC_DCLK:EPDC Doubler Clock Control Register. */
	{
		/**
	     * @todo Change EPDC Doubler Clock Control Register
	     */
		epddblclk_set.bit.DBRUN = 1;				/// - Clock supplied in DEBUG mode.
		epddblclk_set.bit.CLKDIV = 3;				/// EPDC Doubler clock division ratio select(OSC1, 1/8).
		epddblclk_set.bit.CLKSRC = 1;				/// EPDC Doubler clock source select(OSC1).
		EPDDBLCLK = epddblclk_set.reg;
	}

	/* EPDC_BCLK:EPDC Booster Clock Control Register. */
	{
		/**
	     * @todo Change EPDC Booster Clock Control Register
	     */
		epdbstclk_set.bit.DBRUN = 1;				/// - Clock supplied in DEBUG mode.
		epdbstclk_set.bit.CLKDIV = 3;				/// EPDC Booster clock division ratio select(OSC1, 1/8).
		epdbstclk_set.bit.CLKSRC = 1;				/// EPDC Booster clock source select(OSC1).
		EPDBSTCLK = epdbstclk_set.reg;
	}

	/* EPDC_PWR0:EPDC Power Control Register 0. */
	{
		/**
	     * @todo Change EPDC Power Control Register 0
	     */
		epdpwr0_set.bit.DBSRT = 0;					/// VE1-VE2 Doubler short(open).
		epdpwr0_set.bit.DBON = 1;					/// VE1 Doubler on/off(on).
		epdpwr0_set.bit.VECONT = 0x8;				/// VE regulator contrast setting(level 8).
		epdpwr0_set.bit.HVLDVE = 0;					/// VE heavy load protection mode(off).
		epdpwr0_set.bit.VESEL = 0;					/// Reference voltage select(VE1).
		epdpwr0_set.bit.VEON = 1;					/// VE regulator on/off(on).
		EPDPWR0 = epdpwr0_set.reg;
	}

	/// wait(35ms over)
	waitUseSystemClock(35, EPDC_WAIT_IOSC_CLK);

	/* EPDC_PWR1:EPDC Power Control Register 1. */
	{
		/**
	     * @todo Change EPDC Power Control Register 1
	     */
		epdpwr1_set.bit.HVDCONT = 0x6;				/// VH regulator contrast setting(level 6).
		epdpwr1_set.bit.HVDSEL = 0;					/// VH regulator range select(15V).
		epdpwr1_set.bit.HVLDHVD = 0;				/// VH heavy load protection mode(off).
		epdpwr1_set.bit.HVDON = 1;					/// VH regulator on/off(on).
		epdpwr1_set.bit.BSTPLD = 0;					/// Booster pull-down on/off(off).
		epdpwr1_set.bit.BSTON = 1;					/// Booster on/off(on).
		EPDPWR1 = epdpwr1_set.reg;
	}

	/// wait(5ms over)
	waitUseSystemClock(5, EPDC_WAIT_IOSC_CLK);

	/// initializes Segment Data.
	initEpdcSegData();

	/// initializes Waveform Timing Set.
	initEpdcWaveTiming();

}

/************************************************************************//**
 * @brief   This function select Pin Assignment.
 *
 * @param   state
 *  @li @c  0 to 3 : Pin Assignment
 *
 * @retval  none
 ***************************************************************************/
void c17setEpdcPinAssignmentSelect(unsigned char pinpos)
{
	EPDPOS_REG		epdpos_set;

	/* EPDPOS: EPDC Pin Assignment Select Register. */
	{
		epdpos_set.reg = EPDPOS;
		epdpos_set.bit.PINPOS = pinpos;		/// Pin Assignment Select.
		EPDPOS = epdpos_set.reg;
	}
}
/************************************************************************//**
 * @brief   EPDC Interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intEpdc(void)
{
	EPDINTF_REG		epdintf_set;

	/* EPDC_INT:EPDC Interrupt Control Register. Flag read & clear */
	{
		epdintf_set.reg = EPDINTF;
		EPDINTF = epdintf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(epdintf_set.bit.DUPDIF == 1)
	{
	   /**
		* @todo add interrupt function of Display Update interrupt.
		*/
		c17epdcDisplayUpdate = 1;

	}

}


/* ----- static ----- */

/************************************************************************//**
 * @brief   This function initializes EPDC Segment Data.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void initEpdcSegData(void)
{
	unsigned short *segdata = (unsigned short *)EPDC_SEGDATA_TOP_ADDR;
	unsigned short	i;

	for(i = 0; i < EPDC_SEGDATA_SIZE; i++){
		*segdata++ = 0x0000;
	}
}


/************************************************************************//**
 * @brief   This function initializes EPDC Waveform Timing Set.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void initEpdcWaveTiming(void)
{
	unsigned short *wavedata = (unsigned short *)EPDC_WAVE_TOP_ADDR;
	unsigned short	i;

	for(i = 0; i < EPDC_WAVE_SIZE; i++){
		*wavedata++ = 0x0000;
	}
}


/************************************************************************//**
 * @brief   This function is Wait. X[msec] while running with system clock.
 *
 * @param   x         counter value
 * @param   sysClk    system clock value
 *
 * @retval  none
 ***************************************************************************/
static void waitUseSystemClock(unsigned short x, unsigned long sysClk)
{
	unsigned short	mil_ct;		// x milli sec counter
	unsigned long	clk_ct;		// system clock 1 milli sec counter

	if(x == 0)
	{
		for(mil_ct = 0; mil_ct < 0x100; mil_ct++){
			;
		}
	}
	else
	{
		mil_ct = x;

		while(mil_ct > 0){
			clk_ct = sysClk / 1000 / 7;

			while(clk_ct > 0){
				clk_ct--;
			}
			mil_ct--;
		}
	}
}


#endif

