/************************************************************************//**
 *
 * @file    c17_snda.c
 *
 * @brief   This is SNDA driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Change source code.
 *          - 01 Nov. 2018    1.20
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_snda.h"
#include "c17_init_config.h"

#if defined(C17_PE_SNDA)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
     defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
C17_INTERRUPT_HANDLER(_vector20_handler, c17intSnda);		// SNDA interrupt
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
C17_INTERRUPT_HANDLER(_vector17_handler, c17intSnda);		// SNDA interrupt
#endif
#if defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W15)
C17_INTERRUPT_HANDLER(_vector18_handler, c17intSnda);		// SNDA interrupt
#endif

#endif /* CRT0_LINKED */


/* --- definition --- */

/* --- global variable --- */
unsigned short melody_num;					///< Melody data Count.
unsigned short sndaMelodyRun;				///< Melody data flag.
unsigned short sndaMelodyDataSet;			///< Melody data set enable flag.
unsigned short *add_melody_data;			///< Melody data address.

/* --- function declaration --- */
static void setDutyFreq(float duty, float freq);

/************************************************************************//**
 * @brief   This function initializes SNDA driver.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initSnda(void)
{
	/// Stop SNDA.
	c17stopSnda();

	/// SDNA Setting
	if((SNDA_SET_INIT & 0x20) == 0x20)
	{
		/// Initialize SNDA.
		c17setSnda((SNDA_SET_INIT & 0x18)>>3, (SNDA_SET_INIT & 0x07));

		/// Initialize the interrupt.
		ILVSNDA_0 = INTERRUPT_LEVEL_SNDA;		/// - Set interrupt level.
	}
}


#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
/************************************************************************//**
 * @brief   This function sets SNDA.
 *
 * @param   sndClkSrc      Clock Source.
 *  @li @c  C17_SND_CLKSRC_IOSC
 *  @li @c  C17_SND_CLKSRC_OSC1
 *  @li @c  C17_SND_CLKSRC_OSC3
 *  @li @c  C17_SND_CLKSRC_EXT
 * @param   sndClkDiv      Clock Division
 *  @li @c  C17_SND_CLKDIV_1
 *  @li @c  C17_SND_CLKDIV_16
 *  @li @c  C17_SND_CLKDIV_32
 *  @li @c  C17_SND_CLKDIV_64
 *  @li @c  C17_SND_CLKDIV_128
 *  @li @c  C17_SND_CLKDIV_256
 *  @li @c  C17_SND_CLKDIV_512
 *
 * @retval  none
 ***************************************************************************/
#else
/************************************************************************//**
 * @brief   This function sets SNDA.
 *
 * @param   sndClkSrc      Clock Source.
 *  @li @c  C17_SND_CLKSRC_IOSC
 *  @li @c  C17_SND_CLKSRC_OSC1
 *  @li @c  C17_SND_CLKSRC_OSC3
 *  @li @c  C17_SND_CLKSRC_EXT
 * @param   sndClkDiv      Clock Division
 *  @li @c  C17_SND_CLKDIV_1
 *  @li @c  C17_SND_CLKDIV_4
 *  @li @c  C17_SND_CLKDIV_8
 *  @li @c  C17_SND_CLKDIV_16
 *  @li @c  C17_SND_CLKDIV_32
 *  @li @c  C17_SND_CLKDIV_64
 *  @li @c  C17_SND_CLKDIV_128
 *
 * @retval  none
 ***************************************************************************/
#endif
void c17setSnda(unsigned short sndClkSrc, unsigned short sndClkDiv)
{
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_F63)
	P0FNCSEL_REG	p0fncsel_set;
	P0MODSEL_REG	p0modsel_set;
#endif
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18)
	P1FNCSEL_REG	p1fncsel_set;
	P1MODSEL_REG	p1modsel_set;
#endif
#if defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
	P2FNCSEL_REG	p2fncsel_set;
	P2MODSEL_REG	p2modsel_set;
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
	P3FNCSEL_REG	p3fncsel_set;
	P3MODSEL_REG	p3modsel_set;
#endif
#if defined(C17_MCUSEL_M40)
	P4FNCSEL_REG	p4fncsel_set;
	P4MODSEL_REG	p4modsel_set;
#endif

	SNDCLK_REG		sndclk_set;
	SNDCTL_REG		sndctl_set;
	SNDSEL_REG		sndsel_set;

	SNDINTF_REG		sndintf_set;
	SNDINTE_REG		sndinte_set;

#if defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_F63)
	/// - SNDA output function to the ports.
	/*
	 *  P02 : BZOUT
	 *  P03 : #BZOUT
	 */
	/* P0FNCSEL:P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P03MUX = 0;		/// - P03 : #BZOUT.
		p0fncsel_set.bit.P02MUX = 0;		/// - P02 : BZOUT.
		P0FNCSEL = p0fncsel_set.reg;
	}

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P03SEL = 1;		/// - P03 : Enable peripheral I/O function.
		p0modsel_set.bit.P02SEL = 1;		/// - P02 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04)
	/// - SNDA output function to the ports.
	/*
	 *  P04 : BZOUT
	 *  P05 : #BZOUT
	 */
	/* P0FNCSEL:P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P05MUX = 0;		/// - P05 : #BZOUT.
		p0fncsel_set.bit.P04MUX = 0;		/// - P04 : BZOUT.
		P0FNCSEL = p0fncsel_set.reg;
	}

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P05SEL = 1;		/// - P05 : Enable peripheral I/O function.
		p0modsel_set.bit.P04SEL = 1;		/// - P04 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16)
	/// - SNDA output function to the ports.
	/*
	 *  P06 : BZOUT
	 *  P07 : #BZOUT
	 */
	/* P0FNCSEL:P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P07MUX = 0;		/// - P07 : #BZOUT.
		p0fncsel_set.bit.P06MUX = 0;		/// - P06 : BZOUT.
		P0FNCSEL = p0fncsel_set.reg;
	}

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P07SEL = 1;		/// - P07 : Enable peripheral I/O function.
		p0modsel_set.bit.P06SEL = 1;		/// - P06 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23)
	/// - SNDA output function to the ports.
	/*
	 *  P14 : BZOUT
	 *  P13 : #BZOUT
	 */
	/* P1FNCSEL:P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P14MUX = 0;		/// - P14 : BZOUT.
		p1fncsel_set.bit.P13MUX = 0;		/// - P13 : #BZOUT.
		P1FNCSEL = p1fncsel_set.reg;
	}

	/* P1MODSEL:P1 Port Mode Select Register */
	{
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P14SEL = 1;		/// - P14 : Enable peripheral I/O function.
		p1modsel_set.bit.P13SEL = 1;		/// - P13 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18)
	/// - SNDA output function to the ports.
	/*
	 *  P15 : BZOUT
	 *  P14 : #BZOUT
	 */
	/* P1FNCSEL:P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P15MUX = 0;		/// - P15 : BZOUT.
		p1fncsel_set.bit.P14MUX = 0;		/// - P14 : #BZOUT.
		P1FNCSEL = p1fncsel_set.reg;
	}

	/* P1MODSEL:P1 Port Mode Select Register */
	{
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P15SEL = 1;		/// - P15 : Enable peripheral I/O function.
		p1modsel_set.bit.P14SEL = 1;		/// - P14 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
	/// - SNDA output function to the ports.
	/*
	 *  P23 : BZOUT
	 *  P22 : #BZOUT
	 */
	/* P2FNCSEL:P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P23MUX = 0;		/// - P23 : BZOUT.
		p2fncsel_set.bit.P22MUX = 0;		/// - P22 : #BZOUT.
		P2FNCSEL = p2fncsel_set.reg;
	}

	/* P2MODSEL:P2 Port Mode Select Register */
	{
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P23SEL = 1;		/// - P23 : Enable peripheral I/O function.
		p2modsel_set.bit.P22SEL = 1;		/// - P22 : Enable peripheral I/O function.
		P2MODSEL = p2modsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
	/// - SNDA output function to the ports.
	/*
	 *  P33 : BZOUT
	 *  P34 : #BZOUT
	 */
	/* P3FNCSEL:P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P34MUX = 0;		/// - P34 : #BZOUT.
		p3fncsel_set.bit.P33MUX = 0;		/// - P33 : BZOUT.
		P3FNCSEL = p3fncsel_set.reg;
	}

	/* P3MODSEL:P3 Port Mode Select Register */
	{
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P34SEL = 1;		/// - P34 : Enable peripheral I/O function.
		p3modsel_set.bit.P33SEL = 1;		/// - P33 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_M40)
	/// - SNDA output function to the ports.
	/*
	 *  P44 : BZOUT
	 *  P45 : #BZOUT
	 */
	/* P4FNCSEL:P4 Port Function Select Register */
	{
		p4fncsel_set.reg = P4FNCSEL;
		p4fncsel_set.bit.P45MUX = 0;		/// - P45 : #BZOUT.
		p4fncsel_set.bit.P44MUX = 0;		/// - P44 : BZOUT.
		P4FNCSEL = p4fncsel_set.reg;
	}

	/* P4MODSEL:P4 Port Mode Select Register */
	{
		p4modsel_set.reg = P4MODSEL;
		p4modsel_set.bit.P45SEL = 1;		/// - P45 : Enable peripheral I/O function.
		p4modsel_set.bit.P44SEL = 1;		/// - P44 : Enable peripheral I/O function.
		P4MODSEL = p4modsel_set.reg;
	}
#endif

	/* SNDCLK:SNDA Clock Control Register */
	{
		sndclk_set.reg = 0;
		sndclk_set.bit.DBRUN = 1;			/// - Clock supplied in DEBUG mode.
		sndclk_set.bit.CLKDIV = sndClkDiv;	/// - Set division ratio for clock source.
		sndclk_set.bit.CLKSRC = sndClkSrc;	/// - Set clock source.
		SNDCLK = sndclk_set.reg;
	}

	/* SNDCTL:SNDA Control Register */
	{
		sndctl_set.reg = 0;
		sndctl_set.bit.MODEN = 1;			/// - Enable SNDA operations.
		SNDCTL = sndctl_set.reg;
	}

	/* SNDSEL:SNDA Select Register */
	{
		sndsel_set.reg = 0;
	    /**
	     * @todo change SINV.
	     */
		sndsel_set.bit.SINV = 0;			/// - Set output pin drive mode(0=Direct drive mode / 1=Normal drive mode).
		sndsel_set.bit.MOSEL = 0;			/// - Set normal buzzer mode.
		SNDSEL = sndsel_set.reg;
	}

	/// Clear interrupt flags.
	/* SNDINTF:SNDA Interrupt Flag Register. */
	{
		sndintf_set.bit.EDIF = 1;			/// - Sound output completion interrupt flag.
		SNDINTF = sndintf_set.reg;
	}

	/// Disable interrupts.
	/* SNDINTE:SNDA Interrupt Enable Register. */
	{
		sndinte_set.bit.EMIE = 0;			/// - Sound buffer empty interrupt.
		sndinte_set.bit.EDIE = 0;			/// - Sound output completion interrupt.
		SNDINTE = sndinte_set.reg;
	}

	/* SNDCTL:SNDA Control Register */
	{
		sndctl_set.reg = 0;
		sndctl_set.bit.MODEN = 0;			/// - Disable SNDA operations.
		SNDCTL = sndctl_set.reg;
	}

	melody_num = 0;							///< Melody data Count.
	sndaMelodyRun = SNDA_MELODY_STOP;		///< Melody data flag.
	sndaMelodyDataSet = 0;					///< Melody data set enable flag.

}


/************************************************************************//**
 * @brief   Start Sound Generator.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startSnda(void)
{
	SNDCTL_REG		sndctl_set;

	/* SNDCTL:SNDA Control Register */
	{
		sndctl_set.reg = 0;
		sndctl_set.bit.MODEN = 1;			/// - Enable SNDA operations.
		SNDCTL = sndctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Stop Sound Generator.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopSnda(void)
{
	SNDCTL_REG		sndctl_set;
	SNDINTE_REG		sndinte_set;

	/// Stop sound output.
	c17stopSndaSound();

	/* SNDCTL:SNDA Control Register */
	{
		sndctl_set.reg = 0;
		sndctl_set.bit.MODEN = 0;			/// - Disable SNDA operations.
		SNDCTL = sndctl_set.reg;
	}

	/// Disable interrupts.
	/* SNDINTE:SNDA Interrupt Enable Register. */
	{
		sndinte_set.bit.EMIE = 0;			/// - Sound buffer empty interrupt.
		sndinte_set.bit.EDIE = 0;			/// - Sound output completion interrupt.
		SNDINTE = sndinte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Get sound mode.
 *
 * @param   none
 *
 * @retval  C17_SND_MODE_NORMAL     Sound mode : Normal.
 * @retval  C17_SND_MODE_ONE_SHOT   Sound mode : One shot.
 * @retval  C17_SND_MODE_MELODY     Sound mode : Melody.
 * @retval  C17_SND_MODE_OTHER      Sound mode : others.
 ***************************************************************************/
unsigned short c17getSndaMode(void)
{
	SNDSEL_REG		sndsel_set;

	unsigned short result = 0xff;

	/* SNDSEL:SNDA Select Register */
	{
		sndsel_set.reg = SNDSEL;
	}

	switch(sndsel_set.bit.MOSEL){
	case 0:
		result = C17_SND_MODE_NORMAL;
		break;
	case 1:
		result = C17_SND_MODE_ONE_SHOT;
		break;
	case 2:
		result = C17_SND_MODE_MELODY;
		break;
	}

	return result;
}


/************************************************************************//**
 * @brief   Stop sound output.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopSndaSound(void)
{
	SNDCTL_REG		sndctl_set;
	SNDINTF_REG		sndintf_set;

	unsigned char soundOut = 0;

	/* SNDINTF:SNDA Interrupt Flag Register read */
	{
		sndintf_set.reg = SNDINTF;
	}

	/// Check interrupt cause occurrence status.
	if((sndintf_set.bit.SBSY == 1) && (sndintf_set.bit.EDIF == 0))
	{
		soundOut = 1;
	}

	/* SNDCTL:SNDA Control Register */
	{
		sndctl_set.reg = SNDCTL;
		sndctl_set.bit.SSTP = 1;			/// - Stop sound output.
		SNDCTL = sndctl_set.reg;
	}

	/**
	 * @todo Add time-out processing as needed.
	 */
	while (sndctl_set.bit.SSTP == 1) {
		// Wait...
		sndctl_set.reg = SNDCTL;
	}

	/// Wait sound output completion interrupt flag.
	if (soundOut == 1) {
		/**
		 * @todo Add time-out processing as needed.
		 */
		while (sndintf_set.bit.EDIF == 0) {
			// Wait...
			sndintf_set.reg = SNDINTF;
		}
	}
}


/************************************************************************//**
 * @brief   Start SNDA(Normal).
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17runSndaNormal(float duty, float freq)
{
	SNDSEL_REG		sndsel_set;
	SNDINTF_REG		sndintf_set;
	SNDINTE_REG		sndinte_set;

	/* SNDSEL:SNDA Select Register */
	{
		sndsel_set.reg = SNDSEL;
		sndsel_set.bit.MOSEL = 0;			/// - Set normal buzzer mode.
		SNDSEL = sndsel_set.reg;
	}

	/// Clear interrupt flags.
	/* SNDINTF:SNDA Interrupt Flag Register. */
	{
		sndintf_set.bit.EDIF = 1;			/// - Sound output completion interrupt flag.
		SNDINTF = sndintf_set.reg;
	}

	/// Disable interrupts.
	/* SNDINTE:SNDA Interrupt Enable Register. */
	{
		sndinte_set.bit.EMIE = 0;			/// - Sound buffer empty interrupt.
		sndinte_set.bit.EDIE = 0;			/// - Sound output completion interrupt.
		SNDINTE = sndinte_set.reg;
	}

	/// Set buzzer signal duty ratio, buzzer signal frequency.
	setDutyFreq(duty, freq);

}


/************************************************************************//**
 * @brief   Run SNDA(one-shot).
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17runSndaOneShot(float duty, float freq, unsigned short duration)
{
	SNDSEL_REG		sndsel_set;
	SNDINTF_REG		sndintf_set;
	SNDINTE_REG		sndinte_set;

	/* SNDSEL:SNDA Select Register */
	{
		sndsel_set.reg = SNDSEL;
		sndsel_set.bit.STIM = duration;		/// - Set one-shot buzzer output duration.
		sndsel_set.bit.MOSEL = 1;			/// - Set One-shot buzzer mode.
		SNDSEL = sndsel_set.reg;
	}

	/// Clear interrupt flags.
	/* SNDINTF:SNDA Interrupt Flag Register. */
	{
		sndintf_set.bit.EDIF = 1;			/// - Sound output completion interrupt flag.
		SNDINTF = sndintf_set.reg;
	}

	/// Disable interrupts.
	/* SNDINTE:SNDA Interrupt Enable Register. */
	{
		sndinte_set.bit.EMIE = 0;			/// - Sound buffer empty interrupt.
		sndinte_set.bit.EDIE = 0;			/// - Sound output completion interrupt.
		SNDINTE = sndinte_set.reg;
	}

	/// Set buzzer signal duty ratio, buzzer signal frequency.
	setDutyFreq(duty, freq);

}


/************************************************************************//**
 * @brief   Run SNDA(Melody).
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17runSndaMelody(unsigned short melody_data[][4], unsigned short tempo)
{
	SNDSEL_REG		sndsel_set;
	SNDDAT_REG		snddat_set;
	SNDINTF_REG		sndintf_set;
	SNDINTE_REG		sndinte_set;

	/// Set melody data address.(second data)
	add_melody_data = (&melody_data[0][0] + 4);
	/// Set run status.
	sndaMelodyRun = SNDA_MELODY_RUN;
	/// Set Melody data set enable.
	sndaMelodyDataSet = 1;

	/* SNDSEL:SNDA Select Register */
	{
		sndsel_set.reg = SNDSEL;
		sndsel_set.bit.STIM = tempo;		/// - Set melody tempo.
		sndsel_set.bit.MOSEL = 2;			/// - Set melody mode.
		SNDSEL = sndsel_set.reg;
	}

	/// Melody mode first data set.
	/* SNDDAT:SNDA SNDA Data Register. */
	{
		snddat_set.bit.MDTI = melody_data[0][3];	/// - Set melody tie.
		snddat_set.bit.MDRS = melody_data[0][0];	/// - Set melody output type note or rest.
		snddat_set.bit.SLEN = melody_data[0][1];	/// - Set melody duration.
		snddat_set.bit.SFRQ = melody_data[0][2];	/// - Set melody scale.
		SNDDAT = snddat_set.reg;
	}

	/// Clear interrupt flags.
	/* SNDINTF:SNDA Interrupt Flag Register. */
	{
		sndintf_set.bit.EDIF = 1;			/// - Sound output completion interrupt flag.
		SNDINTF = sndintf_set.reg;
	}

	/// Enable interrupts.
	/* SNDINTE:SNDA Interrupt Enable Register. */
	{
		sndinte_set.bit.EMIE = 1;			/// - Sound buffer empty interrupt.
		sndinte_set.bit.EDIE = 1;			/// - Sound output completion interrupt.
		SNDINTE = sndinte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Set melody note data.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17setMelodyNote(unsigned short duration, unsigned short scale, unsigned short tie, unsigned short melody_data[][4])
{

	melody_data[melody_num][0] = C17_SND_TYPE_NOTE;		/// - Set melody output type note.
	melody_data[melody_num][1] = duration;				/// - Set melody duration.
	melody_data[melody_num][2] = scale;					/// - Set melody scale.
	melody_data[melody_num][3] = tie;					/// - Set melody tie.

	/// Set Melody data Count.
	melody_num++;
}


/************************************************************************//**
 * @brief   Set melody rest data.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17setMelodyRest(unsigned short duration, unsigned short melody_data[][4])
{

	melody_data[melody_num][0] = C17_SND_TYPE_REST;		/// - Set melody output type rest.
	melody_data[melody_num][1] = duration;				/// - Set melody duration.
	melody_data[melody_num][2] = 0;						/// - Set melody scale.
	melody_data[melody_num][3] = 0;						/// - Set melody tie.

	/// Set Melody data Count.
	melody_num++;
}


/************************************************************************//**
 * @brief   Run SNDA(Melody).
 *
 * @param   none
 *
 * @retval  SNDA_MELODY_STOP    Stop melody.
 * @retval  SNDA_MELODY_RUN     Running melody.
 ***************************************************************************/
int c17getSndaMelodyState(void)
{
	return 	sndaMelodyRun;
}


/************************************************************************//**
 * @brief   Clear Melody data.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17clearSndaMelodyData(unsigned short size, unsigned short melody_data[][4])
{
	int i;

	/// Clear Melody data.
	for (i = 0; i < size; i++) {
		melody_data[i][0] = 0;
		melody_data[i][1] = 0;
		melody_data[i][2] = 0;
		melody_data[i][3] = 0;
	}

	/// Clear Melody data Count.
	melody_num = 0;
	/// Clear Melody data flag.
	sndaMelodyRun = SNDA_MELODY_STOP;
	/// Set Melody data set disable.
	sndaMelodyDataSet = 0;

}


/************************************************************************//**
 * @brief   SNDA interrupt.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intSnda(void)
{
	SNDDAT_REG		snddat_set;
	SNDINTF_REG		sndintf_set;
	SNDINTE_REG		sndinte_set;

	/* SNDINTF:SNDA Interrupt Flag Register read & clear  */
	{
		sndintf_set.reg = SNDINTF;
		SNDINTF = sndintf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(sndintf_set.bit.EMIF == 1)
	{
		/// Output Melody.
		if (sndaMelodyRun == SNDA_MELODY_RUN) {
			if(sndaMelodyDataSet){
				/// Set melody data.
				/* SNDDAT:SNDA SNDA Data Register. */
				{
					snddat_set.bit.MDRS = *add_melody_data;
					add_melody_data++;
					snddat_set.bit.SLEN = *add_melody_data;
					add_melody_data++;
					snddat_set.bit.SFRQ = *add_melody_data;
					add_melody_data++;
					snddat_set.bit.MDTI = *add_melody_data;
					add_melody_data++;
					/// Last melody data check.
					if(snddat_set.reg == C17_SND_LAST_DATA){
						sndaMelodyDataSet = 0;
					}
					else{
						SNDDAT = snddat_set.reg;
					}
				}
			}
		}
	}

	/// Check interrupt cause occurrence status.
	if(sndintf_set.bit.EDIF == 1)
	{
		sndaMelodyRun = SNDA_MELODY_STOP;

		/// Disable interrupts.
		/* SNDINTE:SNDA Interrupt Enable Register. */
		{
			sndinte_set.bit.EMIE = 0;			/// - Sound buffer empty interrupt.
			sndinte_set.bit.EDIE = 0;			/// - Sound output completion interrupt.
			SNDINTE = sndinte_set.reg;
		}
	}
}


/************************************************************************//**
 * @brief   Set buzzer signal duty ratio, buzzer signal frequency.
 *          (when normal buzzer mode/one-shot buzzer mode)
 *
 * @param   duty       buzzer signal duty ratio.
 * @param   freq       buzzer signal frequency.
 *
 * @retval  none.
 ***************************************************************************/
static void setDutyFreq(float duty, float freq)
{
	SNDDAT_REG		snddat_set;
	unsigned short slen;
	unsigned short sfrq;

   /**
	* @todo change Clock Source, Clock Division. @c17_init_config.h
	*              #define SNDA_SET_INIT SNDA_INIT(enable, OSC1, CLKDIV_1)
	*              Please set the division ratio so that the frequency is in the near of 32768Hz.
	*/
	sfrq = ((CLK_SNDA / freq) - 1);

	slen = (((CLK_SNDA / freq) * (duty/100)) - 1);

	/// Set buzzer output signal duty ratio / frequency.
	/* SNDDAT:SNDA SNDA Data Register. */
	{
		snddat_set.reg = SNDDAT;
		snddat_set.bit.SLEN = slen;			/// - buzzer signal duty ratio.
		snddat_set.bit.SFRQ = sfrq;			/// - buzzer signal frequency.
		SNDDAT = snddat_set.reg;
	}

}

#endif
