/************************************************************************//**
 *
 * @file    main_epdc.c
 *
 * @brief   This is EPDC sample program for S1C17F63 demonstration.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018-2019. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *          - 20 Jun. 2019    1.10
 *              - Change source code.
 *          - 11 Sep. 2019    1.20
 *              - Change source code.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17F63 EPDC
 *
 * @section     intro Introduction
 * Sample code turns on the EPDC display.
 *
 * @subsection  step1   Step  1: Start OSC1.
 * @subsection  step2   Step  2: EPDC register initialize.
 * @subsection  step4   Step  3: Turn on EPD display by Wave Mode.
 * @subsection	step8   Step  4: Turn off EPD display.
 *                               End sample program.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17f63_sample" project.
 *
 ************************************************************************* */
#include <stdlib.h>
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_epdc.h"
#include "c17_init_config.h"

#ifdef C17_PE_EPDC

/* --- definition --- */

/* --- global variable --- */
extern volatile unsigned int c17epdcDisplayUpdate;

/* --- function declaration --- */

int main_epd(void);

static int runWave(void);

/************************************************************************//**
 * main function.
 *
 * @brief   1, EPDC register initialize.
 *
 *          2, start EPD display on.
 *
 *          3, Stop EPD display off.
 *
 * @return 	EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_epdc(void)
{
	/* <<< EPDC driver demonstration start >>> */

	int result = 0;

	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();

	/// start OSC1
	c17startClgOSC1();

	/// Initialize EPD.
	c17initEpdc();

	/// Enable EPDC module.
	c17enableEpdcModule();

	// EPD Wave Mode.
	result = runWave();

	/// Disable EPDC module.
	c17disableEpdcModule();

	/* <<< EPDC driver demonstration finish >>> */

	return result;
}


/************************************************************************//**
 *
 * @brief   runWave function.
 *
 * @return  EXIT_SUCCESS    Success.
 ***************************************************************************/
int runWave(void)
{

	/// Clear EPD Display Update Flag.
	c17epdcDisplayUpdate = 0;

	/// Set Operating Mode.
	c17setEpdcOperatingMode(C17_EPDC_CNT_WAVE);

	/// Select Pin Assignment for SVTmini17F63.
	c17setEpdcPinAssignmentSelect(0x03);

	/// Set Display Mode.
	c17setEpdcDisplayMode(C17_EPDC_DSPMD_NORMAL);

	/// Set EPD Top/Back Plane Data.
	c17setEpdcTopBackPlane(C17_PLN_SEGHZ_NORMAL, C17_PLN_TPHZ_NORMAL, C17_PLN_TP_LOW, C17_PLN_BP_LOW);

	/// Set EPD Segment Data.
	c17setEpdcCheckPattern_100();

	/// Set EPD Segment Data Registers.
	c17setEpdcWaveform(EPDC_WAVE0_ADDR, 0x0004);
	c17setEpdcWaveform(EPDC_WAVE1_ADDR, 0x1b3c);
	c17setEpdcWaveform(EPDC_WAVE2_ADDR, 0x0251);
	c17setEpdcWaveform(EPDC_WAVE3_ADDR, 0x1b8e);
	c17setEpdcWaveform(EPDC_WAVE4_ADDR, 0x027a);
	c17setEpdcWaveform(EPDC_WAVE5_ADDR, 0x1a09);
	c17setEpdcWaveform(EPDC_WAVE6_ADDR, 0x0a09);
	c17setEpdcWaveform(EPDC_WAVE7_ADDR, 0x8004);

	/// Enable Display Update Interrupt.
	c17enableIntEpdc();

	/// Starts display update sequence.
	c17startEpdc();

	while(c17epdcDisplayUpdate == 0){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

	}

	/// Clear EPD Display Update Flag.
	c17epdcDisplayUpdate = 0;
	/// Set Display Mode.
	c17setEpdcDisplayMode(C17_EPDC_DSPMD_WHITE);

	/// Starts display update sequence.
	c17startEpdc();

	while(c17epdcDisplayUpdate == 0){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

	}

	/// Clear EPD Display Update Flag.
	c17epdcDisplayUpdate = 0;
	/// Set Display Mode.
	c17setEpdcDisplayMode(C17_EPDC_DSPMD_BLACK);

	/// Starts display update sequence.
	c17startEpdc();

	while(c17epdcDisplayUpdate == 0){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

	}

	/// Clear EPD Display Update Flag.
	c17epdcDisplayUpdate = 0;
	/// Set Display Mode.
	c17setEpdcDisplayMode(C17_EPDC_DSPMD_REV);

	/// Starts display update sequence.
	c17startEpdc();

	while(c17epdcDisplayUpdate == 0){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

	}

	/// Clear EPD Display Update Flag.
	c17epdcDisplayUpdate = 0;
	/// Set Display Mode.
	c17setEpdcDisplayMode(C17_EPDC_DSPMD_WHITE);

	/// Starts display update sequence.
	c17startEpdc();

	while(c17epdcDisplayUpdate == 0){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

	}

	/// Disable Display Update Interrupt.
	c17disableIntEpdc();

	return EXIT_SUCCESS;

}
#endif

