/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is port sample program for S1C17F63 demonstration.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17F63 Port
 *
 * @section     intro Introduction
 * The sample code set P01 in the output and set P02 to the input.
 * Interrupt is generated by edge of P02.
 *
 * @subsection  step1   Step 1: Initialize PORT.
 * @subsection  step2   Step 2: Initialize P01,P02.
 *                      P01 set to the output.
 *                      P01 set the interrupt.
 *                      P02 set to the input.
 *                      P02 set the interrupt.
 *                      It set that interrupt generate from LOW by HIGH.
 * @subsection  step3   Step 3: P01 set to LOW or HIGH.
 *                      The interrupt of P02 is generated.
 *                      It confirm that P02 is HIGH.
 * @subsection  step4   Step 4: The setting of the interrupt of P02 set.
 *                      It set that interrupt generate from HIGH by LOW.
 * @subsection  step5   Step 5: P01 set to HIGH or LOW.
 *                      The interrupt of P02 is generated.
 *                      It confirm that P02 is LOW.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17f63_sample" project.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "c17_init_config.h"
#include "c17_port_f63.h"

#ifdef C17_PE_PORT
extern volatile unsigned short c17portP0xIntInputFlag;				///< Port P0x interrupt flag.

/* --- function declaration --- */

int main_port(void);


/************************************************************************//**
 * main function.
 *
 * @brief   1, initialize port.
 *
 *          2, P02 set that interrupt generate from LOW by HIGH.
 *
 *          3, P01 set to LOW or HIGH.
 *
 *          4, P02 set that interrupt generate from HIGH by LOW.
 *
 *          5, P01 set to HIGH or LOW.
 *
 * @return  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_port(void)
{
	/// These variable is for debugging.
	/// Warning comes out, but there is no problem.
	unsigned short	inputData;
	unsigned short	outputData;


	/* <<< Port demonstration start >>> */
	/// Initialize Port.
	c17initPort();

	/// P01 set the interrupt.
	c17disableIntP0x(1);
	/// Set output data(LOW=0).
	c17setOutputP0x(1, 0);

	c17portP0xIntInputFlag = 0;

	/// P02 set the interrupt(LOW by HIGH).
	c17enableIntP0x(2, C17_PORT_INT_EDGE_RISING);

	/// P01 Set output data(HIGH=1).
	c17setOutputP0x(1,1);

	/// Get P01 output data.
	outputData = c17getOutputP0x(1);

	/// Wait interrupt.(P02)
	while(1) {
		if ((c17portP0xIntInputFlag & 0x04) == 0x04) {
			break;
		}
		asm("halt");
		asm("nop");

	}
	/// Get P02 input data.
	inputData = c17getInputP0x(2);

	/// P02 set the interrupt(HIGH by LOW).
	c17enableIntP0x(2, C17_PORT_INT_EDGE_FALLING);

	c17portP0xIntInputFlag = 0;

	/// Set P01 output data(LOW).
	c17setOutputP0x(1, 0);


	/// Wait interrupt.
	while(1) {
		if ((c17portP0xIntInputFlag & 0x04) == 0x04) {
			break;
		}
		// Waits for it finishes sending over
		asm("halt");
		asm("nop");

	}

	/// Get P02 input data.
	inputData = c17getInputP0x(2);

	/// Get P01 output data.
	outputData = c17getOutputP0x(1);

	/// P02 disable the interrupt.
	c17disableIntP0x(2);

	/* <<< Port demonstration finish >>> */

	return EXIT_SUCCESS;
}
#endif

