/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief	This is RTCLP sample program for S1C17F63 demonstration.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17F63 Real Time Clock (RTCLP)
 *
 * @section     intro Introduction
 * Sample code for generating interruption at intervals of 1sec after setting time to RTCLP,
 * and execute theoretical regulation.
 * @subsection  step1   Step 1: Start OSC1 and Stop IOSC.
 * @subsection  step2   Step 2: Start RTCLP(1-second interrupt and theoretical regulation).
 * @subsection  step3   Step 3: Wait RTCLP 1sec Interrupt.
 *                              It waits until 1sec interrupt occurs 10 times.
 * @subsection  step4   Step 4: The set value at time is acquired.
 *                              End sample program(halt).
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17f63_sample" project.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "c17_init_config.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_rtclp_f63.h"


#ifdef C17_PE_RTCLP
extern volatile unsigned int c17rtclpIntCount;
extern volatile unsigned int c17swIntCount;

/* --- definition --- */

ST_RTCLP_BCD rtcBcd;

int main_rtclp(void);


/****************************************************************************
 * main function.
 *
 * @brief
 *          Main function starts from real-time clock sample.
 *          First,RTCLP register is initialized.
 *          Second,A setRtclp function is called,
 *          RTCLP information is set up and RTCLP is started.
 *          Next,It waits until 1sec interrupt occurs 10 times.
 *          then,RTCLP is stopped.
 *
 *          Latter half of main function is stopwatch sample.
 *          Firstly, initializes stopwatch.
 *          Secondarily, starts stopwatch and waits till 10 interrupts of 1Hz counts.
 *          Then, stops stopwatch.
 *
 * @retval  EXIT_SUCCESS     Success.
 ***************************************************************************/
int main_rtclp(void)
{

	/* <<< Real Time Clock demonstration start >>> */

	/// Initialize and start OSC1.
	c17startClgOSC1();								// start OSC1.
	c17switchClgSystemClockOSC1();
	c17stopClgOSC3();								// stop OSC3
	c17stopClgIOSC();								// stop IOSC.

	/// Continue operation state before SLEEP
	c17setOsc1OpeInSleep(C17_CLG_CONTINUE);

	/// Initialize RTCLP.
	c17initRtclp();									// initialize RTCLP.

	/// Set RTCLP date.
	rtcBcd.year = 0x16;								// Year = 2016
	rtcBcd.month = 0x12;							// Month  = 12.
	rtcBcd.day = 0x31;								// Day    = 31.
	rtcBcd.hour = 0x23;								// Hour   = 23.
	rtcBcd.minute = 0x59;							// Minute = 59.
	rtcBcd.second = 0x51;							// Second = 51.
	rtcBcd.week = C17_RTCLP_SAT;
	rtcBcd.ampm = C17_RTCLP_PM;
	c17setRtclpDate(&rtcBcd);

	/// Enable Interrupt
	c17enableIntRtclp();

	/// Start RTCLP
	c17startRtclp();

	/// Check RTCLP interrupt count.
	while(c17rtclpIntCount < 10){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ( "halt" );
		asm ( "nop" );
	}

	/// Disable Interrupt
	c17disableIntRtclp();

	/// Get RTCLP date.
	c17getRtclpDate(&rtcBcd);

	c17stopRtclp();


	/// Initialize stop watch.
	c17enableIntSW();
	c17startSw();

	while(c17swIntCount <10){
		// Set halt mode.
		asm ( "halt" );
		asm ( "nop" );
	}

	c17stopSw();
	c17disableIntSW();

	return 0;
}
#endif

