/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is TSRVR sample program for S1C17F63 demonstration.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Jun. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @main page   S1C17F63 Temperature Sensor/Reference Voltage Generator (TSRVR)
 *
 * @section     intro Introduction
 * It is a sample code for Temperature Sensor/Reference Voltage Generator (TSRVR).
 *
 * @subsection  step0    Get the temperature data. (ADC12A A/D conversion)
 *                       -Trigger:t16 ch.2 timer underflow(10times)
 *                       -Conversion mode: one-time
 * @subsection  step1    Convert the digital values corresponding to the respective temperatures,
 *                       that are obtained by the internal A/D converter,
 *                       into voltage values and assign them to VTSEN and VTREF.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17f63_sample" project.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "c17_init_config.h"
#include "c17_misc.h"
#include "c17_tsrvr.h"
#include "c17_clg.h"
#include "c17_adc12a.h"
#include "c17_t16.h"

#ifdef C17_PE_TSRVR
/* --- global variable --- */
extern volatile unsigned char c17adcIntCount;
extern volatile unsigned int c17t16Ch0IntCount;
extern unsigned short adVal[];

/* --- definition --- */
/**
 * @todo change VREFA[V]
 */
#define VREFA	3.3

/* --- function declaration --- */

int main_tsrvr(void);


/****************************************************************************
 * main function.
 *
 * @brief   Halt until ADC12A interrupt.
 *
 * @return  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_tsrvr(void)
{
	/* <<< Temperature Sensor/Reference Voltage Generator demonstration start >>> */

	int i;
	/// This variable is for debugging.
	/// Warning comes out, but there is no problem.
	volatile double Vt[10];

	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();

	/**
	 * Clock Source=IOSC(700KHz)
	 * Clock Division = 1/32
	 * Ft = 1sec
	 * TR = 1[sec] * f(700KHz/32) -1 = 21874
	 */
	/// Initialize T16 CH.0,CH.2 timer.
	c17setT16(0, C17_T16_CLKSRC_IOSC, C17_T16_CLKDIV_32, C17_T16_MODE_REPEAT);
	c17setT16(2, C17_T16_CLKSRC_IOSC, C17_T16_CLKDIV_32, C17_T16_MODE_REPEAT);
	/// Initialize the interrupt.
	ILVT16_0 = INTERRUPT_LEVEL_T16_0;				/// - Set interrupt level.

	/// Set Reload Value T16 CH.0
	c17setCountT16(0, 21874);
	/// Set Reload Value T16 CH.2
	c17setCountT16(2, 21874);

	/// T16 CH.0 Interrupt Enable
	c17enableIntT16(0);

	/// Clear Interruption counter.
	c17t16Ch0IntCount = 0;

	///Initialize TSRVR (set the reference voltage generator output voltage)
    /**
     * @todo change VREFAMD
     */
	c17setVrefMode(C17_TSRVR_VREF_VDD);

	///Enable temperature measurement
	c17enableTempMeas();

	/// Start T16 CH.0 timer.
	c17startT16(0);

	/// tTEMP wait (>200us) & tVREFA wait (>200us)
	while(c17t16Ch0IntCount < 1){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm("halt");
		asm("nop");
	}

	///Initialize ADC12A.(T16 Trigger)
	c17initAdc();
	c17setAdc12Ch0(C17_ADC12A_MODE_ONETIME, C17_ADC12A_TRG_T16, C17_ADC12A_START7, C17_ADC12A_END7);

	/// Clear Interruption counter.
	c17adcIntCount = 0;

	/// Start T16 CH.2 timer.
	c17startT16(2);

	/// ADC12A Interrupt Enable
	c17enableIntAdc();

	///Start ADC12A.
	c17startAdc();

	///Halt.  Interrupt every 1sec, start ADC convert.
	while(c17adcIntCount < 10){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

	}

	// Convert the digital values corresponding to the respective temperatures,
	// that are obtained by the internal A/D converter,
	// into voltage values and assign them to VTSEN and VTREF.
	for(i=0; i<10; i++){

		Vt[i] = ((double)adVal[i] / 4096) * VREFA;

	}

	///Stop ADC12A.
	c17stopAdc();

	///ADC12A Interrupt Disable
	c17disableIntAdc();

	/// Stop T16 CH.0 timer.
	c17stopT16(0);
	/// Stop T16 CH.2 timer.
	c17stopT16(2);

	/// Disable T16 CH.0 timer.
	c17disableT16(0);
	/// Disable T16 CH.2 timer.
	c17disableT16(2);

	///Disable temperature measurement
	c17disableTempMeas();

	/* <<< Temperature Sensor/Reference Voltage Generator demonstration finish >>> */

	return EXIT_SUCCESS;
}
#endif

